@@ -1,9 +1,10 @@
 [
   inputs: [
     "lib/*/{lib,unicode,test}/**/*.{ex,exs}",
-    "lib/*/mix.exs"
+    "lib/*/*.exs",
+    "lib/ex_unit/examples/*.exs",
+    ".formatter.exs"
   ],
-
   locals_without_parens: [
     # Formatter tests
     assert_format: 2,
