@@ -13,41 +13,67 @@ defmodule EEx.Compiler do
   def compile(source, opts) when is_binary(source) and is_list(opts) do
     file = opts[:file] || "nofile"
     line = opts[:line] || 1
+    column = 1
+    indentation = opts[:indentation] || 0
     trim = opts[:trim] || false
+    tokenizer_options = %{trim: trim, indentation: indentation}
 
-    case EEx.Tokenizer.tokenize(source, line, trim: trim) do
+    case EEx.Tokenizer.tokenize(source, line, column, tokenizer_options) do
       {:ok, tokens} ->
         state = %{
           engine: opts[:engine] || @default_engine,
           file: file,
           line: line,
           quoted: [],
-          start_line: nil
+          start_line: nil,
+          start_column: nil,
+          parser_options: Code.get_compiler_option(:parser_options)
         }
 
         init = state.engine.init(opts)
         generate_buffer(tokens, init, [], state)
 
-      {:error, line, message} ->
-        raise EEx.SyntaxError, line: line, file: file, message: message
+      {:error, line, column, message} ->
+        raise EEx.SyntaxError, file: file, line: line, column: column, message: message
     end
   end
 
   # Generates the buffers by handling each expression from the tokenizer.
   # It returns Macro.t/0 or it raises.
 
-  defp generate_buffer([{:text, chars} | rest], buffer, scope, state) do
-    buffer = state.engine.handle_text(buffer, IO.chardata_to_string(chars))
+  defp generate_buffer([{:text, line, column, chars} | rest], buffer, scope, state) do
+    buffer =
+      if function_exported?(state.engine, :handle_text, 3) do
+        meta = [line: line, column: column]
+        state.engine.handle_text(buffer, meta, IO.chardata_to_string(chars))
+      else
+        # TODO: Remove this branch on Elixir v2.0
+        state.engine.handle_text(buffer, IO.chardata_to_string(chars))
+      end
+
     generate_buffer(rest, buffer, scope, state)
   end
 
-  defp generate_buffer([{:expr, line, mark, chars, _} | rest], buffer, scope, state) do
-    expr = Code.string_to_quoted!(chars, line: line, file: state.file)
+  defp generate_buffer([{:expr, line, column, mark, chars} | rest], buffer, scope, state) do
+    options = [file: state.file, line: line, column: column(column, mark)] ++ state.parser_options
+    expr = Code.string_to_quoted!(chars, options)
     buffer = state.engine.handle_expr(buffer, IO.chardata_to_string(mark), expr)
     generate_buffer(rest, buffer, scope, state)
   end
 
-  defp generate_buffer([{:start_expr, start_line, mark, chars, _} | rest], buffer, scope, state) do
+  defp generate_buffer(
+         [{:start_expr, start_line, start_column, mark, chars} | rest],
+         buffer,
+         scope,
+         state
+       ) do
+    if mark != '=' do
+      message =
+        "the contents of this expression won't be output unless the EEx block starts with \"<%=\""
+
+      :elixir_errors.erl_warn(start_line, state.file, message)
+    end
+
     {contents, line, rest} = look_ahead_middle(rest, start_line, chars)
 
     {contents, rest} =
@@ -55,7 +81,13 @@ defmodule EEx.Compiler do
         rest,
         state.engine.handle_begin(buffer),
         [contents | scope],
-        %{state | quoted: [], line: line, start_line: start_line}
+        %{
+          state
+          | quoted: [],
+            line: line,
+            start_line: start_line,
+            start_column: column(start_column, mark)
+        }
       )
 
     buffer = state.engine.handle_expr(buffer, IO.chardata_to_string(mark), contents)
@@ -63,7 +95,7 @@ defmodule EEx.Compiler do
   end
 
   defp generate_buffer(
-         [{:middle_expr, line, '', chars, _} | rest],
+         [{:middle_expr, line, _column, '', chars} | rest],
          buffer,
          [current | scope],
          state
@@ -74,7 +106,7 @@ defmodule EEx.Compiler do
   end
 
   defp generate_buffer(
-         [{:middle_expr, line, modifier, chars, trimmed?} | t],
+         [{:middle_expr, line, column, modifier, chars} | t],
          buffer,
          [_ | _] = scope,
          state
@@ -84,27 +116,35 @@ defmodule EEx.Compiler do
         "please remove \"#{modifier}\" accordingly"
 
     :elixir_errors.erl_warn(line, state.file, message)
-    generate_buffer([{:middle_expr, line, '', chars, trimmed?} | t], buffer, scope, state)
+    generate_buffer([{:middle_expr, line, column, '', chars} | t], buffer, scope, state)
     # TODO: Make this an error on Elixir v2.0 since it accidentally worked previously.
     # raise EEx.SyntaxError, message: message, file: state.file, line: line
   end
 
-  defp generate_buffer([{:middle_expr, line, _, chars, _} | _], _buffer, [], state) do
+  defp generate_buffer([{:middle_expr, line, column, _, chars} | _], _buffer, [], state) do
     raise EEx.SyntaxError,
       message: "unexpected middle of expression <%#{chars}%>",
       file: state.file,
-      line: line
+      line: line,
+      column: column
   end
 
-  defp generate_buffer([{:end_expr, line, '', chars, _} | rest], buffer, [current | _], state) do
+  defp generate_buffer(
+         [{:end_expr, line, _column, '', chars} | rest],
+         buffer,
+         [current | _],
+         state
+       ) do
     {wrapped, state} = wrap_expr(current, line, buffer, chars, state)
-    tuples = Code.string_to_quoted!(wrapped, line: state.start_line, file: state.file)
+    column = state.start_column
+    options = [file: state.file, line: state.start_line, column: column] ++ state.parser_options
+    tuples = Code.string_to_quoted!(wrapped, options)
     buffer = insert_quoted(tuples, state.quoted)
     {buffer, rest}
   end
 
   defp generate_buffer(
-         [{:end_expr, line, modifier, chars, trimmed?} | t],
+         [{:end_expr, line, column, modifier, chars} | t],
          buffer,
          [_ | _] = scope,
          state
@@ -114,27 +154,29 @@ defmodule EEx.Compiler do
         "expression \"<%#{modifier}#{chars}%>\", please remove \"#{modifier}\" accordingly"
 
     :elixir_errors.erl_warn(line, state.file, message)
-    generate_buffer([{:end_expr, line, '', chars, trimmed?} | t], buffer, scope, state)
+    generate_buffer([{:end_expr, line, column, '', chars} | t], buffer, scope, state)
     # TODO: Make this an error on Elixir v2.0 since it accidentally worked previously.
-    # raise EEx.SyntaxError, message: message, file: state.file, line: line
+    # raise EEx.SyntaxError, message: message, file: state.file, line: line, column: column
   end
 
-  defp generate_buffer([{:end_expr, line, _, chars, _} | _], _buffer, [], state) do
+  defp generate_buffer([{:end_expr, line, column, _, chars} | _], _buffer, [], state) do
     raise EEx.SyntaxError,
       message: "unexpected end of expression <%#{chars}%>",
       file: state.file,
-      line: line
+      line: line,
+      column: column
   end
 
-  defp generate_buffer([], buffer, [], state) do
+  defp generate_buffer([{:eof, _, _}], buffer, [], state) do
     state.engine.handle_body(buffer)
   end
 
-  defp generate_buffer([], _buffer, _scope, state) do
+  defp generate_buffer([{:eof, line, column}], _buffer, _scope, state) do
     raise EEx.SyntaxError,
       message: "unexpected end of string, expected a closing '<% end %>'",
       file: state.file,
-      line: state.line
+      line: line,
+      column: column
   end
 
   # Creates a placeholder and wrap it inside the expression block
@@ -152,7 +194,7 @@ defmodule EEx.Compiler do
   # Look middle expressions that immediately follow a start_expr
 
   defp look_ahead_middle(
-         [{:text, text}, {:middle_expr, line, _, chars, _} | rest] = tokens,
+         [{:text, _, _, text}, {:middle_expr, line, _, _, chars} | rest] = tokens,
          start,
          contents
        ) do
@@ -163,7 +205,7 @@ defmodule EEx.Compiler do
     end
   end
 
-  defp look_ahead_middle([{:middle_expr, line, _, chars, _} | rest], _start, contents) do
+  defp look_ahead_middle([{:middle_expr, line, _column, _, chars} | rest], _start, contents) do
     {contents ++ chars, line, rest}
   end
 
@@ -197,4 +239,9 @@ defmodule EEx.Compiler do
   defp insert_quoted(other, _quoted) do
     other
   end
+
+  defp column(column, mark) do
+    # length('<%') == 2
+    column + 2 + length(mark)
+  end
 end
