@@ -6,8 +6,8 @@ defmodule Access do
   keys of any type in a data structure via the `data[key]` syntax.
 
   `Access` supports keyword lists (`Keyword`) and maps (`Map`) out
-  of the box. The key can be of any type and it returns `nil` if
-  the key does not exist:
+  of the box. Keywords supports only atoms keys, keys for maps can
+  be of any type. Both return `nil` if the key does not exist:
 
       iex> keywords = [a: 1, b: 2]
       iex> keywords[:a]
@@ -47,12 +47,11 @@ defmodule Access do
   > `map[key]`, if your map is made of predefined atom keys,
   > you should prefer to access those atom keys with `map.key`
   > instead of `map[key]`, as `map.key` will raise if the key
-  > is missing. This is important because, if a map has a predefined
-  > set of keys and a key is missing, it is most likely a bug
-  > in your software or a typo on the key name. For this reason,
-  > because structs are predefined in nature, they only allow
-  > the `struct.key` syntax and they do not allow the `struct[key]`
-  > access syntax. See the `Map` module for more information.
+  > is missing (which is not supposed to happen if the keys are
+  > predefined). Similarly, since structs are maps and structs
+  > have predefined keys, they only allow the `struct.key`
+  > syntax and they do not allow the `struct[key]` access syntax.
+  > See the `Map` module for more information.
 
   ## Nested data structures
 
@@ -100,16 +99,15 @@ defmodule Access do
   @type key :: any
   @type value :: any
 
-  @type get_fun(data, get_value) ::
-          (:get, data, (term -> term) ->
-             {get_value, new_data :: container})
+  @type get_fun(data) ::
+          (:get, data, (term -> term) -> new_data :: container)
 
-  @type get_and_update_fun(data, get_value) ::
+  @type get_and_update_fun(data, current_value) ::
           (:get_and_update, data, (term -> term) ->
-             {get_value, new_data :: container} | :pop)
+             {current_value, new_data :: container} | :pop)
 
-  @type access_fun(data, get_value) ::
-          get_fun(data, get_value) | get_and_update_fun(data, get_value)
+  @type access_fun(data, current_value) ::
+          get_fun(data) | get_and_update_fun(data, current_value)
 
   @doc """
   Invoked in order to access the value stored under `key` in the given term `term`.
@@ -135,16 +133,16 @@ defmodule Access do
 
   The implementation of this callback should invoke `fun` with the value under
   `key` in the passed structure `data`, or with `nil` if `key` is not present in it.
-  This function must return either `{get_value, update_value}` or `:pop`.
+  This function must return either `{current_value, new_value}` or `:pop`.
 
-  If the passed function returns `{get_value, update_value}`,
-  the return value of this callback should be `{get_value, new_data}`, where:
+  If the passed function returns `{current_value, new_value}`,
+  the return value of this callback should be `{current_value, new_data}`, where:
 
-    * `get_value` is the retrieved value (which can be operated on before being returned)
+    * `current_value` is the retrieved value (which can be operated on before being returned)
 
-    * `update_value` is the new value to be stored under `key`
+    * `new_value` is the new value to be stored under `key`
 
-    * `new_data` is `data` after updating the value of `key` with `update_value`.
+    * `new_data` is `data` after updating the value of `key` with `new_value`.
 
   If the passed function returns `:pop`, the return value of this callback
   must be `{value, new_data}` where `value` is the value under `key`
@@ -153,8 +151,9 @@ defmodule Access do
   See the implementations of `Map.get_and_update/3` or `Keyword.get_and_update/3`
   for more examples.
   """
-  @callback get_and_update(data, key, (value -> {get_value, value} | :pop)) :: {get_value, data}
-            when get_value: var, data: container | any_container
+  @callback get_and_update(data, key, (value | nil -> {current_value, new_value :: value} | :pop)) ::
+              {current_value, new_data :: data}
+            when current_value: value, data: container | any_container
 
   @doc """
   Invoked to "pop" the value under `key` out of the given data structure.
@@ -321,9 +320,9 @@ defmodule Access do
   a struct that implements the `Access` behaviour).
 
   The `fun` argument receives the value of `key` (or `nil` if `key` is not
-  present in `container`) and must return a two-element tuple `{get_value, update_value}`:
-  the "get" value `get_value` (the retrieved value, which can be operated on before
-  being returned) and the new value to be stored under `key` (`update_value`).
+  present in `container`) and must return a two-element tuple `{current_value, new_value}`:
+  the "get" value `current_value` (the retrieved value, which can be operated on before
+  being returned) and the new value to be stored under `key` (`new_value`).
   `fun` may also return `:pop`, which means the current value
   should be removed from the container and returned.
 
@@ -338,8 +337,9 @@ defmodule Access do
       {1, [a: 2]}
 
   """
-  @spec get_and_update(data, key, (value -> {get_value, value} | :pop)) :: {get_value, data}
-        when get_value: var, data: container
+  @spec get_and_update(data, key, (value | nil -> {current_value, new_value :: value} | :pop)) ::
+          {current_value, new_data :: data}
+        when current_value: var, data: container
   def get_and_update(container, key, fun)
 
   def get_and_update(%module{} = container, key, fun) do
@@ -422,7 +422,7 @@ defmodule Access do
   The returned function uses the default value if the key does not exist.
   This can be used to specify defaults and safely traverse missing keys:
 
-      iex> get_in(%{}, [Access.key(:user, %{name: "meg"}), Access.key(:name)])
+      iex> get_in(%{}, [Access.key(:user, %{}), Access.key(:name, "meg")])
       "meg"
 
   Such is also useful when using update functions, allowing us to introduce
@@ -452,7 +452,7 @@ defmodule Access do
       ** (BadMapError) expected a map, got: []
 
   """
-  @spec key(key, term) :: access_fun(data :: struct | map, get_value :: term)
+  @spec key(key, term) :: access_fun(data :: struct | map, current_value :: term)
   def key(key, default \\ nil) do
     fn
       :get, data, next ->
@@ -496,7 +496,7 @@ defmodule Access do
       ** (RuntimeError) Access.key!/1 expected a map/struct, got: []
 
   """
-  @spec key!(key) :: access_fun(data :: struct | map, get_value :: term)
+  @spec key!(key) :: access_fun(data :: struct | map, current_value :: term)
   def key!(key) do
     fn
       :get, %{} = data, next ->
@@ -544,7 +544,7 @@ defmodule Access do
       ** (RuntimeError) Access.elem/1 expected a tuple, got: %{}
 
   """
-  @spec elem(non_neg_integer) :: access_fun(data :: tuple, get_value :: term)
+  @spec elem(non_neg_integer) :: access_fun(data :: tuple, current_value :: term)
   def elem(index) when is_integer(index) and index >= 0 do
     pos = index + 1
 
@@ -598,7 +598,7 @@ defmodule Access do
       ** (RuntimeError) Access.all/0 expected a list, got: %{}
 
   """
-  @spec all() :: access_fun(data :: list, get_value :: list)
+  @spec all() :: access_fun(data :: list, current_value :: list)
   def all() do
     &all/3
   end
@@ -673,7 +673,7 @@ defmodule Access do
       ** (RuntimeError) Access.at/1 expected a list, got: %{}
 
   """
-  @spec at(integer) :: access_fun(data :: list, get_value :: term)
+  @spec at(integer) :: access_fun(data :: list, current_value :: term)
   def at(index) when is_integer(index) do
     fn op, data, next -> at(op, data, index, next) end
   end
@@ -683,36 +683,69 @@ defmodule Access do
   end
 
   defp at(:get_and_update, data, index, next) when is_list(data) do
-    get_and_update_at(data, index, next, [])
+    get_and_update_at(data, index, next, [], fn -> nil end)
   end
 
   defp at(_op, data, _index, _next) do
     raise "Access.at/1 expected a list, got: #{inspect(data)}"
   end
 
-  defp get_and_update_at([head | rest], 0, next, updates) do
+  defp get_and_update_at([head | rest], 0, next, updates, _default_fun) do
     case next.(head) do
       {get, update} -> {get, :lists.reverse([update | updates], rest)}
       :pop -> {head, :lists.reverse(updates, rest)}
     end
   end
 
-  defp get_and_update_at(list, index, next, updates) when index < 0 do
+  defp get_and_update_at([_ | _] = list, index, next, updates, default_fun) when index < 0 do
     list_length = length(list)
 
     if list_length + index >= 0 do
-      get_and_update_at(list, list_length + index, next, updates)
+      get_and_update_at(list, list_length + index, next, updates, default_fun)
     else
-      {nil, list}
+      {default_fun.(), list}
     end
   end
 
-  defp get_and_update_at([head | rest], index, next, updates) when index > 0 do
-    get_and_update_at(rest, index - 1, next, [head | updates])
+  defp get_and_update_at([head | rest], index, next, updates, default_fun) when index > 0 do
+    get_and_update_at(rest, index - 1, next, [head | updates], default_fun)
   end
 
-  defp get_and_update_at([], _index, _next, updates) do
-    {nil, :lists.reverse(updates)}
+  defp get_and_update_at([], _index, _next, updates, default_fun) do
+    {default_fun.(), :lists.reverse(updates)}
+  end
+
+  @doc ~S"""
+  Same as `at/1` except that it raises `Enum.OutOfBoundsError`
+  if the given index is out of bounds.
+
+  ## Examples
+
+      iex> get_in([:a, :b, :c], [Access.at!(2)])
+      :c
+      iex> get_in([:a, :b, :c], [Access.at!(3)])
+      ** (Enum.OutOfBoundsError) out of bounds error
+
+  """
+  @doc since: "1.11.0"
+  @spec at!(integer) :: access_fun(data :: list, current_value :: term)
+  def at!(index) when is_integer(index) do
+    fn op, data, next -> at!(op, data, index, next) end
+  end
+
+  defp at!(:get, data, index, next) when is_list(data) do
+    case Enum.fetch(data, index) do
+      {:ok, value} -> next.(value)
+      :error -> raise Enum.OutOfBoundsError
+    end
+  end
+
+  defp at!(:get_and_update, data, index, next) when is_list(data) do
+    get_and_update_at(data, index, next, [], fn -> raise Enum.OutOfBoundsError end)
+  end
+
+  defp at!(_op, data, _index, _next) do
+    raise "Access.at!/1 expected a list, got: #{inspect(data)}"
   end
 
   @doc ~S"""
@@ -762,7 +795,7 @@ defmodule Access do
 
   """
   @doc since: "1.6.0"
-  @spec filter((term -> boolean)) :: access_fun(data :: list, get_value :: list)
+  @spec filter((term -> boolean)) :: access_fun(data :: list, current_value :: list)
   def filter(func) when is_function(func) do
     fn op, data, next -> filter(op, data, func, next) end
   end
