@@ -83,28 +83,70 @@ defmodule Date do
       366
       iex> Enum.member?(range, ~D[2001-02-01])
       true
-      iex> Enum.reduce(range, 0, fn _date, acc -> acc - 1 end)
-      -366
+      iex> Enum.take(range, 3)
+      [~D[2001-01-01], ~D[2001-01-02], ~D[2001-01-03]]
 
   """
   @doc since: "1.5.0"
-  @spec range(Date.t(), Date.t()) :: Date.Range.t()
-  def range(%Date{calendar: calendar} = first, %Date{calendar: calendar} = last) do
+  @spec range(Calendar.date(), Calendar.date()) :: Date.Range.t()
+  def range(%{calendar: calendar} = first, %{calendar: calendar} = last) do
     {first_days, _} = to_iso_days(first)
     {last_days, _} = to_iso_days(last)
-
-    %Date.Range{
-      first: first,
-      last: last,
-      first_in_iso_days: first_days,
-      last_in_iso_days: last_days
-    }
+    # TODO: Deprecate inferring a range with a step of -1 on Elixir v1.16
+    step = if first_days <= last_days, do: 1, else: -1
+    range(first, first_days, last, last_days, calendar, step)
   end
 
-  def range(%Date{}, %Date{}) do
+  def range(%{calendar: _, year: _, month: _, day: _}, %{calendar: _, year: _, month: _, day: _}) do
     raise ArgumentError, "both dates must have matching calendars"
   end
 
+  @doc """
+  Returns a range of dates with a step.
+
+  ## Examples
+
+      iex> range = Date.range(~D[2001-01-01], ~D[2002-01-01], 2)
+      iex> range
+      #DateRange<~D[2001-01-01], ~D[2002-01-01], 2>
+      iex> Enum.count(range)
+      183
+      iex> Enum.member?(range, ~D[2001-01-03])
+      true
+      iex> Enum.take(range, 3)
+      [~D[2001-01-01], ~D[2001-01-03], ~D[2001-01-05]]
+
+  """
+  @doc since: "1.12.0"
+  @spec range(Calendar.date(), Calendar.date(), step :: pos_integer | neg_integer) ::
+          Date.Range.t()
+  def range(%{calendar: calendar} = first, %{calendar: calendar} = last, step)
+      when is_integer(step) and step != 0 do
+    {first_days, _} = to_iso_days(first)
+    {last_days, _} = to_iso_days(last)
+    range(first, first_days, last, last_days, calendar, step)
+  end
+
+  def range(
+        %{calendar: _, year: _, month: _, day: _} = first,
+        %{calendar: _, year: _, month: _, day: _} = last,
+        step
+      ) do
+    raise ArgumentError,
+          "both dates must have matching calendar and the step must be a " <>
+            "non-zero integer, got: #{inspect(first)}, #{inspect(last)}, #{step}"
+  end
+
+  defp range(first, first_days, last, last_days, calendar, step) do
+    %Date.Range{
+      first: %Date{calendar: calendar, year: first.year, month: first.month, day: first.day},
+      last: %Date{calendar: calendar, year: last.year, month: last.month, day: last.day},
+      first_in_iso_days: first_days,
+      last_in_iso_days: last_days,
+      step: step
+    }
+  end
+
   @doc """
   Returns the current date in UTC.
 
@@ -224,6 +266,33 @@ defmodule Date do
     end
   end
 
+  @doc """
+  Builds a new ISO date.
+
+  Expects all values to be integers. Returns `date` if each
+  entry fits its appropriate range, raises if the date is invalid.
+
+  ## Examples
+
+      iex> Date.new!(2000, 1, 1)
+      ~D[2000-01-01]
+      iex> Date.new!(2000, 13, 1)
+      ** (ArgumentError) cannot build date, reason: :invalid_date
+      iex> Date.new!(2000, 2, 29)
+      ~D[2000-02-29]
+  """
+  @doc since: "1.11.0"
+  @spec new!(Calendar.year(), Calendar.month(), Calendar.day(), Calendar.calendar()) :: t
+  def new!(year, month, day, calendar \\ Calendar.ISO) do
+    case new(year, month, day, calendar) do
+      {:ok, value} ->
+        value
+
+      {:error, reason} ->
+        raise ArgumentError, "cannot build date, reason: #{inspect(reason)}"
+    end
+  end
+
   @doc """
   Converts the given date to a string according to its calendar.
 
@@ -246,7 +315,7 @@ defmodule Date do
 
   @doc """
   Parses the extended "Dates" format described by
-  [ISO 8601:2004](https://en.wikipedia.org/wiki/ISO_8601).
+  [ISO 8601:2019](https://en.wikipedia.org/wiki/ISO_8601).
 
   The year parsed by this function is limited to four digits.
 
@@ -271,7 +340,7 @@ defmodule Date do
 
   @doc """
   Parses the extended "Dates" format described by
-  [ISO 8601:2004](https://en.wikipedia.org/wiki/ISO_8601).
+  [ISO 8601:2019](https://en.wikipedia.org/wiki/ISO_8601).
 
   Raises if the format is invalid.
 
@@ -296,7 +365,7 @@ defmodule Date do
 
   @doc """
   Converts the given `date` to
-  [ISO 8601:2004](https://en.wikipedia.org/wiki/ISO_8601).
+  [ISO 8601:2019](https://en.wikipedia.org/wiki/ISO_8601).
 
   By default, `Date.to_iso8601/2` returns dates formatted in the "extended"
   format, for human readability. It also supports the "basic" format through passing the `:basic` option.
@@ -398,6 +467,45 @@ defmodule Date do
     end
   end
 
+  @doc """
+  Converts a number of gregorian days to a `Date` struct.
+
+  ## Examples
+
+      iex> Date.from_gregorian_days(1)
+      ~D[0000-01-02]
+      iex> Date.from_gregorian_days(730_485)
+      ~D[2000-01-01]
+      iex> Date.from_gregorian_days(-1)
+      ~D[-0001-12-31]
+
+  """
+  @doc since: "1.11.0"
+  @spec from_gregorian_days(integer(), Calendar.calendar()) :: t
+  def from_gregorian_days(days, calendar \\ Calendar.ISO) when is_integer(days) do
+    from_iso_days({days, 0}, calendar)
+  end
+
+  @doc """
+  Converts a `date` struct to a number of gregorian days.
+
+  ## Examples
+
+      iex> Date.to_gregorian_days(~D[0000-01-02])
+      1
+      iex> Date.to_gregorian_days(~D[2000-01-01])
+      730_485
+      iex> Date.to_gregorian_days(~N[2000-01-01 00:00:00])
+      730_485
+
+  """
+  @doc since: "1.11.0"
+  @spec to_gregorian_days(Calendar.date()) :: integer()
+  def to_gregorian_days(date) do
+    {days, _} = to_iso_days(date)
+    days
+  end
+
   @doc """
   Compares two date structs.
 
@@ -434,8 +542,8 @@ defmodule Date do
     end
   end
 
-  def compare(date1, date2) do
-    if Calendar.compatible_calendars?(date1.calendar, date2.calendar) do
+  def compare(%{calendar: calendar1} = date1, %{calendar: calendar2} = date2) do
+    if Calendar.compatible_calendars?(calendar1, calendar2) do
       case {to_iso_days(date1), to_iso_days(date2)} do
         {first, second} when first > second -> :gt
         {first, second} when first < second -> :lt
@@ -593,11 +701,12 @@ defmodule Date do
     end
   end
 
-  defp to_iso_days(%{calendar: Calendar.ISO, year: year, month: month, day: day}) do
+  @doc false
+  def to_iso_days(%{calendar: Calendar.ISO, year: year, month: month, day: day}) do
     {Calendar.ISO.date_to_iso_days(year, month, day), {0, 86_400_000_000}}
   end
 
-  defp to_iso_days(%{calendar: calendar, year: year, month: month, day: day}) do
+  def to_iso_days(%{calendar: calendar, year: year, month: month, day: day}) do
     calendar.naive_datetime_to_iso_days(year, month, day, 0, 0, 0, {0, 0})
   end
 
@@ -618,6 +727,11 @@ defmodule Date do
   calendar (the default), it is an integer from 1 to 7, where
   1 is Monday and 7 is Sunday.
 
+  An optional `starting_on` value may be supplied, which
+  configures the weekday the week starts on. The default value
+  for it is `:default`, which translates to `:monday` for the
+  built-in ISO calendar. Any other weekday may be given to.
+
   ## Examples
 
       iex> Date.day_of_week(~D[2016-10-31])
@@ -629,13 +743,131 @@ defmodule Date do
       iex> Date.day_of_week(~D[-0015-10-30])
       3
 
+      iex> Date.day_of_week(~D[2016-10-31], :sunday)
+      2
+      iex> Date.day_of_week(~D[2016-11-01], :sunday)
+      3
+      iex> Date.day_of_week(~N[2016-11-01 01:23:45], :sunday)
+      3
+      iex> Date.day_of_week(~D[-0015-10-30], :sunday)
+      4
+
   """
   @doc since: "1.4.0"
-  @spec day_of_week(Calendar.date()) :: Calendar.day()
-  def day_of_week(date)
+  @spec day_of_week(Calendar.date(), starting_on :: :default | atom) :: Calendar.day_of_week()
+  def day_of_week(date, starting_on \\ :default)
 
-  def day_of_week(%{calendar: calendar, year: year, month: month, day: day}) do
-    calendar.day_of_week(year, month, day)
+  def day_of_week(%{calendar: calendar, year: year, month: month, day: day}, starting_on) do
+    {day_of_week, _first, _last} = calendar.day_of_week(year, month, day, starting_on)
+    day_of_week
+  end
+
+  @doc """
+  Calculates a date that is the first day of the week for the given `date`.
+
+  If the day is already the first day of the week, it returns the
+  day itself. For the built-in ISO calendar, the week starts on Monday.
+  A weekday rather than `:default` can be given as `starting_on`.
+
+  ## Examples
+
+      iex> Date.beginning_of_week(~D[2020-07-11])
+      ~D[2020-07-06]
+      iex> Date.beginning_of_week(~D[2020-07-06])
+      ~D[2020-07-06]
+      iex> Date.beginning_of_week(~D[2020-07-11], :sunday)
+      ~D[2020-07-05]
+      iex> Date.beginning_of_week(~D[2020-07-11], :saturday)
+      ~D[2020-07-11]
+      iex> Date.beginning_of_week(~N[2020-07-11 01:23:45])
+      ~D[2020-07-06]
+
+  """
+  @doc since: "1.11.0"
+  @spec beginning_of_week(Calendar.date(), starting_on :: :default | atom) :: Date.t()
+  def beginning_of_week(date, starting_on \\ :default)
+
+  def beginning_of_week(%{calendar: Calendar.ISO} = date, starting_on) do
+    %{year: year, month: month, day: day} = date
+    iso_days = Calendar.ISO.date_to_iso_days(year, month, day)
+
+    {year, month, day} =
+      case Calendar.ISO.iso_days_to_day_of_week(iso_days, starting_on) do
+        1 ->
+          {year, month, day}
+
+        day_of_week ->
+          Calendar.ISO.date_from_iso_days(iso_days - day_of_week + 1)
+      end
+
+    %Date{calendar: Calendar.ISO, year: year, month: month, day: day}
+  end
+
+  def beginning_of_week(%{calendar: calendar} = date, starting_on) do
+    %{year: year, month: month, day: day} = date
+
+    case calendar.day_of_week(year, month, day, starting_on) do
+      {day_of_week, day_of_week, _} ->
+        %Date{calendar: calendar, year: year, month: month, day: day}
+
+      {day_of_week, first_day_of_week, _} ->
+        add(date, -(day_of_week - first_day_of_week))
+    end
+  end
+
+  @doc """
+  Calculates a date that is the last day of the week for the given `date`.
+
+  If the day is already the last day of the week, it returns the
+  day itself. For the built-in ISO calendar, the week ends on Sunday.
+  A weekday rather than `:default` can be given as `starting_on`.
+
+  ## Examples
+
+      iex> Date.end_of_week(~D[2020-07-11])
+      ~D[2020-07-12]
+      iex> Date.end_of_week(~D[2020-07-05])
+      ~D[2020-07-05]
+      iex> Date.end_of_week(~D[2020-07-06], :sunday)
+      ~D[2020-07-11]
+      iex> Date.end_of_week(~D[2020-07-06], :sunday)
+      ~D[2020-07-11]
+      iex> Date.end_of_week(~D[2020-07-06], :saturday)
+      ~D[2020-07-10]
+      iex> Date.end_of_week(~N[2020-07-11 01:23:45])
+      ~D[2020-07-12]
+
+  """
+  @doc since: "1.11.0"
+  @spec end_of_week(Calendar.date(), starting_on :: :default | atom) :: Date.t()
+  def end_of_week(date, starting_on \\ :default)
+
+  def end_of_week(%{calendar: Calendar.ISO} = date, starting_on) do
+    %{year: year, month: month, day: day} = date
+    iso_days = Calendar.ISO.date_to_iso_days(year, month, day)
+
+    {year, month, day} =
+      case Calendar.ISO.iso_days_to_day_of_week(iso_days, starting_on) do
+        7 ->
+          {year, month, day}
+
+        day_of_week ->
+          Calendar.ISO.date_from_iso_days(iso_days + 7 - day_of_week)
+      end
+
+    %Date{calendar: Calendar.ISO, year: year, month: month, day: day}
+  end
+
+  def end_of_week(%{calendar: calendar} = date, starting_on) do
+    %{year: year, month: month, day: day} = date
+
+    case calendar.day_of_week(year, month, day, starting_on) do
+      {day_of_week, _, day_of_week} ->
+        %Date{calendar: calendar, year: year, month: month, day: day}
+
+      {day_of_week, _, last_day_of_week} ->
+        add(date, last_day_of_week - day_of_week)
+    end
   end
 
   @doc """
@@ -739,6 +971,45 @@ defmodule Date do
     calendar.day_of_era(year, month, day)
   end
 
+  @doc """
+  Calculates a date that is the first day of the month for the given `date`.
+
+  ## Examples
+
+      iex> Date.beginning_of_month(~D[2000-01-31])
+      ~D[2000-01-01]
+      iex> Date.beginning_of_month(~D[2000-01-01])
+      ~D[2000-01-01]
+      iex> Date.beginning_of_month(~N[2000-01-31 01:23:45])
+      ~D[2000-01-01]
+
+  """
+  @doc since: "1.11.0"
+  @spec beginning_of_month(Calendar.date()) :: t()
+  def beginning_of_month(%{year: year, month: month, calendar: calendar}) do
+    %Date{year: year, month: month, day: 1, calendar: calendar}
+  end
+
+  @doc """
+  Calculates a date that is the last day of the month for the given `date`.
+
+  ## Examples
+
+      iex> Date.end_of_month(~D[2000-01-01])
+      ~D[2000-01-31]
+      iex> Date.end_of_month(~D[2000-01-31])
+      ~D[2000-01-31]
+      iex> Date.end_of_month(~N[2000-01-01 01:23:45])
+      ~D[2000-01-31]
+
+  """
+  @doc since: "1.11.0"
+  @spec end_of_month(Calendar.date()) :: t()
+  def end_of_month(%{year: year, month: month, calendar: calendar} = date) do
+    day = Date.days_in_month(date)
+    %Date{year: year, month: month, day: day, calendar: calendar}
+  end
+
   ## Helpers
 
   defimpl String.Chars do
