@@ -11,11 +11,45 @@ defmodule Config.Provider do
   the file system. For more information on runtime configuration,
   see `mix release`.
 
-  ## Sample config provider
+  ## Multiple config files
 
-  For example, imagine you need to load some configuration from
-  a JSON file and load that into the system. Said configuration
-  provider would look like:
+  One common use of config providers is to specify multiple
+  configuration files in a release. Elixir ships with one provider,
+  called `Config.Reader`, which is capable of handling Elixir's
+  built-in config files.
+
+  For example, imagine you want to list some basic configuration
+  on Mix's built-in `config/runtime.exs` file, but you also want
+  some additional configuration files. To do so, you can do this
+  in your `mix.exs`:
+
+      releases: [
+        demo: [
+          config_providers: [
+            {Config.Reader, {:system, "RELEASE_ROOT", "/extra_config.exs"}}
+          ]
+        ]
+      ]
+
+  You can place this `extra_config.exs` file in your release in
+  multiple ways:
+
+    1. If it is available on the host when assembling the release,
+      you can place it on "rel/overlays/extra_config.exs" and it
+      will be automatically copied to the release root
+
+    2. If it is available on the target during deployment, you can
+      simply copy it to the release root as a step in your deployment
+
+  Now once the system boots, it will load both `config/runtime.exs`
+  and `extra_config.exs` early in the boot process.
+
+  ## Custom config provider
+
+  You can also implement custom config providers, similar to how
+  `Config.Reader` works. For example, imagine you need to load
+  some configuration from a JSON file and load that into the system.
+  Said configuration provider would look like:
 
       defmodule JSONConfigProvider do
         @behaviour Config.Provider
@@ -39,19 +73,17 @@ defmodule Config.Provider do
         end
       end
 
-  Then when specifying your release, you can specify the provider in
+  Then, when specifying your release, you can specify the provider in
   the release configuration:
 
       releases: [
         demo: [
-          # ...,
-          config_providers: [{JSONConfigProvider, "/etc/config.json"}]
+          config_providers: [
+            {JSONConfigProvider, "/etc/config.json"}
+          ]
         ]
       ]
 
-  Now once the system boots, it will invoke the provider early in
-  the boot process, save the merged configuration to the disk, and
-  reboot the system with the new values in place.
   """
 
   @type config :: keyword
@@ -93,7 +125,7 @@ defmodule Config.Provider do
   Loads configuration (typically during system boot).
 
   It receives the current `config` and the `state` returned by
-  `c:init/1`. Then you typically read the extra configuration
+  `c:init/1`. Then, you typically read the extra configuration
   from an external source and merge it into the received `config`.
   Merging should be done with `Config.Reader.merge/2`, as it
   performs deep merge. It should return the updated config.
@@ -109,11 +141,13 @@ defmodule Config.Provider do
     :providers,
     :config_path,
     extra_config: [],
-    prune_after_boot: false,
-    reboot_after_config: true,
+    prune_runtime_sys_config_after_boot: false,
+    reboot_system_after_config: false,
     validate_compile_env: false
   ]
 
+  @reserved_apps [:kernel, :stdlib]
+
   @doc """
   Validates a `t:config_path/0`.
   """
@@ -147,44 +181,44 @@ defmodule Config.Provider do
   def resolve_config_path!(path) when is_binary(path), do: path
   def resolve_config_path!({:system, name, path}), do: System.fetch_env!(name) <> path
 
+  # Private keys
+  @init_key :config_provider_init
+  @booted_key :config_provider_booted
+
+  # Public keys
+  @reboot_mode_key :config_provider_reboot_mode
+
   @doc false
   def init(providers, config_path, opts \\ []) when is_list(providers) and is_list(opts) do
     validate_config_path!(config_path)
     providers = for {provider, init} <- providers, do: {provider, provider.init(init)}
-    struct!(%Config.Provider{config_path: config_path, providers: providers}, opts)
+    init = struct!(%Config.Provider{config_path: config_path, providers: providers}, opts)
+    [elixir: [{@init_key, init}]]
   end
 
   @doc false
-  def boot(app, key, restart_fun \\ &restart_and_sleep/0) do
-    # The app with the config provider settings may not
-    # have been loaded at this point, so make sure we load
-    # its environment before querying it.
-    _ = :application.load(app)
-
+  def boot(reboot_fun \\ &restart_and_sleep/0) do
     # The config provider typically runs very early in the
     # release process, so we need to make sure Elixir is started
     # before we go around running Elixir code.
     {:ok, _} = :application.ensure_all_started(:elixir)
 
-    # The key we store if the system already booted
-    booted_key = :"#{key}_booted"
-
-    case :application.get_env(app, booted_key) do
+    case Application.fetch_env(:elixir, @booted_key) do
       {:ok, {:booted, path}} ->
         path && File.rm(path)
 
-        with {:ok, %Config.Provider{} = provider} <- :application.get_env(app, key) do
+        with {:ok, %Config.Provider{} = provider} <- Application.fetch_env(:elixir, @init_key) do
           maybe_validate_compile_env(provider)
         end
 
         :booted
 
       _ ->
-        case :application.get_env(app, key) do
+        case Application.fetch_env(:elixir, @init_key) do
           {:ok, %Config.Provider{} = provider} ->
             path = resolve_config_path!(provider.config_path)
-            reboot_config = [{app, [{booted_key, booted_value(provider, path)}]}]
-            boot_providers(path, provider, reboot_config, restart_fun)
+            reboot_config = [elixir: [{@booted_key, booted_value(provider, path)}]]
+            boot_providers(path, provider, reboot_config, reboot_fun)
 
           _ ->
             :skip
@@ -192,9 +226,8 @@ defmodule Config.Provider do
     end
   end
 
-  defp boot_providers(path, provider, reboot_config, restart_fun) do
+  defp boot_providers(path, provider, reboot_config, reboot_fun) do
     validate_no_cyclic_boot!(path)
-    loaded_applications = :application.loaded_applications()
     original_config = read_config!(path)
 
     config =
@@ -202,16 +235,16 @@ defmodule Config.Provider do
       |> Config.__merge__(provider.extra_config)
       |> run_providers(provider)
 
-    if provider.reboot_after_config do
+    if provider.reboot_system_after_config do
       config
       |> Config.__merge__(reboot_config)
       |> write_config!(path)
 
-      restart_fun.()
+      reboot_fun.()
     else
-      for {app, _, _} <- loaded_applications, config[app] != original_config[app] do
+      for app <- @reserved_apps, config[app] != original_config[app] do
         abort("""
-        Cannot configure #{inspect(app)} because :reboot_after_config has been set \
+        Cannot configure #{inspect(app)} because :reboot_system_after_config has been set \
         to false and #{inspect(app)} has already been loaded, meaning any further \
         configuration won't have an effect.
 
@@ -238,9 +271,9 @@ defmodule Config.Provider do
   end
 
   @doc false
-  def validate_compile_env(compile_env) do
+  def validate_compile_env(compile_env, ensure_loaded? \\ true) do
     for {app, [key | path], compile_return} <- compile_env,
-        Application.ensure_loaded(app) == :ok do
+        ensure_app_loaded?(app, ensure_loaded?) do
       try do
         traverse_env(Application.fetch_env(app, key), path)
       rescue
@@ -275,6 +308,9 @@ defmodule Config.Provider do
     :ok
   end
 
+  defp ensure_app_loaded?(app, true), do: Application.ensure_loaded(app) == :ok
+  defp ensure_app_loaded?(app, false), do: Application.spec(app, :vsn) != nil
+
   defp path(key, []), do: "for key #{inspect(key)}"
   defp path(key, path), do: "for path #{inspect(path)} inside key #{inspect(key)}"
 
@@ -299,13 +335,22 @@ defmodule Config.Provider do
   defp traverse_env(:error, _paths), do: :error
   defp traverse_env({:ok, value}, [key | keys]), do: traverse_env(Access.fetch(value, key), keys)
 
-  defp restart_and_sleep do
-    :init.restart()
+  @compile {:no_warn_undefined, {:init, :restart, 1}}
+  defp restart_and_sleep() do
+    mode = Application.get_env(:elixir, @reboot_mode_key)
+
+    # TODO: Remove otp_release check once we require Erlang/OTP 23+
+    if :erlang.system_info(:otp_release) >= '23' and mode in [:embedded, :interactive] do
+      :init.restart(mode: mode)
+    else
+      :init.restart()
+    end
+
     Process.sleep(:infinity)
   end
 
-  defp booted_value(%{prune_after_boot: true}, path), do: {:booted, path}
-  defp booted_value(%{prune_after_boot: false}, _path), do: {:booted, nil}
+  defp booted_value(%{prune_runtime_sys_config_after_boot: true}, path), do: {:booted, path}
+  defp booted_value(%{prune_runtime_sys_config_after_boot: false}, _path), do: {:booted, nil}
 
   defp validate_no_cyclic_boot!(path) do
     if System.get_env("ELIXIR_CONFIG_PROVIDER_BOOTED") do
@@ -350,7 +395,7 @@ defmodule Config.Provider do
   defp write_config!(config, path) do
     contents = :io_lib.format("%% coding: utf-8~n~tw.~n", [config])
 
-    case File.write(path, contents, [:utf8]) do
+    case File.write(path, IO.chardata_to_string(contents)) do
       :ok ->
         :ok
 
