@@ -21,15 +21,15 @@ defmodule Exception do
 
   @typedoc "The kind handled by formatting functions"
   @type kind :: :error | non_error_kind
-  @typep non_error_kind :: :exit | :throw | {:EXIT, pid}
+  @type non_error_kind :: :exit | :throw | {:EXIT, pid}
 
   @type stacktrace :: [stacktrace_entry]
   @type stacktrace_entry ::
           {module, atom, arity_or_args, location}
           | {(... -> any), arity_or_args, location}
 
-  @typep arity_or_args :: non_neg_integer | list
-  @typep location :: keyword
+  @type arity_or_args :: non_neg_integer | list
+  @type location :: keyword
 
   @callback exception(term) :: t
   @callback message(t) :: String.t()
@@ -46,6 +46,8 @@ defmodule Exception do
   @doc """
   Returns `true` if the given `term` is an exception.
   """
+  # TODO: Remove this on Elixir v1.15
+  @doc deprecated: "Use Kernel.is_exception/1 instead"
   def exception?(term)
   def exception?(%_{__exception__: true}), do: true
   def exception?(_), do: false
@@ -188,7 +190,7 @@ defmodule Exception do
   Where `definition` is `:def`, `:defp`, `:defmacro` or `:defmacrop`.
   """
   @doc since: "1.5.0"
-  @spec blame_mfa(module, function, args :: [term]) ::
+  @spec blame_mfa(module, function :: atom, args :: [term]) ::
           {:ok, :def | :defp | :defmacro | :defmacrop, [{args :: [term], guards :: [term]}]}
           | :error
   def blame_mfa(module, function, args)
@@ -237,7 +239,11 @@ defmodule Exception do
     binding = :orddict.store(:VAR, call_arg, binding)
 
     try do
-      {:value, _, binding} = :erl_eval.expr({:match, 0, erl_arg, {:var, 0, :VAR}}, binding, :none)
+      ann = :erl_anno.new(0)
+
+      {:value, _, binding} =
+        :erl_eval.expr({:match, ann, erl_arg, {:var, ann, :VAR}}, binding, :none)
+
       {true, binding}
     rescue
       _ -> {false, binding}
@@ -246,8 +252,8 @@ defmodule Exception do
 
   defp rewrite_arg(arg) do
     Macro.prewalk(arg, fn
-      {:%{}, meta, [__struct__: Range, first: first, last: last]} ->
-        {:.., meta, [first, last]}
+      {:%{}, meta, [__struct__: Range, first: first, last: last, step: step]} ->
+        {:"..//", meta, [first, last, step]}
 
       other ->
         other
@@ -261,7 +267,13 @@ defmodule Exception do
       blame_guard(right, scope, binding)
     ]
 
-    {rewrite_guard_call(op), meta, guards}
+    kernel_op =
+      case op do
+        :orelse -> :or
+        :andalso -> :and
+      end
+
+    {kernel_op, meta, guards}
   end
 
   defp blame_guard(ex_guard, scope, binding) do
@@ -280,32 +292,17 @@ defmodule Exception do
 
   defp rewrite_guard(guard) do
     Macro.prewalk(guard, fn
-      {{:., _, [:erlang, :element]}, _, [{{:., _, [:erlang, :+]}, _, [int, 1]}, arg]} ->
-        {:elem, [], [arg, int]}
-
-      {{:., _, [:erlang, :element]}, _, [int, arg]} when is_integer(int) ->
-        {:elem, [], [arg, int - 1]}
-
-      {:., _, [:erlang, call]} ->
-        rewrite_guard_call(call)
-
-      other ->
-        other
+      {{:., _, [mod, fun]}, meta, args} -> erl_to_ex(mod, fun, args, meta)
+      other -> other
     end)
   end
 
-  defp rewrite_guard_call(:orelse), do: :or
-  defp rewrite_guard_call(:andalso), do: :and
-  defp rewrite_guard_call(:"=<"), do: :<=
-  defp rewrite_guard_call(:"/="), do: :!=
-  defp rewrite_guard_call(:"=:="), do: :===
-  defp rewrite_guard_call(:"=/="), do: :!==
-
-  defp rewrite_guard_call(op) when op in [:band, :bor, :bnot, :bsl, :bsr, :bxor],
-    do: {:., [], [Bitwise, op]}
-
-  defp rewrite_guard_call(op) when op in [:xor, :element, :size], do: {:., [], [:erlang, op]}
-  defp rewrite_guard_call(op), do: op
+  defp erl_to_ex(mod, fun, args, meta) do
+    case :elixir_rewrite.erl_to_ex(mod, fun, args) do
+      {Kernel, fun, args} -> {fun, meta, args}
+      {mod, fun, args} -> {{:., [], [mod, fun]}, meta, args}
+    end
+  end
 
   defp blame_wrap(match?, ast), do: %{match?: match?, node: ast}
 
@@ -638,6 +635,7 @@ defmodule Exception do
 
   @doc """
   Formats the given `file` and `line` as shown in stacktraces.
+
   If any of the values are `nil`, they are omitted.
 
   ## Examples
@@ -653,14 +651,42 @@ defmodule Exception do
 
   """
   def format_file_line(file, line, suffix \\ "") do
-    if file do
-      if line && line != 0 do
-        "#{file}:#{line}:#{suffix}"
-      else
-        "#{file}:#{suffix}"
-      end
-    else
+    cond do
+      is_nil(file) -> ""
+      is_nil(line) or line == 0 -> "#{file}:#{suffix}"
+      true -> "#{file}:#{line}:#{suffix}"
+    end
+  end
+
+  @doc """
+  Formats the given `file`, `line`, and `column` as shown in stacktraces.
+
+  If any of the values are `nil`, they are omitted.
+
+  ## Examples
+
+      iex> Exception.format_file_line_column("foo", 1, 2)
+      "foo:1:2:"
+
+      iex> Exception.format_file_line_column("foo", 1, nil)
+      "foo:1:"
+
+      iex> Exception.format_file_line_column("foo", nil, nil)
+      "foo:"
+
+      iex> Exception.format_file_line_column("foo", nil, 2)
+      "foo:"
+
+      iex> Exception.format_file_line_column(nil, nil, nil)
       ""
+
+  """
+  def format_file_line_column(file, line, column, suffix \\ "") do
+    cond do
+      is_nil(file) -> ""
+      is_nil(line) or line == 0 -> "#{file}:#{suffix}"
+      is_nil(column) or column == 0 -> "#{file}:#{line}:#{suffix}"
+      true -> "#{file}:#{line}:#{column}:#{suffix}"
     end
   end
 
@@ -683,7 +709,7 @@ defmodule ArgumentError do
 
   @impl true
   def blame(
-        %{message: "argument error"} = exception,
+        exception,
         [{:erlang, :apply, [module, function, args], _} | _] = stacktrace
       ) do
     message =
@@ -692,7 +718,7 @@ defmodule ArgumentError do
         not is_atom(module) and is_atom(function) and args == [] ->
           "you attempted to apply #{inspect(function)} on #{inspect(module)}. " <>
             "If you are using apply/3, make sure the module is an atom. " <>
-            "If you are using the dot syntax, such as map.field or module.function, " <>
+            "If you are using the dot syntax, such as map.field or module.function(), " <>
             "make sure the left side of the dot is an atom or a map"
 
         not is_atom(module) ->
@@ -757,30 +783,26 @@ defmodule ArithmeticError do
 end
 
 defmodule SystemLimitError do
-  defexception []
-
-  @impl true
-  def message(_) do
-    "a system limit has been reached"
-  end
+  defexception message: "a system limit has been reached"
 end
 
 defmodule SyntaxError do
-  defexception [:file, :line, description: "syntax error"]
+  defexception [:file, :line, :column, description: "syntax error"]
 
   @impl true
-  def message(exception) do
-    Exception.format_file_line(Path.relative_to_cwd(exception.file), exception.line) <>
-      " " <> exception.description
+  def message(%{file: file, line: line, column: column, description: description}) do
+    Exception.format_file_line_column(Path.relative_to_cwd(file), line, column) <>
+      " " <> description
   end
 end
 
 defmodule TokenMissingError do
-  defexception [:file, :line, description: "expression is incomplete"]
+  defexception [:file, :line, :column, description: "expression is incomplete"]
 
   @impl true
-  def message(%{file: file, line: line, description: description}) do
-    Exception.format_file_line(file && Path.relative_to_cwd(file), line) <> " " <> description
+  def message(%{file: file, line: line, column: column, description: description}) do
+    Exception.format_file_line_column(file && Path.relative_to_cwd(file), line, column) <>
+      " " <> description
   end
 end
 
@@ -950,7 +972,7 @@ defmodule UndefinedFunctionError do
   end
 
   defp hint(nil, _function, 0, _loaded?) do
-    ". If you are using the dot syntax, such as map.field or module.function, " <>
+    ". If you are using the dot syntax, such as map.field or module.function(), " <>
       "make sure the left side of the dot is an atom or a map"
   end
 
@@ -965,11 +987,23 @@ defmodule UndefinedFunctionError do
 
   @doc false
   def hint_for_loaded_module(module, function, arity, exports) do
-    if macro_exported?(module, function, arity) do
-      ". However there is a macro with the same name and arity. " <>
-        "Be sure to require #{inspect(module)} if you intend to invoke this macro"
-    else
-      IO.iodata_to_binary(did_you_mean(module, function, exports))
+    cond do
+      macro_exported?(module, function, arity) ->
+        ". However there is a macro with the same name and arity. " <>
+          "Be sure to require #{inspect(module)} if you intend to invoke this macro"
+
+      message = otp_obsolete(module, function, arity) ->
+        ", #{message}"
+
+      true ->
+        IO.iodata_to_binary(did_you_mean(module, function, exports))
+    end
+  end
+
+  defp otp_obsolete(module, function, arity) do
+    case :otp_internal.obsolete(module, function, arity) do
+      {:removed, [_ | _] = string} -> string
+      _ -> nil
     end
   end
 
@@ -1221,6 +1255,10 @@ defmodule KeyError do
   end
 
   @impl true
+  def blame(exception = %{message: message}, stacktrace) when is_binary(message) do
+    {exception, stacktrace}
+  end
+
   def blame(exception = %{term: nil}, stacktrace) do
     message = message(exception.key, exception.term)
     {%{exception | message: message}, stacktrace}
@@ -1329,7 +1367,7 @@ defmodule File.CopyError do
     formatted = IO.iodata_to_binary(:file.format_error(exception.reason))
 
     location =
-      case exception.on() do
+      case exception.on do
         "" -> ""
         on -> ". #{on}"
       end
@@ -1347,7 +1385,7 @@ defmodule File.RenameError do
     formatted = IO.iodata_to_binary(:file.format_error(exception.reason))
 
     location =
-      case exception.on() do
+      case exception.on do
         "" -> ""
         on -> ". #{on}"
       end
@@ -1378,16 +1416,32 @@ defmodule ErlangError do
   end
 
   @doc false
-  def normalize(:badarg, _stacktrace) do
-    %ArgumentError{}
+  def normalize(:badarg, stacktrace) do
+    case error_info(:badarg, stacktrace) do
+      {:ok, args} ->
+        message = "errors were found at the given arguments:\n\n#{args}"
+        %ArgumentError{message: message}
+
+      :error ->
+        %ArgumentError{}
+    end
   end
 
   def normalize(:badarith, _stacktrace) do
     %ArithmeticError{}
   end
 
-  def normalize(:system_limit, _stacktrace) do
-    %SystemLimitError{}
+  def normalize(:system_limit, stacktrace) do
+    case error_info(:system_limit, stacktrace) do
+      {:ok, args} ->
+        message =
+          "a system limit has been reached due to errors at the given arguments:\n\n#{args}"
+
+        %SystemLimitError{message: message}
+
+      :error ->
+        %SystemLimitError{}
+    end
   end
 
   def normalize(:cond_clause, _stacktrace) do
@@ -1477,4 +1531,31 @@ defmodule ErlangError do
   defp from_stacktrace(_) do
     {nil, nil, nil}
   end
+
+  @doc false
+  def error_info(erl_exception, stacktrace) do
+    with [{module, _, args_or_arity, opts} | _] <- stacktrace,
+         %{} = error_info <- opts[:error_info] do
+      module = Map.get(error_info, :module, module)
+      function = Map.get(error_info, :function, :format_error)
+      arity = if is_integer(args_or_arity), do: args_or_arity, else: length(args_or_arity)
+      extra = apply(module, function, [erl_exception, stacktrace])
+      args_errors = Map.take(extra, Enum.to_list(1..arity//1))
+
+      if map_size(args_errors) > 0 do
+        {:ok, IO.iodata_to_binary(Enum.map(args_errors, &arg_error/1))}
+      else
+        :error
+      end
+    else
+      _ -> :error
+    end
+  end
+
+  defp arg_error({n, message}), do: "  * #{nth(n)} argument: #{message}\n"
+
+  defp nth(1), do: "1st"
+  defp nth(2), do: "2nd"
+  defp nth(3), do: "3rd"
+  defp nth(n), do: "#{n}th"
 end
