@@ -26,20 +26,22 @@ defprotocol Inspect do
       defimpl Inspect, for: MapSet do
         import Inspect.Algebra
 
-        def inspect(dict, opts) do
-          concat(["#MapSet<", to_doc(MapSet.to_list(dict), opts), ">"])
+        def inspect(map_set, opts) do
+          concat(["#MapSet<", to_doc(MapSet.to_list(map_set), opts), ">"])
         end
       end
 
-  The [`concat/1`](`Inspect.Algebra.concat/1`) function comes from `Inspect.Algebra` and it
-  concatenates algebra documents together. In the example above it is
-  concatenating the string `"MapSet<"`, the document returned by
-  `Inspect.Algebra.to_doc/2`, and the final string `">"`. All strings are
-  valid algebra documents that keep their formatting when pretty printed.
+  The [`concat/1`](`Inspect.Algebra.concat/1`) function comes from
+  `Inspect.Algebra` and it concatenates algebra documents together.
+  In the example above it is concatenating the string `"#MapSet<"`,
+  the document returned by `Inspect.Algebra.to_doc/2`, and the final
+  string `">"`. We prefix the module name `#` to denote the inspect
+  presentation is not actually valid Elixir syntax.
 
-  Since regular strings are valid entities in an algebra document,
-  an implementation of the `Inspect` protocol may simply return a
-  string, although that will devoid it of any pretty-printing.
+  Finally, note strings themselves are valid algebra documents that
+  keep their formatting when pretty printed. This means your `Inspect`
+  implementation may simply return a string, although that will devoid
+  it of any pretty-printing.
 
   ## Error handling
 
@@ -437,7 +439,7 @@ defimpl Inspect, for: Any do
 
   def inspect(%module{} = struct, opts) do
     try do
-      module.__struct__
+      module.__struct__()
     rescue
       _ -> Inspect.Map.inspect(struct, opts)
     else
@@ -452,16 +454,17 @@ defimpl Inspect, for: Any do
   end
 
   def inspect(map, name, opts) do
-    # Use the :limit option and an extra element to force
-    # `container_doc/6` to append "...".
-    opts = %{opts | limit: min(opts.limit, map_size(map))}
-    map = Map.to_list(map) ++ ["..."]
-
+    map = Map.to_list(map) ++ [:...]
     open = color("#" <> name <> "<", :map, opts)
     sep = color(",", :map, opts)
     close = color(">", :map, opts)
 
-    container_doc(open, map, close, opts, &Inspect.List.keyword/2, separator: sep, break: :strict)
+    fun = fn
+      {key, value}, opts -> Inspect.List.keyword({key, value}, opts)
+      :..., _opts -> "..."
+    end
+
+    container_doc(open, map, close, opts, fun, separator: sep, break: :strict)
   end
 end
 
