@@ -2,16 +2,62 @@ import Kernel, except: [to_string: 1]
 
 defmodule Macro do
   @moduledoc ~S"""
-  Conveniences for working with macros.
+  Macros are compile-time constructs that are invoked with Elixir's AST
+  as input and a superset of Elixir's AST as output.
+
+  Let's see a simple example that shows the difference between functions and macros:
+
+      defmodule Example do
+        defmacro macro_inspect(value) do
+          IO.inspect(value)
+          value
+        end
+
+        def fun_inspect(value) do
+          IO.inspect(value)
+          value
+        end
+      end
+
+  Now let's give it a try:
+
+      import Example
+
+      macro_inspect(1)
+      #=> 1
+      #=> 1
+
+      fun_inspect(1)
+      #=> 1
+      #=> 1
+
+  So far they behave the same, as we are passing an integer as argument.
+  But what happens when we pass an expression:
+
+      macro_inspect(1 + 2)
+      #=> {:+, [line: 3], [1, 2]}
+      #=> 3
+
+      fun_inspect(1 + 2)
+      #=> 3
+      #=> 3
+
+  The macro receives the representation of the code given as argument,
+  while a function receives the result of the code given as argument.
+  A macro must return a superset of the code representation. See
+  `t:input/0` and `t:output/0` for more information.
+
+  To learn more about Elixir's AST and how to build them programmatically,
+  see `quote/2`.
 
   ## Custom Sigils
 
-  To create a custom sigil, define a function with the name
-  `sigil_{identifier}` that takes two arguments. The first argument will be
-  the string, the second will be a charlist containing any modifiers. If the
-  sigil is lower case (such as `sigil_x`) then the string argument will allow
-  interpolation. If the sigil is upper case (such as `sigil_X`) then the string
-  will not be interpolated.
+  Macros are also commonly used to implement custom sigils. To create a custom
+  sigil, define a function with the name `sigil_{identifier}` that takes two
+  arguments. The first argument will be the string, the second will be a charlist
+  containing any modifiers. If the sigil is lower case (such as `sigil_x`) then
+  the string argument will allow interpolation. If the sigil is upper case
+  (such as `sigil_X`) then the string will not be interpolated.
 
   Valid modifiers include only lower and upper case letters. Other characters
   will cause a syntax error.
@@ -59,13 +105,95 @@ defmodule Macro do
   alias Code.Identifier
 
   @typedoc "Abstract Syntax Tree (AST)"
-  @type t :: expr | literal
+  @type t :: input
 
-  @typedoc "Represents expressions in the AST"
-  @type expr :: {expr | atom, keyword, atom | [t]}
+  @typedoc "The inputs of a macro"
+  @type input ::
+          input_expr
+          | {input, input}
+          | [input]
+          | atom
+          | number
+          | binary
 
-  @typedoc "Represents literals in the AST"
-  @type literal :: atom | number | binary | fun | {t, t} | [t]
+  @typep input_expr :: {input_expr | atom, metadata, atom | [input]}
+
+  @typedoc "The output of a macro"
+  @type output ::
+          output_expr
+          | {output, output}
+          | [output]
+          | atom
+          | number
+          | binary
+          | captured_remote_function
+          | pid
+
+  @typep output_expr :: {output_expr | atom, metadata, atom | [output]}
+
+  @typedoc """
+  A keyword list of AST metadata.
+
+  The metadata in Elixir AST is a keyword list of values. Any key can be used
+  and different parts of the compiler may use different keys. For example,
+  the AST received by a macro will always include the `:line` annotation,
+  while the AST emitted by `quote/2` will only have the `:line` annotation if
+  the `:line` option is provided.
+
+  The following metadata keys are public:
+
+    * `:context` - Defines the context in which the AST was generated.
+      For example, `quote/2` will include the module calling `quote/2`
+      as the context. This is often used to distinguish regular code from code
+      generated by a macro or by `quote/2`.
+    * `:counter` - The variable counter used for variable hygiene. In terms of
+      the compiler, each variable is identified by the combination of either
+      `name` and `metadata[:counter]`, or `name` and `context`.
+    * `:generated` - Whether the code should be considered as generated by
+      the compiler or not. This means the compiler and tools like Dialyzer may not
+      emit certain warnings.
+    * `:keep` - Used by `quote/2` with the option `location: :keep` to annotate
+      the file and the line number of the quoted source.
+    * `:line` - The line number of the AST node.
+
+  The following metadata keys are enabled by `Code.string_to_quoted/2`:
+
+    * `:closing` - contains metadata about the closing pair, such as a `}`
+      in a tuple or in a map, or such as the closing `)` in a function call
+      with parens. The `:closing` does not delimit the end of expression if
+      there are `:do` and `:end` metadata  (when `:token_metadata` is true)
+    * `:column` - the column number of the AST node (when `:columns` is true)
+    * `:delimiter` - contains the opening delimiter for sigils, strings,
+      and charlists as a string (such as `"{"`, `"/"`, `"'"`, and the like)
+    * `:format` - set to `:keyword` when an atom is defined as a keyword
+    * `:do` - contains metadata about the `do` location in a function call with
+      `do/end` blocks (when `:token_metadata` is true)
+    * `:end` - contains metadata about the `end` location in a function call with
+      `do/end` blocks (when `:token_metadata` is true)
+    * `:end_of_expression` - denotes when the end of expression effectively
+      happens. Available for all expressions except the last one inside a
+      `__block__` (when `:token_metadata` is true)
+    * `:indentation` - indentation of a sigil heredoc
+
+  The following metadata keys are private:
+
+    * `:alias` - Used for alias hygiene.
+    * `:ambiguous_op` - Used for improved error messages in the compiler.
+    * `:import` - Used for import hygiene.
+    * `:var` - Used for improved error messages on undefined variables.
+
+  Do not rely on them as they may change or be fully removed in future versions
+  of the language. They are often used by `quote/2` and the compiler to provide
+  features like hygiene, better error messages, and so forth.
+
+  If you introduce custom keys into the AST metadata, please make sure to prefix
+  them with the name of your library or application, so that they will not conflict
+  with keys that could potentially be introduced by the compiler in the future.
+  """
+  @type metadata :: keyword
+
+  @typedoc "A captured remote function in the format of &Mod.fun/arity"
+  @type captured_remote_function :: fun
 
   @doc """
   Breaks a pipeline expression into a list.
@@ -138,11 +266,11 @@ defmodule Macro do
   end
 
   # {:fn, _, _} is what we get when we pipe into an anonymous function without
-  # calling it, e.g., `:foo |> (fn x -> x end)`.
+  # calling it, for example, `:foo |> (fn x -> x end)`.
   def pipe(expr, {:fn, _, _}, _integer) do
     raise ArgumentError,
           "cannot pipe #{to_string(expr)} into an anonymous function without" <>
-            " calling the function; use something like (fn ... end).() or" <>
+            " calling the function; use Kernel.then/2 instead or" <>
             " define the anonymous function as a regular private function"
   end
 
@@ -150,13 +278,26 @@ defmodule Macro do
     {call, line, List.insert_at([], integer, expr)}
   end
 
-  def pipe(expr, {call, line, args} = call_args, integer) when is_list(args) do
-    if is_atom(call) and Identifier.binary_op(call) != :error do
-      raise ArgumentError,
-            "cannot pipe #{to_string(expr)} into #{to_string(call_args)}, " <>
-              "the #{to_string(call)} operator can only take two arguments"
-    else
-      {call, line, List.insert_at(args, integer, expr)}
+  def pipe(_expr, {op, _line, [arg]}, _integer) when op == :+ or op == :- do
+    raise ArgumentError,
+          "piping into a unary operator is not supported, please use the qualified name: " <>
+            "Kernel.#{op}(#{to_string(arg)}), instead of #{op}#{to_string(arg)}"
+  end
+
+  def pipe(expr, {op, line, args} = op_args, integer) when is_list(args) do
+    cond do
+      is_atom(op) and Identifier.unary_op(op) != :error ->
+        raise ArgumentError,
+              "cannot pipe #{to_string(expr)} into #{to_string(op_args)}, " <>
+                "the #{to_string(op)} operator can only take one argument"
+
+      is_atom(op) and Identifier.binary_op(op) != :error ->
+        raise ArgumentError,
+              "cannot pipe #{to_string(expr)} into #{to_string(op_args)}, " <>
+                "the #{to_string(op)} operator can only take two arguments"
+
+      true ->
+        {op, line, List.insert_at(args, integer, expr)}
     end
   end
 
@@ -196,8 +337,13 @@ defmodule Macro do
   end
 
   @doc """
-  Generates AST nodes for a given number of required argument variables using
-  `Macro.var/2`.
+  Generates AST nodes for a given number of required argument
+  variables using `Macro.var/2`.
+
+  Note the arguments are not unique. If you later on want
+  to access the same variables, you can invoke this function
+  with the same inputs. Use `generate_unique_arguments/2` to
+  generate a unique arguments that can't be overridden.
 
   ## Examples
 
@@ -208,19 +354,47 @@ defmodule Macro do
   @doc since: "1.5.0"
   @spec generate_arguments(0, context :: atom) :: []
   @spec generate_arguments(pos_integer, context) :: [{atom, [], context}, ...] when context: atom
-  def generate_arguments(amount, context)
+  def generate_arguments(amount, context), do: generate_arguments(amount, context, &var/2)
 
-  def generate_arguments(0, context) when is_atom(context), do: []
+  @doc """
+  Generates AST nodes for a given number of required argument
+  variables using `Macro.unique_var/2`.
 
-  def generate_arguments(amount, context)
-      when is_integer(amount) and amount > 0 and is_atom(context) do
-    for id <- 1..amount, do: var(String.to_atom("arg" <> Integer.to_string(id)), context)
+  ## Examples
+
+      iex> [var1, var2] = Macro.generate_unique_arguments(2, __MODULE__)
+      iex> {:arg1, [counter: c1], __MODULE__} = var1
+      iex> {:arg2, [counter: c2], __MODULE__} = var2
+      iex> is_integer(c1) and is_integer(c2)
+      true
+
+  """
+  @doc since: "1.11.3"
+  @spec generate_unique_arguments(0, context :: atom) :: []
+  @spec generate_unique_arguments(pos_integer, context) :: [
+          {atom, [counter: integer], context},
+          ...
+        ]
+        when context: atom
+  def generate_unique_arguments(amount, context),
+    do: generate_arguments(amount, context, &unique_var/2)
+
+  defp generate_arguments(0, context, _fun) when is_atom(context), do: []
+
+  defp generate_arguments(amount, context, fun)
+       when is_integer(amount) and amount > 0 and is_atom(context) do
+    for id <- 1..amount, do: fun.(String.to_atom("arg" <> Integer.to_string(id)), context)
   end
 
   @doc """
   Generates an AST node representing the variable given
   by the atoms `var` and `context`.
 
+  Note this variable is not unique. If you later on want
+  to access this same variable, you can invoke `var/2`
+  again with the same arguments. Use `unique_var/2` to
+  generate a unique variable that can't be overridden.
+
   ## Examples
 
   In order to build a variable, a context is expected.
@@ -242,6 +416,24 @@ defmodule Macro do
     {var, [], context}
   end
 
+  @doc """
+  Generates an AST node representing a unique variable
+  given by the atoms `var` and `context`.
+
+  ## Examples
+
+      iex> {:foo, [counter: c], __MODULE__} = Macro.unique_var(:foo, __MODULE__)
+      iex> is_integer(c)
+      true
+
+  """
+  @doc since: "1.11.3"
+  @spec unique_var(var, context) :: {var, [counter: integer], context}
+        when var: atom, context: atom
+  def unique_var(var, context) when is_atom(var) and is_atom(context) do
+    {var, [counter: :elixir_module.next_counter(context)], context}
+  end
+
   @doc """
   Performs a depth-first traversal of quoted expressions
   using an accumulator.
@@ -286,10 +478,14 @@ defmodule Macro do
   end
 
   defp do_traverse_args(args, acc, pre, post) when is_list(args) do
-    Enum.map_reduce(args, acc, fn x, acc ->
-      {x, acc} = pre.(x, acc)
-      do_traverse(x, acc, pre, post)
-    end)
+    :lists.mapfoldl(
+      fn x, acc ->
+        {x, acc} = pre.(x, acc)
+        do_traverse(x, acc, pre, post)
+      end,
+      acc,
+      args
+    )
   end
 
   @doc """
@@ -349,10 +545,15 @@ defmodule Macro do
       iex> Macro.decompose_call(quote(do: 42))
       :error
 
+      iex> Macro.decompose_call(quote(do: {:foo, [], []}))
+      :error
+
   """
   @spec decompose_call(t()) :: {atom, [t()]} | {t(), atom, [t()]} | :error
   def decompose_call(ast)
 
+  def decompose_call({:{}, _, args}) when is_list(args), do: :error
+
   def decompose_call({{:., _, [remote, function]}, _, args})
       when is_tuple(remote) or is_atom(remote),
       do: {remote, function, args}
@@ -392,7 +593,7 @@ defmodule Macro do
       As an example, `ExUnit` stores the AST of every assertion, so when
       an assertion fails we can show code snippets to users. Without this
       option, each time the test module is compiled, we get a different
-      MD5 of the module byte code, because the AST contains metadata,
+      MD5 of the module bytecode, because the AST contains metadata,
       such as counters, specific to the compilation environment. By pruning
       the metadata, we ensure that the module is deterministic and reduce
       the amount of data `ExUnit` needs to keep around.
@@ -435,17 +636,19 @@ defmodule Macro do
 
   This is useful when a struct needs to be expanded at
   compilation time and the struct being expanded may or may
-  not have been compiled. This function is even capable of
+  not have been compiled. This function is also capable of
   expanding structs defined under the module being compiled.
 
   It will raise `CompileError` if the struct is not available.
+  From Elixir v1.12, calling this function also adds an export
+  dependency on the given struct.
   """
   @doc since: "1.8.0"
   @spec struct!(module, Macro.Env.t()) :: %{__struct__: module} when module: module()
   def struct!(module, env) when is_atom(module) do
     if module == env.module do
-      Module.get_attribute(module, :struct)
-    end || :elixir_map.load_struct([line: env.line], module, [], env)
+      Module.get_attribute(module, :__struct__)
+    end || :elixir_map.load_struct([line: env.line], module, [], [], env)
   end
 
   @doc """
@@ -522,8 +725,8 @@ defmodule Macro do
   and return a version with it unescaped.
   """
   @spec unescape_string(String.t()) :: String.t()
-  def unescape_string(chars) do
-    :elixir_interpolation.unescape_chars(chars)
+  def unescape_string(string) do
+    :elixir_interpolation.unescape_string(string)
   end
 
   @doc ~S"""
@@ -538,8 +741,9 @@ defmodule Macro do
   representing the code point of the character it wants to unescape.
   Here is the default mapping function implemented by Elixir:
 
-      def unescape_map(unicode), do: true
-      def unescape_map(hex), do: true
+      def unescape_map(:newline), do: true
+      def unescape_map(:unicode), do: true
+      def unescape_map(:hex), do: true
       def unescape_map(?0), do: ?0
       def unescape_map(?a), do: ?\a
       def unescape_map(?b), do: ?\b
@@ -556,9 +760,9 @@ defmodule Macro do
   If the `unescape_map/1` function returns `false`, the char is
   not escaped and the backslash is kept in the string.
 
-  Hexadecimals and Unicode code points will be escaped if the map
-  function returns `true` for `?x`. Unicode code points if the map
-  function returns `true` for `?u`.
+  Newlines, Unicode, and hexadecimals code points will be escaped if
+  the map returns `true` respectively for `:newline`, `:unicode`, and
+  `:hex`.
 
   ## Examples
 
@@ -568,25 +772,23 @@ defmodule Macro do
 
   """
   @spec unescape_string(String.t(), (non_neg_integer -> non_neg_integer | false)) :: String.t()
-  def unescape_string(chars, map) do
-    :elixir_interpolation.unescape_chars(chars, map)
+  def unescape_string(string, map) do
+    :elixir_interpolation.unescape_string(string, map)
   end
 
   @doc false
   @deprecated "Traverse over the arguments using Enum.map/2 instead"
   def unescape_tokens(tokens) do
-    case :elixir_interpolation.unescape_tokens(tokens) do
-      {:ok, unescaped_tokens} -> unescaped_tokens
-      {:error, reason} -> raise ArgumentError, to_string(reason)
+    for token <- tokens do
+      if is_binary(token), do: unescape_string(token), else: token
     end
   end
 
   @doc false
   @deprecated "Traverse over the arguments using Enum.map/2 instead"
   def unescape_tokens(tokens, map) do
-    case :elixir_interpolation.unescape_tokens(tokens, map) do
-      {:ok, unescaped_tokens} -> unescaped_tokens
-      {:error, reason} -> raise ArgumentError, to_string(reason)
+    for token <- tokens do
+      if is_binary(token), do: unescape_string(token, map), else: token
     end
   end
 
@@ -755,9 +957,19 @@ defmodule Macro do
   end
 
   # All other calls
+  def to_string({{:., _, [left, _]} = target, meta, []} = ast, fun) do
+    to_string = call_to_string(target, fun)
+
+    if is_tuple(left) && meta[:no_parens] do
+      fun.(ast, to_string)
+    else
+      fun.(ast, to_string <> "()")
+    end
+  end
+
   def to_string({target, _, args} = ast, fun) when is_list(args) do
     with :error <- unary_call(ast, fun),
-         :error <- binary_call(ast, fun),
+         :error <- op_call(ast, fun),
          :error <- sigil_call(ast, fun) do
       {list, last} = split_last(args)
 
@@ -812,10 +1024,14 @@ defmodule Macro do
     Kernel.inspect(value, limit: :infinity, printable_limit: :infinity)
   end
 
-  defp bitpart_to_string({:"::", _, [left, right]} = ast, fun) do
+  defp bitpart_to_string({:"::", meta, [left, right]} = ast, fun) do
     result =
-      op_to_string(left, fun, :"::", :left) <>
-        "::" <> bitmods_to_string(right, fun, :"::", :right)
+      if meta[:inferred_bitstring_spec] do
+        to_string(left, fun)
+      else
+        op_to_string(left, fun, :"::", :left) <>
+          "::" <> bitmods_to_string(right, fun, :"::", :right)
+      end
 
     fun.(ast, result)
   end
@@ -858,20 +1074,31 @@ defmodule Macro do
     false
   end
 
-  defp interpolate({:<<>>, _, parts}, fun) do
+  defp interpolate(ast, fun), do: interpolate(ast, "\"", "\"", fun)
+
+  defp interpolate({:<<>>, _, [parts]}, left, right, _) when left in [~s["""\n], ~s['''\n]] do
+    <<left::binary, parts::binary, right::binary>>
+  end
+
+  defp interpolate({:<<>>, _, parts}, left, right, fun) do
     parts =
       Enum.map_join(parts, "", fn
         {:"::", _, [{{:., _, [Kernel, :to_string]}, _, [arg]}, {:binary, _, _}]} ->
           "\#{" <> to_string(arg, fun) <> "}"
 
         binary when is_binary(binary) ->
-          binary = inspect_no_limit(binary)
-          binary_part(binary, 1, byte_size(binary) - 2)
+          escape_sigil(binary, left)
       end)
 
-    <<?", parts::binary, ?">>
+    <<left::binary, parts::binary, right::binary>>
   end
 
+  defp escape_sigil(parts, "("), do: String.replace(parts, ")", ~S"\)")
+  defp escape_sigil(parts, "{"), do: String.replace(parts, "}", ~S"\}")
+  defp escape_sigil(parts, "["), do: String.replace(parts, "]", ~S"\]")
+  defp escape_sigil(parts, "<"), do: String.replace(parts, ">", ~S"\>")
+  defp escape_sigil(parts, delimiter), do: String.replace(parts, delimiter, "\\#{delimiter}")
+
   defp module_to_string(atom, _fun) when is_atom(atom) do
     inspect_no_limit(atom)
   end
@@ -914,7 +1141,14 @@ defmodule Macro do
     :error
   end
 
-  defp binary_call({op, _, [left, right]} = ast, fun) when is_atom(op) do
+  defp op_call({:"..//", _, [left, middle, right]} = ast, fun) do
+    left = op_to_string(left, fun, :.., :left)
+    middle = op_to_string(middle, fun, :.., :right)
+    right = op_to_string(right, fun, :"//", :right)
+    {:ok, fun.(ast, left <> ".." <> middle <> "//" <> right)}
+  end
+
+  defp op_call({op, _, [left, right]} = ast, fun) when is_atom(op) do
     case Identifier.binary_op(op) do
       {_, _} ->
         left = op_to_string(left, fun, op, :left)
@@ -927,29 +1161,26 @@ defmodule Macro do
     end
   end
 
-  defp binary_call(_, _) do
+  defp op_call(_, _) do
     :error
   end
 
-  defp sigil_call({sigil, _, [{:<<>>, _, _} = parts, args]} = ast, fun)
+  defp sigil_call({sigil, meta, [{:<<>>, _, _} = parts, args]} = ast, fun)
        when is_atom(sigil) and is_list(args) do
+    delimiter = Keyword.get(meta, :delimiter, "\"")
+    {left, right} = delimiter_pair(delimiter)
+
     case Atom.to_string(sigil) do
       <<"sigil_", name>> when name >= ?A and name <= ?Z ->
+        args = sigil_args(args, fun)
         {:<<>>, _, [binary]} = parts
-
-        formatted =
-          if :binary.last(binary) == ?\n do
-            binary = String.replace(binary, ~s["""], ~s["\\""])
-            <<?~, name, ~s["""\n], binary::binary, ~s["""], sigil_args(args, fun)::binary>>
-          else
-            {left, right} = select_sigil_container(binary)
-            <<?~, name, left, binary::binary, right, sigil_args(args, fun)::binary>>
-          end
-
+        formatted = <<?~, name, left::binary, binary::binary, right::binary, args::binary>>
         {:ok, fun.(ast, formatted)}
 
       <<"sigil_", name>> when name >= ?a and name <= ?z ->
-        {:ok, fun.(ast, "~" <> <<name>> <> interpolate(parts, fun) <> sigil_args(args, fun))}
+        args = sigil_args(args, fun)
+        formatted = "~" <> <<name>> <> interpolate(parts, left, right, fun) <> args
+        {:ok, fun.(ast, formatted)}
 
       _ ->
         :error
@@ -960,17 +1191,13 @@ defmodule Macro do
     :error
   end
 
-  defp select_sigil_container(binary) do
-    cond do
-      :binary.match(binary, ["\""]) == :nomatch -> {?", ?"}
-      :binary.match(binary, ["\'"]) == :nomatch -> {?', ?'}
-      :binary.match(binary, ["(", ")"]) == :nomatch -> {?(, ?)}
-      :binary.match(binary, ["[", "]"]) == :nomatch -> {?[, ?]}
-      :binary.match(binary, ["{", "}"]) == :nomatch -> {?{, ?}}
-      :binary.match(binary, ["<", ">"]) == :nomatch -> {?<, ?>}
-      true -> {?/, ?/}
-    end
-  end
+  defp delimiter_pair("["), do: {"[", "]"}
+  defp delimiter_pair("{"), do: {"{", "}"}
+  defp delimiter_pair("("), do: {"(", ")"}
+  defp delimiter_pair("<"), do: {"<", ">"}
+  defp delimiter_pair("\"\"\""), do: {"\"\"\"\n", "\"\"\""}
+  defp delimiter_pair("'''"), do: {"'''\n", "'''"}
+  defp delimiter_pair(str), do: {str, str}
 
   defp sigil_args([], _fun), do: ""
   defp sigil_args(args, fun), do: fun.(args, List.to_string(args))
@@ -1213,10 +1440,10 @@ defmodule Macro do
     elem(do_expand_once(ast, env), 0)
   end
 
-  defp do_expand_once({:__aliases__, _, _} = original, env) do
-    case :elixir_aliases.expand(original, env.aliases, env.macro_aliases, env.lexical_tracker) do
+  defp do_expand_once({:__aliases__, meta, _} = original, env) do
+    case :elixir_aliases.expand(original, env) do
       receiver when is_atom(receiver) ->
-        :elixir_lexical.record_remote(receiver, env.function, env.lexical_tracker)
+        :elixir_env.trace({:alias_reference, meta, receiver}, env)
         {receiver, true}
 
       aliases ->
@@ -1225,7 +1452,7 @@ defmodule Macro do
         case :lists.all(&is_atom/1, aliases) do
           true ->
             receiver = :elixir_aliases.concat(aliases)
-            :elixir_lexical.record_remote(receiver, env.function, env.lexical_tracker)
+            :elixir_env.trace({:alias_reference, meta, receiver}, env)
             {receiver, true}
 
           false ->
@@ -1252,17 +1479,9 @@ defmodule Macro do
     end
   end
 
-  # Expand possible macro import invocation
-  defp do_expand_once({atom, meta, context} = original, env)
+  defp do_expand_once({atom, meta, context} = original, _env)
        when is_atom(atom) and is_list(meta) and is_atom(context) do
-    if Macro.Env.has_var?(env, {atom, Keyword.get(meta, :counter, context)}) do
-      {original, false}
-    else
-      case do_expand_once({atom, meta, []}, env) do
-        {_, true} = exp -> exp
-        {_, false} -> {original, false}
-      end
-    end
+    {original, false}
   end
 
   defp do_expand_once({atom, meta, args} = original, env)
@@ -1347,19 +1566,46 @@ defmodule Macro do
   def operator?(name, arity) when is_atom(name) and is_integer(arity), do: false
 
   @doc """
-  Returns `true` if the given quoted expression is an AST literal.
+  Returns `true` if the given quoted expression represents a quoted literal.
+
+  Atoms, numbers, and functions are always literals. Binaries, lists, tuples,
+  maps, and structs are only literals if all of their terms are also literals.
+
+  ## Examples
+
+      iex> Macro.quoted_literal?(quote(do: "foo"))
+      true
+      iex> Macro.quoted_literal?(quote(do: {"foo", 1}))
+      true
+      iex> Macro.quoted_literal?(quote(do: {"foo", 1, :baz}))
+      true
+      iex> Macro.quoted_literal?(quote(do: %{foo: "bar"}))
+      true
+      iex> Macro.quoted_literal?(quote(do: %URI{path: "/"}))
+      true
+      iex> Macro.quoted_literal?(quote(do: URI.parse("/")))
+      false
+      iex> Macro.quoted_literal?(quote(do: {foo, var}))
+      false
+
   """
   @doc since: "1.7.0"
-  @spec quoted_literal?(literal) :: true
-  @spec quoted_literal?(expr) :: false
+  @spec quoted_literal?(t) :: boolean
   def quoted_literal?(term)
 
+  def quoted_literal?({:__aliases__, _, args}),
+    do: quoted_literal?(args)
+
+  def quoted_literal?({:%, _, [left, right]}),
+    do: quoted_literal?(left) and quoted_literal?(right)
+
+  def quoted_literal?({:%{}, _, args}), do: quoted_literal?(args)
+  def quoted_literal?({:{}, _, args}), do: quoted_literal?(args)
   def quoted_literal?({left, right}), do: quoted_literal?(left) and quoted_literal?(right)
   def quoted_literal?(list) when is_list(list), do: Enum.all?(list, &quoted_literal?/1)
 
-  def quoted_literal?(term) do
-    is_atom(term) or is_number(term) or is_binary(term) or is_function(term)
-  end
+  def quoted_literal?(term),
+    do: is_atom(term) or is_number(term) or is_binary(term) or is_function(term)
 
   @doc """
   Receives an AST node and expands it until it can no longer
@@ -1433,7 +1679,8 @@ defmodule Macro do
   end
 
   defp do_underscore(<<h, t, rest::binary>>, _)
-       when h >= ?A and h <= ?Z and not (t >= ?A and t <= ?Z) and t != ?. and t != ?_ do
+       when h >= ?A and h <= ?Z and not (t >= ?A and t <= ?Z) and not (t >= ?0 and t <= ?9) and
+              t != ?. and t != ?_ do
     <<?_, to_lower_char(h), t>> <> do_underscore(rest, t)
   end
 
