@@ -57,7 +57,7 @@ defmodule MapSet do
   @opaque t(value) :: %__MODULE__{map: %{optional(value) => []}}
   @type t :: t(term)
 
-  # TODO: Remove version key on v2.0
+  # TODO: Remove version key on Elixir v2.0
   defstruct map: %{}, version: 2
 
   @doc """
@@ -223,7 +223,9 @@ defmodule MapSet do
   @doc """
   Checks if two sets are equal.
 
-  The comparison between elements must be done using `===/2`.
+  The comparison between elements is done using `===/2`,
+  which a set with `1` is not equivalent to a set with
+  `1.0`.
 
   ## Examples
 
@@ -231,14 +233,16 @@ defmodule MapSet do
       true
       iex> MapSet.equal?(MapSet.new([1, 2]), MapSet.new([3, 4]))
       false
+      iex> MapSet.equal?(MapSet.new([1]), MapSet.new([1.0]))
+      false
 
   """
   @spec equal?(t, t) :: boolean
   def equal?(%MapSet{map: map1, version: version}, %MapSet{map: map2, version: version}) do
-    Map.equal?(map1, map2)
+    map1 === map2
   end
 
-  # Elixir v1.5 change the map representation, so on
+  # Elixir v1.5 changed the map representation, so on
   # version mismatch we need to compare the keys directly.
   def equal?(%MapSet{map: map1}, %MapSet{map: map2}) do
     map_size(map1) == map_size(map2) and all_in?(map1, map2)
