@@ -57,7 +57,7 @@ defmodule Module do
   one of the callbacks, a warning will be raised.
 
   For detailed documentation, see the
-  [behaviour typespec documentation](typespecs.html#behaviours).
+  [behaviour typespec documentation](typespecs.md#behaviours).
 
   ### `@impl`
 
@@ -235,7 +235,7 @@ defmodule Module do
       end
 
   For the list of supported warnings, see
-  [`:dialyzer` module](http://www.erlang.org/doc/man/dialyzer.html).
+  [`:dialyzer` module](`:dialyzer`).
 
   Multiple uses of `@dialyzer` will accumulate instead of overriding
   previous ones.
@@ -248,8 +248,13 @@ defmodule Module do
   attribute allows the module to annotate which external resources
   have been used.
 
-  Tools like Mix may use this information to ensure the module is
-  recompiled in case any of the external resources change.
+  Tools may use this information to ensure the module is recompiled
+  in case any of the external resources change, see for example:
+  [`mix compile.elixir`](https://hexdocs.pm/mix/Mix.Tasks.Compile.Elixir.html).
+
+  If the external resource does not exist, the module still has
+  a dependency on it, causing the module to be recompiled as soon
+  as the file is added.
 
   ### `@file`
 
@@ -300,9 +305,9 @@ defmodule Module do
 
   Accepts the function name (as an atom) of a function in the current module or
   `{function_name, 0}` tuple where `function_name` is the name of a function in
-  the current module. The function must be public and have an arity of 0 (no
-  arguments). If the function does not return `:ok`, the loading of the module
-  will be aborted. For example:
+  the current module. The function must have an arity of 0 (no arguments). If
+  the function does not return `:ok`, the loading of the module will be aborted.
+  For example:
 
       defmodule MyModule do
         @on_load :load_check
@@ -330,6 +335,16 @@ defmodule Module do
         @vsn "1.0"
       end
 
+  ### Struct attributes
+
+    * `@derive` - derives an implementation for the given protocol for the
+      struct defined in the current module
+
+    * `@enforce_keys` - ensures the given keys are always set when building
+      the struct defined in the current module
+
+  See `Kernel.defstruct/1` for more information on building and using structs.
+
   ### Typespec attributes
 
   The following attributes are part of typespecs and are also built-in in
@@ -345,6 +360,8 @@ defmodule Module do
       behaviour callbacks are optional
     * `@impl` - declares an implementation of a callback function or macro
 
+  For detailed documentation, see the [typespec documentation](typespecs.md).
+
   ### Custom attributes
 
   In addition to the built-in attributes outlined above, custom attributes may
@@ -499,8 +516,6 @@ defmodule Module do
       `@compile {:no_warn_undefined, {Mod, fun, arity}}` - does not warn if
       the given module or the given `Mod.fun/arity` are not defined
 
-  You can see a handful more options used by the Erlang compiler in
-  the documentation for the [`:compile` module](http://www.erlang.org/doc/man/compile.html).
   '''
 
   @typep definition :: {atom, arity}
@@ -538,6 +553,103 @@ defmodule Module do
   @callback __info__(:md5) :: binary()
   @callback __info__(:module) :: module()
 
+  @doc """
+  Returns information about module attributes used by Elixir.
+
+  See the "Module attributes" section in the module documentation for more
+  information on each attribute.
+
+  ## Examples
+
+      iex> map = Module.reserved_attributes()
+      iex> Map.has_key?(map, :moduledoc)
+      true
+      iex> Map.has_key?(map, :doc)
+      true
+
+  """
+  @doc since: "1.12.0"
+  def reserved_attributes() do
+    %{
+      after_compile: %{
+        doc: "A hook that will be invoked right after the current module is compiled."
+      },
+      before_compile: %{
+        doc: "A hook that will be invoked before the module is compiled."
+      },
+      behaviour: %{
+        doc: "Specifies that the current module implements a given behaviour."
+      },
+      on_definition: %{
+        doc:
+          "A hook that will be invoked when each function or macro in the current module is defined."
+      },
+      impl: %{
+        doc: "Declares an implementation of a callback function or macro."
+      },
+      compile: %{
+        doc: "Defines options for module compilation."
+      },
+      deprecated: %{
+        doc: "Provides the deprecation reason for a function."
+      },
+      moduledoc: %{
+        doc: "Provides documentation for the current module."
+      },
+      doc: %{
+        doc: "Provides documentation for a function/macro/callback."
+      },
+      typedoc: %{
+        doc: "Provides documentation for a type."
+      },
+      dialyzer: %{
+        doc: "Defines Dialyzer warnings to request or suppress."
+      },
+      external_resource: %{
+        doc: "Specifies an external resource for the current module."
+      },
+      file: %{
+        doc:
+          "Changes the filename used in stacktraces for the function or macro that follows the attribute."
+      },
+      on_load: %{
+        doc: "A hook that will be invoked whenever the module is loaded."
+      },
+      vsn: %{
+        doc: "Specify the module version."
+      },
+      type: %{
+        doc: "Defines a type to be used in `@spec`."
+      },
+      typep: %{
+        doc: "Defines a private type to be used in `@spec`."
+      },
+      opaque: %{
+        doc: "Defines an opaque type to be used in `@spec`."
+      },
+      spec: %{
+        doc: "Provides a specification for a function."
+      },
+      callback: %{
+        doc: "Provides a specification for a behaviour callback."
+      },
+      macrocallback: %{
+        doc: "Provides a specification for a macro behaviour callback."
+      },
+      optional_callbacks: %{
+        doc: "Specifies which behaviour callbacks and macro behaviour callbacks are optional."
+      },
+      derive: %{
+        doc:
+          "Derives an implementation for the given protocol for the struct defined in the current module."
+      },
+      enforce_keys: %{
+        doc:
+          "Ensures the given keys are always set when building the struct defined in the current module."
+      }
+    }
+  end
+
   @doc """
   Checks if a module is open.
 
@@ -625,7 +737,7 @@ defmodule Module do
 
   It returns a tuple of shape `{:module, module, binary, term}`
   where `module` is the module name, `binary` is the module
-  byte code and `term` is the result of the last expression in
+  bytecode and `term` is the result of the last expression in
   `quoted`.
 
   Similar to `Kernel.defmodule/2`, the binary will only be
@@ -719,9 +831,6 @@ defmodule Module do
 
   ## Examples
 
-      iex> Module.safe_concat([Module, Unknown])
-      ** (ArgumentError) argument error
-
       iex> Module.safe_concat([List, Chars])
       List.Chars
 
@@ -740,9 +849,6 @@ defmodule Module do
 
   ## Examples
 
-      iex> Module.safe_concat(Module, Unknown)
-      ** (ArgumentError) argument error
-
       iex> Module.safe_concat(List, Chars)
       List.Chars
 
@@ -1049,11 +1155,67 @@ defmodule Module do
 
   """
   @spec definitions_in(module, def_kind) :: [definition]
-  def definitions_in(module, def_kind)
-      when is_atom(module) and def_kind in [:def, :defp, :defmacro, :defmacrop] do
+  def definitions_in(module, kind)
+      when is_atom(module) and kind in [:def, :defp, :defmacro, :defmacrop] do
     assert_not_compiled!(__ENV__.function, module, @extra_error_msg_definitions_in)
     {set, _} = data_tables_for(module)
-    :lists.concat(:ets.match(set, {{:def, :"$1"}, def_kind, :_, :_, :_, :_}))
+    :ets.select(set, [{{{:def, :"$1"}, kind, :_, :_, :_, :_}, [], [:"$1"]}])
+  end
+
+  @doc """
+  Returns the definition for the given name-arity pair.
+
+  It returns a tuple with the `version`, the `kind`,
+  the definition `metadata`, and a list with each clause.
+  Each clause is a four-element tuple with metadata,
+  the arguments, the guards, and the clause AST.
+
+  The clauses are returned in the Elixir AST but a subset
+  that has already been expanded and normalized. This makes
+  it useful for analyzing code but it cannot be reinjected
+  into the module as it will have lost some of its original
+  context. Given this AST representation is mostly internal,
+  it is versioned and it may change at any time. Therefore,
+  **use this API with caution**.
+  """
+  @spec get_definition(module, definition) ::
+          {:v1, def_kind, meta :: keyword,
+           [{meta :: keyword, arguments :: [Macro.t()], guards :: [Macro.t()], Macro.t()}]}
+  @doc since: "1.12.0"
+  def get_definition(module, {name, arity})
+      when is_atom(module) and is_atom(name) and is_integer(arity) do
+    assert_not_compiled!(__ENV__.function, module, "")
+    {set, bag} = data_tables_for(module)
+
+    case :ets.lookup(set, {:def, {name, arity}}) do
+      [{_key, kind, meta, _, _, _}] ->
+        {:v1, kind, meta, bag_lookup_element(bag, {:clauses, {name, arity}}, 2)}
+
+      [] ->
+        nil
+    end
+  end
+
+  @doc """
+  Deletes a definition from a module.
+
+  It returns true if the definition exists and it was removed,
+  otherwise it returns false.
+  """
+  @doc since: "1.12.0"
+  @spec delete_definition(module, definition) :: boolean()
+  def delete_definition(module, {name, arity})
+      when is_atom(module) and is_atom(name) and is_integer(arity) do
+    assert_not_readonly!(__ENV__.function, module)
+
+    case :elixir_def.take_definition(module, {name, arity}) do
+      false ->
+        false
+
+      _ ->
+        :elixir_locals.yank({name, arity}, module)
+        true
+    end
   end
 
   @doc """
@@ -1397,7 +1559,7 @@ defmodule Module do
       if doc, do: {:error, :private_doc}, else: :ok
     else
       {set, _bag} = data_tables_for(module)
-      compile_doc(set, line, kind, name, arity, signature, nil, doc, %{}, __ENV__, false)
+      compile_doc(set, nil, line, kind, name, arity, signature, nil, doc, %{}, __ENV__, false)
       :ok
     end
   end
@@ -1410,16 +1572,17 @@ defmodule Module do
     {set, bag} = data_tables_for(module)
     {arity, defaults} = args_count(args, 0, 0)
 
-    impl = compile_impl(set, bag, name, env, kind, arity, defaults)
+    context = Keyword.get(:ets.lookup_element(set, {:def, {name, arity}}, 3), :context)
+    impl = compile_impl(set, bag, context, name, env, kind, arity, defaults)
     doc_meta = compile_doc_meta(set, bag, name, arity, defaults)
 
     {line, doc} = get_doc_info(set, env)
-    compile_doc(set, line, kind, name, arity, args, body, doc, doc_meta, env, impl)
+    compile_doc(set, context, line, kind, name, arity, args, body, doc, doc_meta, env, impl)
 
     :ok
   end
 
-  defp compile_doc(_table, line, kind, name, arity, _args, _body, doc, _doc_meta, env, _impl)
+  defp compile_doc(_table, _ctx, line, kind, name, arity, _args, _body, doc, _meta, env, _impl)
        when kind in [:defp, :defmacrop] do
     if doc do
       message =
@@ -1430,21 +1593,38 @@ defmodule Module do
     end
   end
 
-  defp compile_doc(table, line, kind, name, arity, args, _body, doc, doc_meta, env, impl) do
+  defp compile_doc(table, ctx, line, kind, name, arity, args, body, doc, doc_meta, env, impl) do
     key = {doc_key(kind), name, arity}
     signature = build_signature(args, env)
 
     case :ets.lookup(table, key) do
       [] ->
         doc = if is_nil(doc) && impl, do: false, else: doc
-        :ets.insert(table, {key, line, signature, doc, doc_meta})
+        :ets.insert(table, {key, ctx, line, signature, doc, doc_meta})
+
+      [{_, current_ctx, current_line, current_sign, current_doc, current_doc_meta}] ->
+        if is_binary(current_doc) and is_binary(doc) and body != nil and is_nil(current_ctx) do
+          message = ~s'''
+          redefining @doc attribute previously set at line #{current_line}.
+
+          Please remove the duplicate docs. If instead you want to override a \
+          previously defined @doc, attach the @doc attribute to a function head \
+          (the function signature not followed by any do-block). For example:
+
+              @doc """
+              new docs
+              """
+              def #{name}(...)
+          '''
+
+          IO.warn(message, Macro.Env.stacktrace(%{env | line: line}))
+        end
 
-      [{_, current_line, current_sign, current_doc, current_doc_meta}] ->
         signature = merge_signatures(current_sign, signature, 1)
         doc = if is_nil(doc), do: current_doc, else: doc
         doc = if is_nil(doc) && impl, do: false, else: doc
         doc_meta = Map.merge(current_doc_meta, doc_meta)
-        :ets.insert(table, {key, current_line, signature, doc, doc_meta})
+        :ets.insert(table, {key, ctx, current_line, signature, doc, doc_meta})
     end
   end
 
@@ -1492,14 +1672,12 @@ defmodule Module do
   defp deprecated_reason(name, arity, reason),
     do: {:deprecated, {{name, arity}, reason}}
 
-  defp compile_impl(set, bag, name, env, kind, arity, defaults) do
+  defp compile_impl(set, bag, context, name, env, kind, arity, defaults) do
     %{line: line, file: file} = env
 
     case :ets.take(set, :impl) do
       [{:impl, value, _}] ->
-        pair = {name, arity}
-        meta = :ets.lookup_element(set, {:def, pair}, 3)
-        impl = {pair, Keyword.get(meta, :context), defaults, kind, line, file, value}
+        impl = {{name, arity}, context, defaults, kind, line, file, value}
         :ets.insert(bag, {:impls, impl})
         value
 
@@ -1953,20 +2131,13 @@ defmodule Module do
   end
 
   defp preprocess_attribute(:impl, value) do
-    case value do
-      _ when is_boolean(value) ->
-        value
-
-      module when is_atom(module) and module != nil ->
-        # Attempt to compile behaviour but ignore failure (will warn later)
-        _ = Code.ensure_compiled(module)
-        value
-
-      _ ->
-        raise ArgumentError,
-              "@impl is a built-in module attribute that marks the next definition " <>
-                "as a callback implementation. It should be a module or a boolean, " <>
-                "got: #{inspect(value)}"
+    if is_boolean(value) or (is_atom(value) and value != nil) do
+      value
+    else
+      raise ArgumentError,
+            "@impl is a built-in module attribute that marks the next definition " <>
+              "as a callback implementation. It should be a module or a boolean, " <>
+              "got: #{inspect(value)}"
     end
   end
 
@@ -2014,10 +2185,46 @@ defmodule Module do
     end
   end
 
+  defp preprocess_attribute(:dialyzer, value) do
+    # From https://github.com/erlang/otp/blob/master/lib/stdlib/src/erl_lint.erl
+    :lists.foreach(
+      fn attr ->
+        if not valid_dialyzer_attribute?(attr) do
+          raise ArgumentError, "invalid value for @dialyzer attribute: #{inspect(attr)}"
+        end
+      end,
+      List.wrap(value)
+    )
+
+    value
+  end
+
   defp preprocess_attribute(_key, value) do
     value
   end
 
+  defp valid_dialyzer_attribute?({key, fun_arities}) when is_atom(key) do
+    (key == :nowarn_function or valid_dialyzer_attribute?(key)) and
+      :lists.all(
+        fn
+          {fun, arity} when is_atom(fun) and is_integer(arity) -> true
+          _ -> false
+        end,
+        List.wrap(fun_arities)
+      )
+  end
+
+  defp valid_dialyzer_attribute?(attr) do
+    :lists.member(
+      attr,
+      [:no_return, :no_unused, :no_improper_lists, :no_fun_app] ++
+        [:no_match, :no_opaque, :no_fail_call, :no_contracts] ++
+        [:no_behaviours, :no_undefined_callbacks, :unmatched_returns] ++
+        [:error_handling, :race_conditions, :no_missing_calls] ++
+        [:specdiffs, :overspecs, :underspecs, :unknown, :no_underspecs]
+    )
+  end
+
   defp preprocess_doc_meta([], _module, _line, map), do: map
 
   defp preprocess_doc_meta([{key, _} | tail], module, line, map)
