@@ -19,7 +19,7 @@ defmodule Task do
 
   Besides `async/1` and `await/2`, tasks can also be
   started as part of a supervision tree and dynamically spawned
-  on remote nodes. We will explore all three scenarios next.
+  on remote nodes. We will explore these scenarios next.
 
   ## async and await
 
@@ -43,78 +43,8 @@ defmodule Task do
        meant to receive the result no longer exists, there is
        no purpose in completing the computation.
 
-       If this is not desired, use `Task.start/1` or consider starting
-       the task under a `Task.Supervisor` using `async_nolink` or
-       `start_child`.
-
-  `Task.yield/2` is an alternative to `await/2` where the caller will
-  temporarily block, waiting until the task replies or crashes. If the
-  result does not arrive within the timeout, it can be called again at a
-  later moment. This allows checking for the result of a task multiple
-  times. If a reply does not arrive within the desired time,
-  `Task.shutdown/2` can be used to stop the task.
-
-  ## Supervised tasks
-
-  It is also possible to spawn a task under a supervisor. The `Task`
-  module implements the `child_spec/1` function, which allows it to
-  be started directly under a supervisor by passing a tuple with
-  a function to run:
-
-      Supervisor.start_link([
-        {Task, fn -> :some_work end}
-      ], strategy: :one_for_one)
-
-  However, if you want to invoke a specific module, function and
-  arguments, or give the task process a name, you need to define
-  the task in its own module:
-
-      defmodule MyTask do
-        use Task
-
-        def start_link(arg) do
-          Task.start_link(__MODULE__, :run, [arg])
-        end
-
-        def run(arg) do
-          # ...
-        end
-      end
-
-  And then passing it to the supervisor:
-
-      Supervisor.start_link([
-        {MyTask, arg}
-      ], strategy: :one_for_one)
-
-  Since these tasks are supervised and not directly linked to
-  the caller, they cannot be awaited on. `start_link/1`, unlike
-  `async/1`, returns `{:ok, pid}` (which is the result expected
-  by supervisors).
-
-  `use Task` defines a `child_spec/1` function, allowing the
-  defined module to be put under a supervision tree. The generated
-  `child_spec/1` can be customized with the following options:
-
-    * `:id` - the child specification identifier, defaults to the current module
-    * `:restart` - when the child should be restarted, defaults to `:temporary`
-    * `:shutdown` - how to shut down the child, either immediately or by giving it time to shut down
-
-  Opposite to `GenServer`, `Agent` and `Supervisor`, a Task has
-  a default `:restart` of `:temporary`. This means the task will
-  not be restarted even if it crashes. If you desire the task to
-  be restarted for non-successful exits, do:
-
-      use Task, restart: :transient
-
-  If you want the task to always be restarted:
-
-      use Task, restart: :permanent
-
-  See the "Child specification" section in the `Supervisor` module
-  for more detailed information. The `@doc` annotation immediately
-  preceding `use Task` will be attached to the generated `child_spec/1`
-  function.
+       If this is not desired, you will want to use supervised
+       tasks, described next.
 
   ## Dynamically supervised tasks
 
@@ -139,22 +69,36 @@ defmodule Task do
         {Task.Supervisor, name: MyApp.TaskSupervisor}
       ], strategy: :one_for_one)
 
-  Now you can dynamically start supervised tasks:
-
-      Task.Supervisor.start_child(MyApp.TaskSupervisor, fn ->
-        # Do something
-      end)
-
-  Or even use the async/await pattern:
+  And now you can use async/await once again passig the name of
+  the supervisor isntead of the pid:
 
       Task.Supervisor.async(MyApp.TaskSupervisor, fn ->
         # Do something
       end)
       |> Task.await()
 
-  Finally, check `Task.Supervisor` for other supported operations.
+  We encourage developers to rely on supervised tasks as much as
+  possible. Supervised tasks enable a huge variety of patterns
+  which allows you explicit control on how to handle the results,
+  errors, and timeouts. Here is a summary:
 
-  ## Distributed tasks
+    * Use `Task.Supervisor.start_child/2` to start a fire-and-forget
+      task and you don't care about its results nor about if it completes
+      successfully
+
+    * Use `Task.Supervisor.async/2` + `Task.await/2` allows you to execute
+      tasks concurrently and retrieve its result. If the task fails,
+      the caller will also fail
+
+    * Use `Task.Supervisor.async_nolink/2` + `Task.yield/2` + `Task.shutdown/2`
+      allows you to execute tasks concurrently and retrieve their results
+      or the reason they failed within a given time frame. If the task fails,
+      the caller won't fail: you will receive the error reason either on
+      `yield` or `shutdown`
+
+  See the `Task.Supervisor` module for details on the supported operations.
+
+  ### Distributed tasks
 
   Since Elixir provides a `Task.Supervisor`, it is easy to use one
   to dynamically start tasks across nodes:
@@ -166,12 +110,79 @@ defmodule Task do
       supervisor = {MyApp.DistSupervisor, :remote@local}
       Task.Supervisor.async(supervisor, MyMod, :my_fun, [arg1, arg2, arg3])
 
-  Note that, when working with distributed tasks, one should use the `Task.Supervisor.async/4` function
-  that expects explicit module, function and arguments, instead of `Task.Supervisor.async/2` that
-  works with anonymous functions. That's because anonymous functions expect
-  the same module version to exist on all involved nodes. Check the `Agent` module
-  documentation for more information on distributed processes as the limitations
-  described there apply to the whole ecosystem.
+  Note that, when working with distributed tasks, one should use the
+  `Task.Supervisor.async/4` function that expects explicit module, function,
+  and arguments, instead of `Task.Supervisor.async/2` that works with anonymous
+  functions. That's because anonymous functions expect the same module version
+  to exist on all involved nodes. Check the `Agent` module documentation for
+  more information on distributed processes as the limitations described there
+  apply to the whole ecosystem.
+
+  ## Statically supervised tasks
+
+  The `Task` module implements the `child_spec/1` function, which
+  allows it to be started directly under a regular `Supervisor` -
+  instead of a `Task.Supervisor` - by passing a tuple with a function
+  to run:
+
+      Supervisor.start_link([
+        {Task, fn -> :some_work end}
+      ], strategy: :one_for_one)
+
+  This is often useful when you need to execute some steps while
+  setting up your supervision tree. For example: to warm up caches,
+  log the initialization status, etc.
+
+  If you don't want to put the Task code directly under the `Supervisor`,
+  you can wrap the `Task` in its own module, similar to how you would
+  do with a `GenServer` or an `Agent`:
+
+      defmodule MyTask do
+        use Task
+
+        def start_link(arg) do
+          Task.start_link(__MODULE__, :run, [arg])
+        end
+
+        def run(arg) do
+          # ...
+        end
+      end
+
+  And then passing it to the supervisor:
+
+      Supervisor.start_link([
+        {MyTask, arg}
+      ], strategy: :one_for_one)
+
+  Since these tasks are supervised and not directly linked to the caller,
+  they cannot be awaited on. By default, the functions `Task.start`
+  and `Task.start_link` are for fire-and-forget tasks, where you don't
+  care about the results or if it completes successfully or not.
+
+  `use Task` defines a `child_spec/1` function, allowing the
+  defined module to be put under a supervision tree. The generated
+  `child_spec/1` can be customized with the following options:
+
+    * `:id` - the child specification identifier, defaults to the current module
+    * `:restart` - when the child should be restarted, defaults to `:temporary`
+    * `:shutdown` - how to shut down the child, either immediately or by giving it time to shut down
+
+  Opposite to `GenServer`, `Agent` and `Supervisor`, a Task has
+  a default `:restart` of `:temporary`. This means the task will
+  not be restarted even if it crashes. If you desire the task to
+  be restarted for non-successful exits, do:
+
+      use Task, restart: :transient
+
+  If you want the task to always be restarted:
+
+      use Task, restart: :permanent
+
+  See the "Child specification" section in the `Supervisor` module
+  for more detailed information. The `@doc` annotation immediately
+  preceding `use Task` will be attached to the generated `child_spec/1`
+  function.
 
   ## Ancestor and Caller Tracking
 
@@ -203,6 +214,9 @@ defmodule Task do
   a list `[pid_n, ..., pid2, pid1]` with at least one entry Where `pid_n` is
   the PID that called the current process, `pid2` called `pid_n`, and `pid2` was
   called by `pid1`.
+
+  If a task crashes, the callers field is included as part of the log message
+  metadata under the `:callers` key.
   """
 
   @doc """
@@ -284,11 +298,11 @@ defmodule Task do
   end
 
   @doc """
-  Starts a process linked to the current process.
+  Starts a task as part of a supervision tree with the given `fun`.
 
   `fun` must be a zero-arity anonymous function.
 
-  This is often used to start the process as part of a supervision tree.
+  This is used to start a statically supervised task under a supervision tree.
   """
   @spec start_link((() -> any)) :: {:ok, pid}
   def start_link(fun) when is_function(fun, 0) do
@@ -296,12 +310,15 @@ defmodule Task do
   end
 
   @doc """
-  Starts a task as part of a supervision tree.
+  Starts a task as part of a supervision tree with the given
+  `module`, `function`, and `args`.
+
+  This is used to start a statically supervised task under a supervision tree.
   """
   @spec start_link(module, atom, [term]) :: {:ok, pid}
-  def start_link(module, function_name, args)
-      when is_atom(module) and is_atom(function_name) and is_list(args) do
-    mfa = {module, function_name, args}
+  def start_link(module, function, args)
+      when is_atom(module) and is_atom(function) and is_list(args) do
+    mfa = {module, function, args}
     Task.Supervised.start_link(get_owner(self()), get_callers(self()), mfa)
   end
 
@@ -310,9 +327,14 @@ defmodule Task do
 
   `fun` must be a zero-arity anonymous function.
 
-  This is only used when the task is used for side-effects
-  (i.e. no interest in the returned result) and it should not
-  be linked to the current process.
+  This should only used when the task is used for side-effects
+  (like I/O) and you have no interest on its results nor if it
+  completes successfully.
+
+  If the current node is shutdown, the node will terminate even
+  if the task was not completed. For this reason, we recommend
+  to use `Task.Supervisor.start_child/2` instead, which allows
+  you to control the shutdown time via the `:shutdown` option.
   """
   @spec start((() -> any)) :: {:ok, pid}
   def start(fun) when is_function(fun, 0) do
@@ -322,9 +344,14 @@ defmodule Task do
   @doc """
   Starts a task.
 
-  This is only used when the task is used for side-effects
-  (i.e. no interest in the returned result) and it should not
-  be linked to the current process.
+  This should only used when the task is used for side-effects
+  (like I/O) and you have no interest on its results nor if it
+  completes successfully.
+
+  If the current node is shutdown, the node will terminate even
+  if the task was not completed. For this reason, we recommend
+  to use `Task.Supervisor.start_child/2` instead, which allows
+  you to control the shutdown time via the `:shutdown` option.
   """
   @spec start(module, atom, [term]) :: {:ok, pid}
   def start(module, function_name, args)
@@ -336,30 +363,14 @@ defmodule Task do
   @doc """
   Starts a task that must be awaited on.
 
-  `fun` must be a zero-arity anonymous function.
-  This function spawns a process that is linked to and monitored
-  by the caller process. A `Task` struct is returned containing
-  the relevant information.
-
-  Read the `Task` module documentation for more information about the
-  general usage of `async/1` and `async/3`.
-
-  See also `async/3`.
-  """
-  @spec async((() -> any)) :: t
-  def async(fun) when is_function(fun, 0) do
-    async(:erlang, :apply, [fun, []])
-  end
-
-  @doc """
-  Starts a task that must be awaited on.
-
-  A `Task` struct is returned containing the relevant information.
-  Developers must eventually call `Task.await/2` or `Task.yield/2`
-  followed by `Task.shutdown/2` on the returned task.
+  `fun` must be a zero-arity anonymous function. This function
+  spawns a process that is linked to and monitored by the caller
+  process. A `Task` struct is returned containing the relevant
+  information. Developers must eventually call `Task.await/2` or
+  `Task.yield/2` followed by `Task.shutdown/2` on the returned task.
 
   Read the `Task` module documentation for more information about
-  the general usage of `async/1` and `async/3`.
+  the general usage of async tasks.
 
   ## Linking
 
@@ -408,11 +419,17 @@ defmodule Task do
       to any supervisor, you may leave dangling tasks in case
       the parent dies.
 
-  ## Message format
+  """
+  @spec async((() -> any)) :: t
+  def async(fun) when is_function(fun, 0) do
+    async(:erlang, :apply, [fun, []])
+  end
 
-  The reply sent by the task will be in the format `{ref, result}`,
-  where `ref` is the monitor reference held by the task struct
-  and `result` is the return value of the task function.
+  @doc """
+  Starts a task that must be awaited on.
+
+  Similar to `async/1` except the function to be started is
+  specified by the given `module`, `function_name`, and `args`.
   """
   @spec async(module, atom, [term]) :: t
   def async(module, function_name, args)
@@ -453,13 +470,16 @@ defmodule Task do
       at the same time. Defaults to `System.schedulers_online/0`.
 
     * `:ordered` - whether the results should be returned in the same order
-      as the input stream. This option is useful when you have large
-      streams and don't want to buffer results before they are delivered.
-      This is also useful when you're using the tasks for side effects.
-      Defaults to `true`.
+      as the input stream. When the output is ordered, Elixir may need to
+      buffer results to emit them in the original order. Setting this option
+      to false disables the need to buffer at the cost of removing ordering.
+      This is also useful when you're using the tasks only for the side effects.
+      Note that regardless of what `:ordered` is set to, the tasks will
+      process asynchronously. If you need to process elements in order,
+      consider using `Enum.map/2` or `Enum.each/2` instead. Defaults to `true`.
 
-    * `:timeout` - the maximum amount of time (in milliseconds) each
-      task is allowed to execute for. Defaults to `5000`.
+    * `:timeout` - the maximum amount of time (in milliseconds or `:infinity`)
+      each task is allowed to execute for. Defaults to `5000`.
 
     * `:on_timeout` - what to do when a task times out. The possible
       values are:
@@ -488,6 +508,38 @@ defmodule Task do
       stream = Task.async_stream(collection, Mod, :expensive_fun, [], ordered: false)
       Stream.run(stream)
 
+  ## Attention: async + take
+
+  Given items in an async stream are processed concurrently, doing
+  `async_stream` followed by `Enum.take/2` may cause more items than
+  requested to be processed. Let's see an example:
+
+      1..100
+      |> Task.async_stream(fn i ->
+        Process.sleep(100)
+        IO.puts(to_string(i))
+      end)
+      |> Enum.take(10)
+
+  For a machine with 8 cores, the above will process 16 items instead
+  of 10. The reason is that `async_stream/5` always have 8 elements
+  processing at once. So by the time `Enum` says it got all elements
+  it needed, there are still 6 elements left to be processed.
+
+  The solution here is to use `Stream.take/2` instead of `Enum.take/2`
+  to filter elements before-hand:
+
+      1..100
+      |> Stream.take(10)
+      |> Task.async_stream(fn i ->
+        Process.sleep(100)
+        IO.puts(to_string(i))
+      end)
+      |> Enum.to_list()
+
+  If for some reason you cannot take the elements before hand,
+  you can use `:max_concurrency` to limit how many elements
+  may be over processed at the cost of reducing concurrency.
   """
   @doc since: "1.4.0"
   @spec async_stream(Enumerable.t(), module, atom, [term], keyword) :: Enumerable.t()
@@ -552,17 +604,18 @@ defmodule Task do
     end
   end
 
-  @doc """
+  @doc ~S"""
   Awaits a task reply and returns it.
 
   In case the task process dies, the current process will exit with the same
   reason as the task.
 
-  A timeout in milliseconds or `:infinity`, can be given with a default value of `5000`. If the
-  timeout is exceeded, then the current process will exit. If the task process
-  is linked to the current process which is the case when a task is started with
-  `async`, then the task process will also exit. If the task process is trapping
-  exits or not linked to the current process, then it will continue to run.
+  A timeout, in milliseconds or `:infinity`, can be given with a default value
+  of `5000`. If the timeout is exceeded, then the current process will exit. If
+  the task process is linked to the current process which is the case when a
+  task is started with `async`, then the task process will also exit. If the
+  task process is trapping exits or not linked to the current process, then it
+  will continue to run.
 
   This function assumes the task's monitor is still active or the monitor's
   `:DOWN` message is in the message queue. If it has been demonitored, or the
@@ -573,20 +626,86 @@ defmodule Task do
   to be able to check multiple times if a long-running task has finished
   its computation, use `yield/2` instead.
 
-  ## Compatibility with OTP behaviours
-
-  It is not recommended to `await` a long-running task inside an OTP
-  behaviour such as `GenServer`. Instead, you should match on the message
-  coming from a task inside your `c:GenServer.handle_info/2` callback. For
-  more information on the format of the message, see the documentation for
-  `async/1`.
-
   ## Examples
 
       iex> task = Task.async(fn -> 1 + 1 end)
       iex> Task.await(task)
       2
 
+  ## Compatibility with OTP behaviours
+
+  It is not recommended to `await` a long-running task inside an OTP
+  behaviour such as `GenServer`. Instead, you should match on the message
+  coming from a task inside your `c:GenServer.handle_info/2` callback.
+
+  A GenServer will receive two messages on `handle_info/2`:
+
+    * `{ref, result}` - the reply message where `ref` is the monitor
+      reference returned by the `task.ref` and `result` is the task
+      result
+
+    * `{:DOWN, ref, :process, pid, reason}` - since all tasks are also
+      monitored, you will also receive the `:DOWN` message delivered by
+      `Process.monitor/1`. If you receive the `:DOWN` message without a
+      a reply, it means the task crashed
+
+  Another consideration to have in mind is that tasks started by `Task.async/1`
+  are always linked to their callers and you may not want the GenServer to
+  crash if the task crashes. Therefore, it is preferable to instead use
+  `Task.Supervisor.async_nolink/3` inside OTP behaviours. For completeness, here
+  is an example of a GenServer that start tasks and handles their results:
+
+      defmodule GenServerTaskExample do
+        use GenServer
+
+        def start_link(opts) do
+          GenServer.start_link(__MODULE__, :ok, opts)
+        end
+
+        def init(_opts) do
+          # We will keep all running tasks in a map
+          {:ok, %{tasks: %{}}}
+        end
+
+        # Imagine we invoke a task from the GenServer to access a URL...
+        def handle_call(:some_message, _from, state) do
+          url = ...
+          task = Task.Supervisor.async_nolink(MyApp.TaskSupervisor, fn -> fetch_url(url) end)
+
+          # After we start the task, we store its reference and the url it is fetching
+          state = put_in(state.tasks[task.ref], url)
+
+          {:reply, :ok, state}
+        end
+
+        # If the task succeeds...
+        def handle_info({ref, result}, state) do
+          # The task succeed so we can cancel the monitoring and discard the DOWN message
+          Process.demonitor(ref, [:flush])
+
+          {url, state} = pop_in(state.tasks[ref])
+          IO.puts "Got #{inspect(result)} for URL #{inspect url}"
+          {:noreply, state}
+        end
+
+        # If the task fails...
+        def handle_info({:DOWN, ref, _, _, reason}, state) do
+          {url, state} = pop_in(state.tasks[ref])
+          IO.puts "URL #{inspect url} failed with reason #{inspect(reason)}"
+          {:noreply, state}
+        end
+      end
+
+  With the server defined, you will want to start the task supervisor
+  above and the GenServer in your supervision tree:
+
+      children = [
+        {Task.Supervisor, name: MyApp.TaskSupervisor},
+        {GenServerTaskExample, name: MyApp.GenServerTaskExample}
+      ]
+
+      Supervisor.start_link(children, strategy: :one_for_one)
+
   """
   @spec await(t, timeout) :: term
   def await(%Task{ref: ref, owner: owner} = task, timeout \\ 5000) when is_timeout(timeout) do
@@ -608,6 +727,109 @@ defmodule Task do
     end
   end
 
+  @doc """
+  Awaits replies from multiple tasks and returns them.
+
+  This function receives a list of tasks and waits for their replies in the
+  given time interval. It returns a list of the results, in the same order as
+  the tasks supplied in the `tasks` input argument.
+
+  If any of the task processes dies, the current process will exit with the
+  same reason as that task.
+
+  A timeout, in milliseconds or `:infinity`, can be given with a default value
+  of `5000`. If the timeout is exceeded, then the current process will exit.
+  Any task processes that are linked to the current process (which is the case
+  when a task is started with `async`) will also exit. Any task processes that
+  are trapping exits or not linked to the current process will continue to run.
+
+  This function assumes the tasks' monitors are still active or the monitors'
+  `:DOWN` message is in the message queue. If any tasks have been demonitored,
+  or the message already received, this function will wait for the duration of
+  the timeout.
+
+  This function can only be called once for any given task. If you want to be
+  able to check multiple times if a long-running task has finished its
+  computation, use `yield_many/2` instead.
+
+  ## Compatibility with OTP behaviours
+
+  It is not recommended to `await` long-running tasks inside an OTP behaviour
+  such as `GenServer`. See `await/2` for more information.
+
+  ## Examples
+
+      iex> tasks = [
+      ...>   Task.async(fn -> 1 + 1 end),
+      ...>   Task.async(fn -> 2 + 3 end)
+      ...> ]
+      iex> Task.await_many(tasks)
+      [2, 5]
+
+  """
+  @doc since: "1.11.0"
+  @spec await_many([t], timeout) :: [term]
+  def await_many(tasks, timeout \\ 5000) when is_timeout(timeout) do
+    awaiting =
+      for task <- tasks, into: %{} do
+        %Task{ref: ref, owner: owner} = task
+
+        if owner != self() do
+          raise ArgumentError, invalid_owner_error(task)
+        end
+
+        {ref, true}
+      end
+
+    timeout_ref = make_ref()
+
+    timer_ref =
+      if timeout != :infinity do
+        Process.send_after(self(), timeout_ref, timeout)
+      end
+
+    try do
+      await_many(tasks, timeout, awaiting, %{}, timeout_ref)
+    after
+      timer_ref && Process.cancel_timer(timer_ref)
+      receive do: (^timeout_ref -> :ok), after: (0 -> :ok)
+    end
+  end
+
+  defp await_many(tasks, _timeout, awaiting, replies, _timeout_ref)
+       when map_size(awaiting) == 0 do
+    for %{ref: ref} <- tasks, do: Map.fetch!(replies, ref)
+  end
+
+  defp await_many(tasks, timeout, awaiting, replies, timeout_ref) do
+    receive do
+      ^timeout_ref ->
+        demonitor_pending_tasks(awaiting)
+        exit({:timeout, {__MODULE__, :await_many, [tasks, timeout]}})
+
+      {:DOWN, ref, _, proc, reason} when is_map_key(awaiting, ref) ->
+        demonitor_pending_tasks(awaiting)
+        exit({reason(reason, proc), {__MODULE__, :await_many, [tasks, timeout]}})
+
+      {ref, reply} when is_map_key(awaiting, ref) ->
+        Process.demonitor(ref, [:flush])
+
+        await_many(
+          tasks,
+          timeout,
+          Map.delete(awaiting, ref),
+          Map.put(replies, ref, reply),
+          timeout_ref
+        )
+    end
+  end
+
+  defp demonitor_pending_tasks(awaiting) do
+    Enum.each(awaiting, fn {ref, _} ->
+      Process.demonitor(ref, [:flush])
+    end)
+  end
+
   @doc false
   @deprecated "Pattern match directly on the message instead"
   def find(tasks, {ref, reply}) when is_reference(ref) do
@@ -647,10 +869,9 @@ defmodule Task do
     * the caller is trapping exits
 
   A timeout, in milliseconds or `:infinity`, can be given with a default value
-  of `5000`. If the time runs out before a message from
-  the task is received, this function will return `nil`
-  and the monitor will remain active. Therefore `yield/2` can be
-  called multiple times on the same task.
+  of `5000`. If the time runs out before a message from the task is received,
+  this function will return `nil` and the monitor will remain active. Therefore
+  `yield/2` can be called multiple times on the same task.
 
   This function assumes the task's monitor is still active or the
   monitor's `:DOWN` message is in the message queue. If it has been
