@@ -59,11 +59,7 @@ fetch_definitions(File, Module) ->
     error:badarg -> []
   end,
 
-  {All, Private} = fetch_definition(Entries, File, Module, Set, Bag, [], []),
-  Unreachable = elixir_locals:warn_unused_local(File, Module, All, Private),
-  elixir_locals:ensure_no_undefined_local(File, Module, All),
-  elixir_locals:ensure_no_import_conflict(File, Module, All),
-  {All, Unreachable}.
+  fetch_definition(Entries, File, Module, Set, Bag, [], []).
 
 fetch_definition([Tuple | T], File, Module, Set, Bag, All, Private) ->
   [{_, Kind, Meta, _, Check, {MaxDefaults, _, Defaults}}] = ets:lookup(Set, {def, Tuple}),
@@ -83,7 +79,7 @@ fetch_definition([Tuple | T], File, Module, Set, Bag, All, Private) ->
       fetch_definition(T, File, Module, Set, Bag, NewAll, NewPrivate)
   catch
     error:badarg ->
-      check_bodiless_function(Check, Meta, File, Module, Kind, Tuple),
+      elixir_errors:form_error(Meta, File, ?MODULE, {function_head, Kind, Tuple}),
       fetch_definition(T, File, Module, Set, Bag, All, Private)
   end;
 
@@ -246,7 +242,7 @@ store_definition(Check, Kind, Meta, Name, Arity, File, Module, Defaults, Clauses
   {MaxDefaults, FirstMeta} =
     case ets:lookup(Set, {def, Tuple}) of
       [{_, StoredKind, StoredMeta, StoredFile, StoredCheck, {StoredDefaults, LastHasBody, LastDefaults}}] ->
-        check_valid_kind(Meta, File, Name, Arity, Kind, StoredKind),
+        check_valid_kind(Meta, File, Name, Arity, Kind, StoredKind, StoredFile, StoredMeta),
         check_valid_defaults(Meta, File, Name, Arity, Kind, Defaults, StoredDefaults, LastDefaults, HasBody, LastHasBody),
         (Check and StoredCheck) andalso
           check_valid_clause(Meta, File, Name, Arity, Kind, Set, StoredMeta, StoredFile, Clauses),
@@ -304,10 +300,10 @@ default_var(Counter) ->
 
 %% Validations
 
-check_valid_kind(_Meta, _File, _Name, _Arity, Kind, Kind) -> ok;
-check_valid_kind(Meta, File, Name, Arity, Kind, StoredKind) ->
+check_valid_kind(_Meta, _File, _Name, _Arity, Kind, Kind, _StoredFile, _StoredMeta) -> ok;
+check_valid_kind(Meta, File, Name, Arity, Kind, StoredKind, StoredFile, StoredMeta) ->
   elixir_errors:form_error(Meta, File, ?MODULE,
-    {changed_kind, {Name, Arity, StoredKind, Kind}}).
+    {changed_kind, {Name, Arity, StoredKind, Kind, StoredFile, ?line(StoredMeta)}}).
 
 check_valid_clause(Meta, File, Name, Arity, Kind, Set, StoredMeta, StoredFile, Clauses) ->
   case ets:lookup_element(Set, ?last_def, 2) of
@@ -342,12 +338,6 @@ check_valid_defaults(Meta, File, Name, Arity, Kind, 0, _, LastDefaults, true, tr
 check_valid_defaults(_Meta, _File, _Name, _Arity, _Kind, 0, _StoredDefaults, _LastDefaults, _HasBody, _LastHasBody) ->
   ok.
 
-check_bodiless_function(Check, _Meta, _File, Module, _Kind, _Tuple)
-    when Check == false; Module == 'Elixir.Module' ->
-  ok;
-check_bodiless_function(_Check, Meta, File, _Module, Kind, Tuple) ->
-  elixir_errors:form_error(Meta, File, ?MODULE, {function_head, Kind, Tuple}).
-
 check_args_for_function_head(Meta, Args, E) ->
   [begin
      elixir_errors:form_error(Meta, E, ?MODULE, invalid_args_for_function_head)
@@ -450,8 +440,9 @@ format_error({late_function_head, {Kind, Name, Arity}}) ->
     "    def add(a, b), do: a + b\n",
     [Kind, Name, Arity]);
 
-format_error({changed_kind, {Name, Arity, Previous, Current}}) ->
-  io_lib:format("~ts ~ts/~B already defined as ~ts", [Current, Name, Arity, Previous]);
+format_error({changed_kind, {Name, Arity, Previous, Current, OrigFile, OrigLine}}) ->
+  OrigFileRelative = elixir_utils:relative_to_cwd(OrigFile),
+  io_lib:format("~ts ~ts/~B already defined as ~ts in ~ts:~B", [Current, Name, Arity, Previous, OrigFileRelative, OrigLine]);
 
 format_error({no_alias, Atom}) ->
   io_lib:format("function names should start with lowercase characters or underscore, invalid name ~ts", [Atom]);
