@@ -1,7 +1,7 @@
 % Handle string and string-like interpolations.
 -module(elixir_interpolation).
--export([extract/6, unescape_chars/1, unescape_chars/2,
-unescape_tokens/1, unescape_tokens/2, unescape_map/1]).
+-export([extract/6, unescape_string/1, unescape_string/2,
+unescape_tokens/1, unescape_map/1]).
 -include("elixir.hrl").
 -define(is_hex(S), ((S >= $0 andalso S =< $9) orelse
                     (S >= $A andalso S =< $F) orelse
@@ -16,35 +16,22 @@ extract(Line, Column, Raw, Interpol, String, Last) ->
 
 %% Terminators
 
-extract(Line, Column, _Scope, _Interpol, [], Buffer, Output, none) ->
-  finish_extraction(Line, Column, Buffer, Output, []);
-
 extract(Line, Column, _Scope, _Interpol, [], _Buffer, _Output, Last) ->
   {error, {string, Line, Column, io_lib:format("missing terminator: ~ts", [[Last]]), []}};
 
-extract(Line, Column, _Scope, _Interpol, [Last | Remaining], Buffer, Output, Last) ->
-  finish_extraction(Line, Column + 1, Buffer, Output, Remaining);
+extract(Line, Column, _Scope, _Interpol, [Last | Rest], Buffer, Output, Last) ->
+  finish_extraction(Line, Column + 1, Buffer, Output, Rest);
 
 %% Going through the string
 
-extract(Line, _Column, Scope, true, [$\\, $\n | Rest], Buffer, Output, Last) ->
-  NewBuffer =
-    case Scope#elixir_tokenizer.unescape of
-      true -> Buffer;
-      false -> [$\n, $\\ |  Buffer]
-    end,
-  extract(Line+1, 1, Scope, true, Rest, NewBuffer, Output, Last);
+extract(Line, _Column, Scope, Interpol, [$\\, $\r, $\n | Rest], Buffer, Output, Last) ->
+  extract_nl(Line, Scope, Interpol, Rest, [$\n, $\r, $\\ | Buffer], Output, Last);
 
-extract(Line, _Column, Scope, true, [$\\, $\r, $\n | Rest], Buffer, Output, Last) ->
-  NewBuffer =
-    case Scope#elixir_tokenizer.unescape of
-      true -> Buffer;
-      false -> [$\n, $\r, $\\ |  Buffer]
-    end,
-  extract(Line+1, 1, Scope, true, Rest, NewBuffer, Output, Last);
+extract(Line, _Column, Scope, Interpol, [$\\, $\n | Rest], Buffer, Output, Last) ->
+  extract_nl(Line, Scope, Interpol, Rest, [$\n, $\\ | Buffer], Output, Last);
 
 extract(Line, _Column, Scope, Interpol, [$\n | Rest], Buffer, Output, Last) ->
-  extract(Line+1, 1, Scope, Interpol, Rest, [$\n | Buffer], Output, Last);
+  extract_nl(Line, Scope, Interpol, Rest, [$\n | Buffer], Output, Last);
 
 extract(Line, Column, Scope, Interpol, [$\\, Last | Rest], Buffer, Output, Last) ->
   extract(Line, Column+2, Scope, Interpol, Rest, [Last | Buffer], Output, Last);
@@ -72,16 +59,30 @@ extract(Line, Column, Scope, Interpol, [$\\, Char | Rest], Buffer, Output, Last)
 extract(Line, Column, Scope, Interpol, [Char | Rest], Buffer, Output, Last) ->
   extract(Line, Column + 1, Scope, Interpol, Rest, [Char | Buffer], Output, Last).
 
+%% Handle newlines. Heredocs require special attention
+
+extract_nl(Line, Scope, Interpol, Rest, Buffer, Output, [H,H,H] = Last) ->
+  case strip_horizontal_space(Rest, Buffer, 1) of
+    {[H,H,H|NewRest], _NewBuffer, Column} ->
+      finish_extraction(Line + 1, Column + 3, Buffer, Output, NewRest);
+    {NewRest, NewBuffer, Column} ->
+      extract(Line + 1, Column, Scope, Interpol, NewRest, NewBuffer, Output, Last)
+  end;
+extract_nl(Line, Scope, Interpol, Rest, Buffer, Output, Last) ->
+  extract(Line + 1, 1, Scope, Interpol, Rest, Buffer, Output, Last).
+
+strip_horizontal_space([H | T], Buffer, Counter) when H =:= $\s; H =:= $\t ->
+  strip_horizontal_space(T, [H | Buffer], Counter + 1);
+strip_horizontal_space(T, Buffer, Counter) ->
+  {T, Buffer, Counter}.
+
 %% Unescape a series of tokens as returned by extract.
 
 unescape_tokens(Tokens) ->
-  unescape_tokens(Tokens, fun unescape_map/1).
-
-unescape_tokens(Tokens, Map) ->
-  try [unescape_token(Token, Map) || Token <- Tokens] of
+  try [unescape_token(Token, fun unescape_map/1) || Token <- Tokens] of
     Unescaped -> {ok, Unescaped}
   catch
-    {error, _Reason} = Error -> Error
+    {error, _Reason, _Token} = Error -> Error
   end.
 
 unescape_token(Token, Map) when is_list(Token) ->
@@ -91,10 +92,21 @@ unescape_token(Token, Map) when is_binary(Token) ->
 unescape_token(Other, _Map) ->
   Other.
 
-% Unescape chars. For instance, "\" "n" (two chars) needs to be converted to "\n" (one char).
+% Unescape string. This is called by Elixir. Wrapped by convenience.
 
-unescape_chars(String) ->
-  unescape_chars(String, fun unescape_map/1).
+unescape_string(String) ->
+  unescape_string(String, fun unescape_map/1).
+
+unescape_string(String, Map) ->
+  try
+    unescape_chars(String, Map)
+  catch
+    {error, Reason, _} ->
+      Message = elixir_utils:characters_to_binary(Reason),
+      error('Elixir.ArgumentError':exception([{message, Message}]))
+  end.
+
+% Unescape chars. For instance, "\" "n" (two chars) needs to be converted to "\n" (one char).
 
 unescape_chars(String, Map) ->
   unescape_chars(String, Map, <<>>).
@@ -111,6 +123,18 @@ unescape_chars(<<$\\, $u, Rest/binary>>, Map, Acc) ->
     false -> unescape_chars(Rest, Map, <<Acc/binary, $\\, $u>>)
   end;
 
+unescape_chars(<<$\\, $\n, Rest/binary>>, Map, Acc) ->
+  case Map(newline) of
+    true  -> unescape_chars(Rest, Map, Acc);
+    false -> unescape_chars(Rest, Map, <<Acc/binary, $\\, $\n>>)
+  end;
+
+unescape_chars(<<$\\, $\r, $\n, Rest/binary>>, Map, Acc) ->
+  case Map(newline) of
+    true  -> unescape_chars(Rest, Map, Acc);
+    false -> unescape_chars(Rest, Map, <<Acc/binary, $\\, $\r, $\n>>)
+  end;
+
 unescape_chars(<<$\\, Escaped, Rest/binary>>, Map, Acc) ->
   case Map(Escaped) of
     false -> unescape_chars(Rest, Map, <<Acc/binary, $\\, Escaped>>);
@@ -159,7 +183,7 @@ unescape_hex(<<${, A, B, C, D, E, F, $}, Rest/binary>>, Map, Acc) when ?is_hex(A
   append_codepoint(Rest, Map, [A, B, C, D, E, F], Acc, 16);
 
 unescape_hex(<<_/binary>>, _Map, _Acc) ->
-  throw({error, "missing hex sequence after \\x, expected \\xHH"}).
+  throw({error, "invalid hex escape character, expected \\xHH where H is a hexadecimal digit", "\\x"}).
 
 %% Finish deprecated sequences
 
@@ -185,8 +209,7 @@ unescape_unicode(<<${, A, B, C, D, E, F, $}, Rest/binary>>, Map, Acc) when ?is_h
   append_codepoint(Rest, Map, [A, B, C, D, E, F], Acc, 16);
 
 unescape_unicode(<<_/binary>>, _Map, _Acc) ->
-  Msg = <<"invalid Unicode sequence after \\u, expected \\uHHHH or \\u{H*}">>,
-  error('Elixir.ArgumentError':exception([{message, Msg}])).
+  throw({error, "invalid Unicode escape character, expected \\uHHHH or \\u{H*} where H is a hexadecimal digit", "\\u"}).
 
 append_codepoint(Rest, Map, List, Acc, Base) ->
   Codepoint = list_to_integer(List, Base),
@@ -194,10 +217,10 @@ append_codepoint(Rest, Map, List, Acc, Base) ->
     Binary -> unescape_chars(Rest, Map, Binary)
   catch
     error:badarg ->
-      Msg = <<"invalid or reserved Unicode code point ", (integer_to_binary(Codepoint))/binary>>,
-      error('Elixir.ArgumentError':exception([{message, Msg}]))
+      throw({error, "invalid or reserved Unicode code point \\u{" ++ List ++ "}", "\\u"})
   end.
 
+unescape_map(newline) -> true;
 unescape_map(unicode) -> true;
 unescape_map(hex) -> true;
 unescape_map($0) -> 0;
