@@ -121,13 +121,13 @@ format_error({function_conflict, {Receiver, {Name, Arity}}}) ->
     [elixir_aliases:inspect(Receiver), Name, Arity]);
 
 format_error({unused_args, {Name, Arity}}) ->
-  io_lib:format("default arguments in ~ts/~B are never used", [Name, Arity]);
+  io_lib:format("default values for the optional arguments in ~ts/~B are never used", [Name, Arity]);
 
 format_error({unused_args, {Name, Arity}, 1}) ->
-  io_lib:format("the first default argument in ~ts/~B is never used", [Name, Arity]);
+  io_lib:format("the default value for the first optional argument in ~ts/~B is never used", [Name, Arity]);
 
 format_error({unused_args, {Name, Arity}, Count}) ->
-  io_lib:format("the first ~B default arguments in ~ts/~B are never used", [Count, Name, Arity]);
+  io_lib:format("the default values for the first ~B optional arguments in ~ts/~B are never used", [Count, Name, Arity]);
 
 format_error({unused_def, {Name, Arity}, defp}) ->
   io_lib:format("function ~ts/~B is unused", [Name, Arity]);
@@ -135,8 +135,16 @@ format_error({unused_def, {Name, Arity}, defp}) ->
 format_error({unused_def, {Name, Arity}, defmacrop}) ->
   io_lib:format("macro ~ts/~B is unused", [Name, Arity]);
 
+format_error({undefined_function, {F, A}})
+  when F == '__info__', A == 1;
+       F == 'behaviour_info', A == 1;
+       F == 'module_info', A == 1;
+       F == 'module_info', A == 0 ->
+  io_lib:format("undefined function ~ts/~B (this function is auto-generated by the compiler "
+                "and must always be called as a remote, as in __MODULE__.~ts/~B)", [F, A, F, A]);
+
 format_error({undefined_function, {F, A}}) ->
   io_lib:format("undefined function ~ts/~B", [F, A]);
 
 format_error({incorrect_dispatch, {F, A}}) ->
-  io_lib:format("cannot invoke macro ~ts/~B before its definition", [F, A]).
\ No newline at end of file
+  io_lib:format("cannot invoke macro ~ts/~B before its definition", [F, A]).
