@@ -309,6 +309,14 @@ defmodule Code.Formatter.IntegrationTest do
     """
   end
 
+  test "do/end inside binary" do
+    assert_same """
+    <<if true do
+        "hello"
+      end::binary>>
+    """
+  end
+
   test "anonymous function with parens around integer argument" do
     bad = """
     fn (1) -> "hello" end
@@ -515,6 +523,14 @@ defmodule Code.Formatter.IntegrationTest do
     "this works" ++ (&String.upcase/1) ++ &String.downcase/1
     """
 
+    bad = """
+    "this works" +++ (&String.upcase/1) +++ (&String.downcase/1)
+    """
+
+    assert_format bad, """
+    "this works" +++ (&String.upcase/1) +++ &String.downcase/1
+    """
+
     bad = """
     "this works" | (&String.upcase/1) | (&String.downcase/1)
     """
@@ -531,4 +547,77 @@ defmodule Code.Formatter.IntegrationTest do
     "this works" \\ &String.upcase/1 \\ &String.downcase/1
     """
   end
+
+  test "multiline expression inside interpolation" do
+    bad = ~S"""
+    Logger.info("Example: #{
+      inspect(%{
+        a: 1,
+        b: 2
+      })
+    }")
+    """
+
+    assert_format bad, ~S"""
+    Logger.info("Example: #{inspect(%{a: 1, b: 2})}")
+    """
+  end
+
+  test "comment inside operator with when" do
+    bad = """
+    raise function(x) ::
+            # Comment
+            any
+    """
+
+    assert_format bad, """
+    # Comment
+    raise function(x) ::
+            any
+    """
+
+    bad = """
+    raise function(x) ::
+            # Comment
+            any
+          when x: any
+    """
+
+    assert_format bad, """
+    raise function(x) ::
+            any
+          # Comment
+          when x: any
+    """
+
+    bad = """
+    @spec function(x) ::
+            # Comment
+            any
+          when x: any
+    """
+
+    assert_format bad, """
+    @spec function(x) ::
+            any
+          # Comment
+          when x: any
+    """
+
+    bad = """
+    @spec function(x) ::
+            # Comment
+            any
+          when x
+          when y
+    """
+
+    assert_format bad, """
+    @spec function(x) ::
+            any
+          # Comment
+          when x
+          when y
+    """
+  end
 end
