@@ -110,6 +110,17 @@ defmodule Code.Formatter.OperatorsTest do
     end
   end
 
+  describe "ternary without space" do
+    test "formats without spaces" do
+      assert_format "1 .. 2 // 3", "1..2//3"
+      assert_same "(1..2//3).step"
+    end
+
+    test "never breaks" do
+      assert_same "123_456_789..987_654_321//147_268_369", @short_length
+    end
+  end
+
   describe "binary without newline" do
     test "formats without spaces" do
       assert_same "1 in 2"
@@ -121,8 +132,7 @@ defmodule Code.Formatter.OperatorsTest do
     end
 
     test "not in" do
-      assert_format "not(foo in bar)", "not (foo in bar)"
-      assert_format "not(foo in bar)", "foo not in bar", rename_deprecated_at: "1.5.0"
+      assert_format "not(foo in bar)", "foo not in bar"
 
       assert_same "foo not in bar"
       assert_same "(not foo) in bar"
@@ -130,8 +140,8 @@ defmodule Code.Formatter.OperatorsTest do
     end
 
     test "bitwise precedence" do
-      assert_format "(crc ^^^ byte) &&& 0xFF", "crc ^^^ byte &&& 0xFF"
-      assert_same "(crc >>> 8) ^^^ byte"
+      assert_format "(crc >>> 8) ||| byte", "crc >>> 8 ||| byte"
+      assert_same "crc >>> (8 ||| byte)"
     end
   end
 
@@ -245,6 +255,8 @@ defmodule Code.Formatter.OperatorsTest do
 
     test "with multiple of the different entry, same precedence and right associative" do
       assert_format "foo ++ bar ++ baz -- bat", "foo ++ bar ++ (baz -- bat)"
+
+      assert_format "foo +++ bar +++ baz --- bat", "foo +++ bar +++ (baz --- bat)"
     end
 
     test "preserves user choice even when it fits" do
@@ -277,6 +289,8 @@ defmodule Code.Formatter.OperatorsTest do
   describe "binary with following new line" do
     test "formats with spaces" do
       assert_format "1++2", "1 ++ 2"
+
+      assert_format "1+++2", "1 +++ 2"
     end
 
     test "breaks into new line" do
@@ -892,6 +906,7 @@ defmodule Code.Formatter.OperatorsTest do
       assert_same "&and/2"
       assert_same "& &&/2"
       assert_same "& &/1"
+      assert_same "&//2"
     end
 
     test "Module.remote/arity" do
