@@ -63,6 +63,10 @@ defmodule Kernel.DocsTest do
     assert Code.fetch_docs(InMemoryDocs) == {:error, :module_not_found}
   end
 
+  test "non-existent beam file" do
+    assert {:error, :module_not_found} = Code.fetch_docs("bad.beam")
+  end
+
   test "raises on invalid @doc since: ..." do
     assert_raise ArgumentError, ~r"should be a string representing the version", fn ->
       defmodule InvalidSince do
@@ -250,7 +254,7 @@ defmodule Kernel.DocsTest do
       ] = Enum.sort(docs)
 
       assert {{:callback, :bar, 0}, _, [], :hidden, %{}} = callback_bar
-      assert {{:callback, :baz, 2}, _, [], :none, %{}} = callback_baz
+      assert {{:callback, :baz, 2}, _, [], %{}, %{}} = callback_baz
 
       assert {{:callback, :foo, 1}, _, [], %{"en" => "Callback doc"},
               %{since: "1.2.3", deprecated: "use baz/2 instead", color: :blue, stable: true}} =
@@ -300,6 +304,29 @@ defmodule Kernel.DocsTest do
     end
   end
 
+  test "fetch docs chunk from doc/chunks" do
+    Code.compiler_options(docs: false)
+
+    doc_chunks_path = Path.join([tmp_path(), "doc", "chunks"])
+    File.rm_rf!(doc_chunks_path)
+    File.mkdir_p!(doc_chunks_path)
+
+    write_beam(
+      defmodule ExternalDocs do
+      end
+    )
+
+    assert Code.fetch_docs(ExternalDocs) == {:error, :chunk_not_found}
+
+    path = Path.join([doc_chunks_path, "#{ExternalDocs}.chunk"])
+    chunk = {:docs_v1, 1, :elixir, "text/markdown", %{"en" => "Some docs"}, %{}}
+    File.write!(path, :erlang.term_to_binary(chunk))
+
+    assert Code.fetch_docs(ExternalDocs) == chunk
+  after
+    Code.compiler_options(docs: true)
+  end
+
   test "@impl true doesn't set @doc false if previous implementation has docs" do
     write_beam(
       defmodule Docs do
@@ -335,7 +362,7 @@ defmodule Kernel.DocsTest do
              {{:bar, 0}, :hidden},
              {{:baz, 0}, %{"en" => "Baz docs"}},
              {{:foo, 1}, %{"en" => "Foo docs"}},
-             {{:fuz, 0}, :none}
+             {{:fuz, 0}, %{}}
            ] = Enum.sort(function_docs)
   end
 
