@@ -310,6 +310,38 @@ defmodule Kernel.GuardTest do
         end
       end
 
+      assert_raise CompileError,
+                   ~r"cannot invoke remote function :erlang\.is_record/2 inside guards",
+                   fn ->
+                     defmodule IsRecord2Usage do
+                       defguard foo(rec) when :erlang.is_record(rec, :tag)
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"cannot invoke remote function :erlang\.is_record/3 inside guards",
+                   fn ->
+                     defmodule IsRecord3Usage do
+                       defguard foo(rec) when :erlang.is_record(rec, :tag, 7)
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"cannot invoke remote function :erlang\.\+\+/2 inside guards",
+                   fn ->
+                     defmodule ListSubtractionUsage do
+                       defguard foo(list) when list ++ []
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"cannot invoke remote function :erlang\.\-\-/2 inside guards",
+                   fn ->
+                     defmodule ListSubtractionUsage do
+                       defguard foo(list) when list -- []
+                     end
+                   end
+
       assert_raise CompileError, ~r"invalid expression in guard", fn ->
         defmodule LocalCallUsage do
           defguard foo(local, call) when local.(call)
@@ -396,8 +428,8 @@ defmodule Kernel.GuardTest do
 
       assert expand_defguard_to_string(:with_unused_vars, args, nil) == """
              (
-               {arg0, arg1} = {1 + 1, 2 + 2}
-               :erlang.+(arg0, arg1)
+               {arg1, arg2} = {1 + 1, 2 + 2}
+               :erlang.+(arg1, arg2)
              )
              """
     end
@@ -413,8 +445,8 @@ defmodule Kernel.GuardTest do
 
       assert expand_defguard_to_string(:with_reused_vars, args, nil) == """
              (
-               {arg0, arg1, arg2} = {1 + 1, 2 + 2, 3 + 3}
-               :erlang.+(:erlang.+(:erlang.+(arg0, arg0), arg1), arg2)
+               {arg1, arg2, arg3} = {1 + 1, 2 + 2, 3 + 3}
+               :erlang.+(:erlang.+(:erlang.+(arg1, arg1), arg2), arg3)
              )
              """
     end
