@@ -22,6 +22,10 @@ defmodule Kernel.WarningTest do
     assert output =~ "nofile:2"
   end
 
+  test "does not warn on incomplete tokenization" do
+    assert {:error, _} = Code.string_to_quoted(~s[:"foobar" do])
+  end
+
   test "operators formed by many of the same character followed by that character" do
     output =
       capture_err(fn ->
@@ -43,6 +47,9 @@ defmodule Kernel.WarningTest do
     test "does not warn for unnecessary quotes in uppercase atoms/keywords" do
       assert capture_err(fn -> Code.eval_string(~s/:"Foo"/) end) == ""
       assert capture_err(fn -> Code.eval_string(~s/["Foo": :bar]/) end) == ""
+      assert capture_err(fn -> Code.eval_string(~s/:"Foo"/) end) == ""
+      assert capture_err(fn -> Code.eval_string(~s/:"foo@bar"/) end) == ""
+      assert capture_err(fn -> Code.eval_string(~s/:"héllò"/) end) == ""
     end
 
     test "warns for unnecessary quotes" do
@@ -79,6 +86,52 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "unused variable that could be pinned" do
+    output =
+      capture_err(fn ->
+        # Note we use compile_string because eval_string does not emit unused vars warning
+        Code.compile_string("""
+        defmodule Sample do
+          def test do
+            compare_local = "hello"
+            match?(compare_local, "hello")
+
+            compare_nested = "hello"
+            case "hello" do
+              compare_nested -> true
+              _other -> false
+            end
+          end
+        end
+        """)
+      end)
+
+    assert output =~
+             "variable \"compare_local\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)"
+
+    assert output =~
+             "variable \"compare_nested\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)"
+  after
+    purge(Sample)
+  end
+
+  test "unused compiler variable" do
+    output =
+      capture_err(fn ->
+        Code.eval_string("""
+        defmodule Sample do
+          def hello(__MODULE___), do: :ok
+          def world(_R), do: :ok
+        end
+        """)
+      end)
+
+    assert output =~ "unknown compiler variable \"__MODULE___\""
+    refute output =~ "unknown compiler variable \"_R\""
+  after
+    purge(Sample)
+  end
+
   test "nested unused variable" do
     message = "variable \"x\" is unused"
 
@@ -432,7 +485,7 @@ defmodule Kernel.WarningTest do
                defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3), do: [arg1, arg2, arg3]
              end
              """)
-           end) =~ "default arguments in b/3 are never used\n  nofile:3"
+           end) =~ "default values for the optional arguments in b/3 are never used\n  nofile:3"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
@@ -441,7 +494,8 @@ defmodule Kernel.WarningTest do
                defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3), do: [arg1, arg2, arg3]
              end
              """)
-           end) =~ "the first 2 default arguments in b/3 are never used\n  nofile:3"
+           end) =~
+             "the default values for the first 2 optional arguments in b/3 are never used\n  nofile:3"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
@@ -450,7 +504,8 @@ defmodule Kernel.WarningTest do
                defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3), do: [arg1, arg2, arg3]
              end
              """)
-           end) =~ "the first default argument in b/3 is never used\n  nofile:3"
+           end) =~
+             "the default value for the first optional argument in b/3 is never used\n  nofile:3"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
@@ -470,7 +525,7 @@ defmodule Kernel.WarningTest do
                defp b(arg1, arg2, arg3), do: [arg1, arg2, arg3]
              end
              """)
-           end) =~ "default arguments in b/3 are never used\n  nofile:3"
+           end) =~ "default values for the optional arguments in b/3 are never used\n  nofile:3"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
@@ -481,7 +536,8 @@ defmodule Kernel.WarningTest do
                defp b(arg1, arg2, arg3), do: [arg1, arg2, arg3]
              end
              """)
-           end) =~ "the first 2 default arguments in b/3 are never used\n  nofile:3"
+           end) =~
+             "the default values for the first 2 optional arguments in b/3 are never used\n  nofile:3"
   after
     purge([Sample1, Sample2, Sample3, Sample4, Sample5, Sample6])
   end
@@ -823,6 +879,20 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "parens with module attribute" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample do
+               @foo 13
+               @foo()
+             end
+             """)
+           end) =~
+             "the @foo() notation (with parenthesis) is deprecated, please use @foo (without parenthesis) instead"
+  after
+    purge(Sample)
+  end
+
   test "undefined module attribute in function" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -904,26 +974,33 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  # TODO: Simplify when we require OTP 24
+  if System.otp_release() >= "24" do
+    @argument_error_message "the call to :erlang.atom_to_binary/2"
+    @arithmetic_error_message "the call to +/2"
+  else
+    @argument_error_message "this expression"
+    @arithmetic_error_message "this expression"
+  end
+
   test "eval failure warning" do
     assert capture_err(fn ->
-             assert_raise ArgumentError, fn ->
-               Code.eval_string("""
-               defmodule Sample do
-                 Atom.to_string "abc"
-               end
-               """)
+             Code.eval_string("""
+             defmodule Sample1 do
+               def foo, do: Atom.to_string "abc"
              end
-           end) =~ ~r"this expression will fail with ArgumentError\n.*nofile:2"
+             """)
+           end) =~ "#{@argument_error_message} will fail with ArgumentError\n  nofile:2"
 
     assert capture_err(fn ->
              Code.eval_string("""
-             defmodule Sample do
+             defmodule Sample2 do
                def foo, do: 1 + nil
              end
              """)
-           end) =~ ~r"this expression will fail with ArithmeticError\n.*nofile:2"
+           end) =~ "#{@arithmetic_error_message} will fail with ArithmeticError\n  nofile:2"
   after
-    purge([Sample])
+    purge([Sample1, Sample2])
   end
 
   test "undefined function for behaviour" do
@@ -1120,31 +1197,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "with and do clauses emit errors, else clauses do not" do
-    assert capture_err(fn ->
-             Code.compile_string("""
-             with {:first, int} when is_integer(int) <- {:second, Integer.gcd(2, 4)} do
-               int
-             end
-             """)
-           end) =~ "this clause cannot match"
-
-    assert capture_err(fn ->
-             Code.compile_string("""
-             with {:first, int1} when is_integer(int1) <- {:first, Integer.gcd(2, 4)},
-                  {:second, int2} when is_integer(int2) <- {:second, Integer.gcd(2, 4)} do
-               {:ok, int1 + int2}
-             else
-               {:first, nil} -> {:error, "first number is not integer"}
-               {:second, nil} -> {:error, "second number is not integer"}
-             end
-             """)
-           end) == ""
-  after
-    purge(Sample1)
-    purge(Sample2)
-  end
-
   test "warning on code point escape" do
     assert capture_err(fn ->
              Code.eval_string("? ")
@@ -1174,6 +1226,51 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "duplicate docs across clauses" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample1 do
+               defmacro __using__(_) do
+                 quote do
+                   @doc "hello"
+                   def add(a, 1), do: a + 1
+                 end
+               end
+             end
+
+             defmodule Sample2 do
+               use Sample1
+               @doc "world"
+               def add(a, 2), do: a + 2
+             end
+             """)
+           end) == ""
+
+    assert capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample3 do
+               @doc "hello"
+               def add(a, 1), do: a + 1
+               @doc "world"
+               def add(a, b)
+             end
+             """)
+           end) =~ ""
+
+    assert capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample4 do
+               @doc "hello"
+               def add(a, 1), do: a + 1
+               @doc "world"
+               def add(a, 2), do: a + 2
+             end
+             """)
+           end) =~ "redefining @doc attribute previously set at line "
+  after
+    purge([Sample1, Sample2, Sample3, Sample4])
+  end
+
   test "reserved doc metadata keys" do
     output =
       capture_err(fn ->
@@ -1547,26 +1644,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "System.stacktrace is deprecated outside catch/rescue" do
-    output = capture_err(fn -> Code.eval_string("System.stacktrace()") end)
-    assert output =~ "System.stacktrace/0 outside of rescue/catch clauses is deprecated"
-
-    output =
-      capture_err(fn ->
-        Code.eval_string("""
-        try do
-          :trying
-        rescue
-          _ -> System.stacktrace()
-        catch
-          _ -> System.stacktrace()
-        end
-        """)
-      end)
-
-    assert output == ""
-  end
-
   test "unused variable in defguard" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -1652,7 +1729,7 @@ defmodule Kernel.WarningTest do
     end
   end
 
-  test "deprecated GenServer super" do
+  test "deprecated GenServer super on callbacks" do
     assert capture_err(fn ->
              Code.eval_string("""
              defmodule Sample do
@@ -1668,6 +1745,22 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "super is allowed on GenServer.child_spec/1" do
+    refute capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample do
+               use GenServer
+
+               def child_spec(opts) do
+                 super(opts)
+               end
+             end
+             """)
+           end) =~ "calling super for GenServer callback child_spec/1 is deprecated"
+  after
+    purge(Sample)
+  end
+
   test "nested comparison operators" do
     message =
       capture_err(fn ->
@@ -1754,6 +1847,22 @@ defmodule Kernel.WarningTest do
     purge(TestMod)
   end
 
+  test "deprecate nullary remote zero-arity capture with parens" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             import System, only: [pid: 0]
+             &pid/0
+             """)
+           end) == ""
+
+    assert capture_err(fn ->
+             Code.eval_string("""
+             &System.pid()/0
+             """)
+           end) =~
+             "extra parentheses on a remote function capture &System.pid()/0 have been deprecated. Please remove the parentheses: &System.pid/0"
+  end
+
   defp purge(list) when is_list(list) do
     Enum.each(list, &purge/1)
   end
