@@ -21,6 +21,12 @@ defmodule KernelTest do
     end
   end
 
+  test "op ambiguity" do
+    max = 1
+    assert max == 1
+    assert max(1, 2) == 2
+  end
+
   describe "=/2" do
     test "can be reassigned" do
       var = 1
@@ -208,6 +214,7 @@ defmodule KernelTest do
 
     user = struct(User, name: "meg")
     assert user == %User{name: "meg"}
+    assert struct(user, %{name: "meg"}) == user
 
     assert struct(user, unknown: "key") == user
     assert struct(user, %{name: "john"}) == %User{name: "john"}
@@ -334,6 +341,97 @@ defmodule KernelTest do
     assert struct_or_map?(10) == false
   end
 
+  defp struct?(arg, name) when is_struct(arg, name), do: true
+  defp struct?(_arg, _name), do: false
+
+  defp struct_or_map?(arg, name) when is_struct(arg, name) or is_map(arg), do: true
+  defp struct_or_map?(_arg, _name), do: false
+
+  defp not_atom(), do: "not atom"
+
+  test "is_struct/2" do
+    assert is_struct(%{}, Macro.Env) == false
+    assert is_struct([], Macro.Env) == false
+    assert is_struct(%Macro.Env{}, Macro.Env) == true
+    assert is_struct(%Macro.Env{}, URI) == false
+    assert struct?(%Macro.Env{}, Macro.Env) == true
+    assert struct?(%Macro.Env{}, URI) == false
+    assert struct?(%{__struct__: "foo"}, "foo") == false
+    assert struct?(%{__struct__: "foo"}, Macro.Env) == false
+    assert struct?([], Macro.Env) == false
+    assert struct?(%{}, Macro.Env) == false
+
+    assert_raise ArgumentError, "argument error", fn ->
+      is_struct(%{}, not_atom())
+    end
+  end
+
+  test "is_struct/2 and other match works" do
+    assert struct_or_map?(%{}, "foo") == false
+    assert struct_or_map?(%{}, Macro.Env) == true
+    assert struct_or_map?(%Macro.Env{}, Macro.Env) == true
+  end
+
+  defp exception?(arg) when is_exception(arg), do: true
+  defp exception?(_arg), do: false
+
+  defp exception_or_map?(arg) when is_exception(arg) or is_map(arg), do: true
+  defp exception_or_map?(_arg), do: false
+
+  test "is_exception/1" do
+    assert is_exception(%{}) == false
+    assert is_exception([]) == false
+    assert is_exception(%RuntimeError{}) == true
+    assert is_exception(%{__exception__: "foo"}) == false
+    assert exception?(%RuntimeError{}) == true
+    assert exception?(%{__exception__: "foo"}) == false
+    assert exception?([]) == false
+    assert exception?(%{}) == false
+  end
+
+  test "is_exception/1 and other match works" do
+    assert exception_or_map?(%RuntimeError{}) == true
+    assert exception_or_map?(%{}) == true
+    assert exception_or_map?(10) == false
+  end
+
+  defp exception?(arg, name) when is_exception(arg, name), do: true
+  defp exception?(_arg, _name), do: false
+
+  defp exception_or_map?(arg, name) when is_exception(arg, name) or is_map(arg), do: true
+  defp exception_or_map?(_arg, _name), do: false
+
+  test "is_exception/2" do
+    assert is_exception(%{}, RuntimeError) == false
+    assert is_exception([], RuntimeError) == false
+    assert is_exception(%RuntimeError{}, RuntimeError) == true
+    assert is_exception(%RuntimeError{}, Macro.Env) == false
+    assert exception?(%RuntimeError{}, RuntimeError) == true
+    assert exception?(%RuntimeError{}, Macro.Env) == false
+    assert exception?(%{__exception__: "foo"}, "foo") == false
+    assert exception?(%{__exception__: "foo"}, RuntimeError) == false
+    assert exception?([], RuntimeError) == false
+    assert exception?(%{}, RuntimeError) == false
+
+    assert_raise ArgumentError, "argument error", fn ->
+      is_exception(%{}, not_atom())
+    end
+  end
+
+  test "is_exception/2 and other match works" do
+    assert exception_or_map?(%{}, "foo") == false
+    assert exception_or_map?(%{}, RuntimeError) == true
+    assert exception_or_map?(%RuntimeError{}, RuntimeError) == true
+  end
+
+  test "then/2" do
+    assert 1 |> then(fn x -> x * 2 end) == 2
+
+    assert_raise BadArityError, fn ->
+      1 |> then(fn x, y -> x * y end)
+    end
+  end
+
   test "if/2 boolean optimization does not leak variables during expansion" do
     if false do
       :ok
@@ -343,6 +441,13 @@ defmodule KernelTest do
   end
 
   describe "in/2" do
+    test "too large list in guards" do
+      defmodule TooLargeList do
+        @list Enum.map(1..1024, & &1)
+        defguard is_value(value) when value in @list
+      end
+    end
+
     test "with literals on right side" do
       assert 2 in [1, 2, 3]
       assert 2 in 1..3
@@ -384,6 +489,10 @@ defmodule KernelTest do
     def fun_in(x) when x in @at_range, do: :at_range
     def fun_in(x) when x in [9 | [10, 11]], do: :list_cons
     def fun_in(x) when x in [12 | @at_list2], do: :list_cons_at
+    def fun_in(x) when x in 21..15//1, do: raise("oops positive")
+    def fun_in(x) when x in 15..21//-1, do: raise("oops negative")
+    def fun_in(x) when x in 15..21//2, do: :range_step_2
+    def fun_in(x) when x in 15..21//1, do: :range_step_1
     def fun_in(_), do: :none
 
     test "in function guard" do
@@ -401,6 +510,10 @@ defmodule KernelTest do
       assert fun_in(12) == :list_cons_at
       assert fun_in(13) == :list_cons_at
       assert fun_in(14) == :list_cons_at
+      assert fun_in(15) == :range_step_2
+      assert fun_in(16) == :range_step_1
+      assert fun_in(17) == :range_step_2
+      assert fun_in(22) == :none
 
       assert fun_in(0.0) == :none
       assert fun_in(1.0) == :none
@@ -415,12 +528,15 @@ defmodule KernelTest do
       assert fun_in(12.0) == :none
       assert fun_in(13.0) == :none
       assert fun_in(14.0) == :none
+      assert fun_in(15.0) == :none
+      assert fun_in(16.0) == :none
+      assert fun_in(17.0) == :none
     end
 
     def dynamic_in(x, y, z) when x in y..z, do: true
     def dynamic_in(_x, _y, _z), do: false
 
-    test "in dynamic function guard" do
+    test "in dynamic range function guard" do
       assert dynamic_in(1, 1, 3)
       assert dynamic_in(2, 1, 3)
       assert dynamic_in(3, 1, 3)
@@ -439,6 +555,33 @@ defmodule KernelTest do
       refute dynamic_in(2.0, 1, 3)
     end
 
+    def dynamic_step_in(x, y, z, w) when x in y..z//w, do: true
+    def dynamic_step_in(_x, _y, _z, _w), do: false
+
+    test "in dynamic range with step function guard" do
+      assert dynamic_step_in(1, 1, 3, 1)
+      assert dynamic_step_in(2, 1, 3, 1)
+      assert dynamic_step_in(3, 1, 3, 1)
+
+      refute dynamic_step_in(1, 1, 3, -1)
+      refute dynamic_step_in(2, 1, 3, -1)
+      refute dynamic_step_in(3, 1, 3, -1)
+
+      assert dynamic_step_in(1, 3, 1, -1)
+      assert dynamic_step_in(2, 3, 1, -1)
+      assert dynamic_step_in(3, 3, 1, -1)
+
+      refute dynamic_step_in(1, 3, 1, 1)
+      refute dynamic_step_in(2, 3, 1, 1)
+      refute dynamic_step_in(3, 3, 1, 1)
+
+      assert dynamic_step_in(1, 1, 3, 2)
+      refute dynamic_step_in(2, 1, 3, 2)
+      assert dynamic_step_in(3, 1, 3, 2)
+      assert dynamic_step_in(3, 1, 4, 2)
+      refute dynamic_step_in(4, 1, 4, 2)
+    end
+
     defmacrop case_in(x, y) do
       quote do
         case 0 do
@@ -456,6 +599,34 @@ defmodule KernelTest do
       assert case_in(-3, -1..-3) == true
     end
 
+    def map_dot(map) when map.field, do: true
+    def map_dot(_other), do: false
+
+    test "map dot guard" do
+      refute map_dot(:foo)
+      refute map_dot(%{})
+      refute map_dot(%{field: false})
+      assert map_dot(%{field: true})
+
+      message =
+        "cannot invoke remote function in guard. " <>
+          "If you want to do a map lookup instead, please remove parens from map.field()"
+
+      assert_raise CompileError, Regex.compile!(message), fn ->
+        defmodule MapDot do
+          def map_dot(map) when map.field(), do: true
+        end
+      end
+
+      message = ~r"cannot invoke remote function Module.fun/0 inside guards"
+
+      assert_raise CompileError, message, fn ->
+        defmodule MapDot do
+          def map_dot(map) when Module.fun(), do: true
+        end
+      end
+    end
+
     test "performs all side-effects" do
       assert 1 in [1, send(self(), 2)]
       assert_received 2
@@ -474,16 +645,6 @@ defmodule KernelTest do
       _ = a
     end
 
-    test "setting attribute with uppercase" do
-      message = "module attributes set via @ cannot start with an uppercase letter"
-
-      assert_raise ArgumentError, message, fn ->
-        defmodule UpcaseAttrSample do
-          @Upper
-        end
-      end
-    end
-
     test "in module body" do
       defmodule InSample do
         @foo [:a, :b]
@@ -598,15 +759,13 @@ defmodule KernelTest do
 
       result = expand_to_string(quote(do: rand() in [1 | some_call()]))
       assert result =~ "var = rand()"
-      assert result =~ "{arg0} = {some_call()}"
-      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, 1), :lists.member(var, arg0))"
+      assert result =~ "{arg1} = {some_call()}"
+      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, 1), :lists.member(var, arg1))"
 
       result = expand_to_string(quote(do: rand() in [{1}, {2}, {3} | some_call()]))
       assert result =~ "var = rand()"
-      assert result =~ "{arg0, arg1, arg2, arg3} = {{1}, {2}, {3}, some_call()}"
-
-      assert result =~
-               ":erlang.orelse(:erlang.\"=:=\"(var, arg1), :erlang.orelse(:erlang.\"=:=\"(var, arg2), :lists.member(var, arg3))))"
+      assert result =~ "{arg1, arg2, arg3, arg4} = {{1}, {2}, {3}, some_call()}"
+      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, arg3), :lists.member(var, arg4)))"
     end
 
     defp quote_case_in(left, right) do
@@ -625,7 +784,7 @@ defmodule KernelTest do
                """
 
       assert expand_to_string(quote(do: foo in [foo])) =~
-               ~r/{arg0} = {foo}\n\s+:erlang."=:="\(foo, arg0\)\n/
+               ~r/{arg1} = {foo}\n\s+:erlang."=:="\(foo, arg1\)\n/
 
       assert expand_to_string(quote(do: foo in 0..1)) ==
                ":erlang.andalso(:erlang.is_integer(foo), :erlang.andalso(:erlang.>=(foo, 0), :erlang.\"=<\"(foo, 1)))"
@@ -666,6 +825,40 @@ defmodule KernelTest do
     end
   end
 
+  describe "@" do
+    test "setting attribute with do-block" do
+      exception =
+        catch_error(
+          defmodule UpcaseAttrSample do
+            @foo quote do
+              :ok
+            end
+          end
+        )
+
+      assert exception.message =~ "expected 0 or 1 argument for @foo, got 2"
+      assert exception.message =~ "You probably want to wrap the argument value in parentheses"
+    end
+
+    test "setting attribute with uppercase" do
+      message = "module attributes set via @ cannot start with an uppercase letter"
+
+      assert_raise ArgumentError, message, fn ->
+        defmodule UpcaseAttrSample do
+          @Upper
+        end
+      end
+    end
+
+    test "matching attribute" do
+      assert_raise ArgumentError, ~r"invalid write attribute syntax", fn ->
+        defmodule MatchAttributeInModule do
+          @foo = 42
+        end
+      end
+    end
+  end
+
   describe "defdelegate" do
     defdelegate my_flatten(list), to: List, as: :flatten
 
@@ -1137,13 +1330,20 @@ defmodule KernelTest do
     end
   end
 
+  test "tap/1" do
+    import ExUnit.CaptureIO
+
+    assert capture_io(fn ->
+             tap("foo", &IO.puts/1)
+           end) == "foo\n"
+
+    assert 1 = tap(1, fn x -> x + 1 end)
+  end
+
   test "tl/1" do
     assert tl([:one]) == []
     assert tl([1, 2, 3]) == [2, 3]
-
-    assert_raise ArgumentError, "argument error", fn ->
-      tl(empty_list())
-    end
+    assert_raise ArgumentError, fn -> tl(empty_list()) end
 
     assert tl([:a | :b]) == :b
     assert tl([:a, :b | :c]) == [:b | :c]
@@ -1151,11 +1351,7 @@ defmodule KernelTest do
 
   test "hd/1" do
     assert hd([1, 2, 3, 4]) == 1
-
-    assert_raise ArgumentError, "argument error", fn ->
-      hd(empty_list())
-    end
-
+    assert_raise ArgumentError, fn -> hd(empty_list()) end
     assert hd([1 | 2]) == 1
   end
 
@@ -1207,6 +1403,10 @@ defmodule KernelTest do
       Code.eval_string(~s{~U[2015-01-13 13:00]})
     end
 
+    assert_raise ArgumentError, ~r"reason: :invalid_format", fn ->
+      Code.eval_string(~s{~U[20150113 130007Z]})
+    end
+
     assert_raise ArgumentError, ~r"reason: :missing_offset", fn ->
       Code.eval_string(~s{~U[2015-01-13 13:00:07]})
     end
