@@ -7,6 +7,8 @@ defmodule MapTest do
 
   @sample %{a: 1, b: 2}
 
+  defp sample, do: @sample
+
   test "maps in attributes" do
     assert @sample == %{a: 1, b: 2}
   end
@@ -133,18 +135,18 @@ defmodule MapTest do
   end
 
   test "update maps" do
-    assert %{@sample | a: 3} == %{a: 3, b: 2}
+    assert %{sample() | a: 3} == %{a: 3, b: 2}
 
     assert_raise KeyError, fn ->
-      %{@sample | c: 3}
+      %{sample() | c: 3}
     end
   end
 
   test "map dot access" do
-    assert @sample.a == 1
+    assert sample().a == 1
 
     assert_raise KeyError, fn ->
-      @sample.c
+      sample().c
     end
   end
 
@@ -191,6 +193,28 @@ defmodule MapTest do
              %{a: 1, b: 2, c: :x, d: 5}
   end
 
+  test "replace/3" do
+    map = %{c: 3, b: 2, a: 1}
+    assert Map.replace(map, :b, 10) == %{c: 3, b: 10, a: 1}
+    assert Map.replace(map, :a, 1) == map
+    assert Map.replace(map, :x, 1) == map
+    assert Map.replace(%{}, :x, 1) == %{}
+  end
+
+  test "replace!/3" do
+    map = %{c: 3, b: 2, a: 1}
+    assert Map.replace!(map, :b, 10) == %{c: 3, b: 10, a: 1}
+    assert Map.replace!(map, :a, 1) == map
+
+    assert_raise KeyError, "key :x not found in: %{a: 1, b: 2, c: 3}", fn ->
+      Map.replace!(map, :x, 10)
+    end
+
+    assert_raise KeyError, "key :x not found in: %{}", fn ->
+      Map.replace!(%{}, :x, 10)
+    end
+  end
+
   test "implements (almost) all functions in Keyword" do
     assert Keyword.__info__(:functions) -- Map.__info__(:functions) == [
              delete: 3,
@@ -223,6 +247,8 @@ defmodule MapTest do
     end
   end
 
+  defp empty_map(), do: %{}
+
   test "structs" do
     assert %ExternalUser{} == %{__struct__: ExternalUser, name: "john", age: 27}
 
@@ -234,10 +260,8 @@ defmodule MapTest do
     %ExternalUser{name: name} = %ExternalUser{}
     assert name == "john"
 
-    map = %{}
-
     assert_raise BadStructError, "expected a struct named MapTest.ExternalUser, got: %{}", fn ->
-      %ExternalUser{map | name: "meg"}
+      %ExternalUser{empty_map() | name: "meg"}
     end
   end
 
