@@ -2,34 +2,40 @@ Code.require_file("test_helper.exs", __DIR__)
 
 defmodule PathTest do
   use ExUnit.Case, async: true
-
   doctest Path
 
-  import PathHelpers
-
   if :file.native_name_encoding() == :utf8 do
-    test "wildcard with UTF-8" do
-      File.mkdir_p(tmp_path("héllò"))
-      assert Path.wildcard(tmp_path("héllò")) == [tmp_path("héllò")]
+    @tag :tmp_dir
+    test "wildcard with UTF-8", config do
+      File.mkdir_p(Path.join(config.tmp_dir, "héllò"))
+
+      assert Path.wildcard(Path.join(config.tmp_dir, "héllò")) ==
+               [Path.join(config.tmp_dir, "héllò")]
     after
-      File.rm_rf(tmp_path("héllò"))
+      File.rm_rf(Path.join(config.tmp_dir, "héllò"))
     end
   end
 
-  test "wildcard/2" do
-    hello = tmp_path("wildcard/.hello")
-    world = tmp_path("wildcard/.hello/world")
+  @tag :tmp_dir
+  test "wildcard/2", config do
+    hello = Path.join(config.tmp_dir, "wildcard/.hello")
+    world = Path.join(config.tmp_dir, "wildcard/.hello/world")
     File.mkdir_p(world)
 
-    assert Path.wildcard(tmp_path("wildcard/*/*")) == []
-    assert Path.wildcard(tmp_path("wildcard/**/*")) == []
-    assert Path.wildcard(tmp_path("wildcard/?hello/world")) == []
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/*/*")) == []
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/**/*")) == []
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/?hello/world")) == []
 
-    assert Path.wildcard(tmp_path("wildcard/*/*"), match_dot: true) == [world]
-    assert Path.wildcard(tmp_path("wildcard/**/*"), match_dot: true) == [hello, world]
-    assert Path.wildcard(tmp_path("wildcard/?hello/world"), match_dot: true) == [world]
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/*/*"), match_dot: true) ==
+             [world]
+
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/**/*"), match_dot: true) ==
+             [hello, world]
+
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/?hello/world"), match_dot: true) ==
+             [world]
   after
-    File.rm_rf(tmp_path("wildcard"))
+    File.rm_rf(Path.join(config.tmp_dir, "wildcard"))
   end
 
   test "wildcard/2 raises on null byte" do
@@ -44,6 +50,16 @@ defmodule PathTest do
       assert Path.absname("\\\\host\\path") == "//host/path"
       assert Path.absname("\\/host\\path") == "//host/path"
       assert Path.absname("/\\host\\path") == "//host/path"
+
+      assert Path.absname("c:/") == "c:/"
+      assert Path.absname("c:/host/path") == "c:/host/path"
+
+      cwd = File.cwd!()
+      assert Path.absname(cwd |> String.split("/") |> hd()) == cwd
+
+      <<letter, _::binary>> = cwd
+      random = Enum.random(Enum.to_list(?c..?z) -- [letter])
+      assert Path.absname(<<random, ?:>>) == <<random, ?:, ?/>>
     end
 
     test "relative/1" do
@@ -187,7 +203,9 @@ defmodule PathTest do
     assert Path.relative_to("/usr/local/foo", "/usr/local") == "foo"
     assert Path.relative_to("/usr/local/foo", "/") == "usr/local/foo"
     assert Path.relative_to("/usr/local/foo", "/etc") == "/usr/local/foo"
-    assert Path.relative_to("/usr/local/foo", "/usr/local/foo") == "/usr/local/foo"
+    assert Path.relative_to("/usr/local/foo", "/usr/local/foo") == "."
+    assert Path.relative_to("/usr/local/foo/", "/usr/local/foo") == "."
+    assert Path.relative_to("/usr/local/foo", "/usr/local/foo/") == "."
 
     assert Path.relative_to("usr/local/foo", "usr/local") == "foo"
     assert Path.relative_to("usr/local/foo", "etc") == "usr/local/foo"
@@ -275,7 +293,7 @@ defmodule PathTest do
     assert Path.split([?/, "foo/bar"]) == ["/", "foo", "bar"]
   end
 
-  if windows?() do
+  if PathHelpers.windows?() do
     defp strip_drive_letter_if_windows([_d, ?: | rest]), do: rest
     defp strip_drive_letter_if_windows(<<_d, ?:, rest::binary>>), do: rest
   else
