@@ -36,10 +36,12 @@ defmodule RegistryTest do
         {:ok, pid} = Registry.register(registry, "hello", :value)
         assert is_pid(pid)
         assert Registry.keys(registry, self()) == ["hello"]
+        assert Registry.values(registry, "hello", self()) == [:value]
 
         assert {:error, {:already_registered, pid}} = Registry.register(registry, "hello", :value)
         assert pid == self()
         assert Registry.keys(registry, self()) == ["hello"]
+        assert Registry.values(registry, "hello", self()) == [:value]
 
         {:ok, pid} = Registry.register(registry, "world", :value)
         assert is_pid(pid)
@@ -49,11 +51,15 @@ defmodule RegistryTest do
       test "has unique registrations across processes", %{registry: registry} do
         {_, task} = register_task(registry, "hello", :value)
         Process.link(Process.whereis(registry))
+        assert Registry.keys(registry, task) == ["hello"]
+        assert Registry.values(registry, "hello", task) == [:value]
 
         assert {:error, {:already_registered, ^task}} =
                  Registry.register(registry, "hello", :recent)
 
         assert Registry.keys(registry, self()) == []
+        assert Registry.values(registry, "hello", self()) == []
+
         {:links, links} = Process.info(self(), :links)
         assert Process.whereis(registry) in links
       end
@@ -383,7 +389,9 @@ defmodule RegistryTest do
         assert is_pid(pid)
         assert sum_pid_entries(registry, partitions) == 2
 
-        assert {:error, {:already_registered, pid}} = Registry.register(registry, "hello", :value)
+        assert {:error, {:already_registered, _pid}} =
+                 Registry.register(registry, "hello", :value)
+
         assert sum_pid_entries(registry, partitions) == 2
       end
 
@@ -431,16 +439,30 @@ defmodule RegistryTest do
         {:ok, pid} = Registry.register(registry, "hello", :value)
         assert is_pid(pid)
         assert Registry.keys(registry, self()) == ["hello"]
+        assert Registry.values(registry, "hello", self()) == [:value]
 
         assert {:ok, pid} = Registry.register(registry, "hello", :value)
         assert is_pid(pid)
         assert Registry.keys(registry, self()) == ["hello", "hello"]
+        assert Registry.values(registry, "hello", self()) == [:value, :value]
 
         {:ok, pid} = Registry.register(registry, "world", :value)
         assert is_pid(pid)
         assert Registry.keys(registry, self()) |> Enum.sort() == ["hello", "hello", "world"]
       end
 
+      test "has duplicate registrations across processes", %{registry: registry} do
+        {_, task} = register_task(registry, "hello", :world)
+        assert Registry.keys(registry, self()) == []
+        assert Registry.keys(registry, task) == ["hello"]
+        assert Registry.values(registry, "hello", self()) == []
+        assert Registry.values(registry, "hello", task) == [:world]
+
+        assert {:ok, _pid} = Registry.register(registry, "hello", :value)
+        assert Registry.keys(registry, self()) == ["hello"]
+        assert Registry.values(registry, "hello", self()) == [:value]
+      end
+
       test "compares using matches", %{registry: registry} do
         {:ok, _} = Registry.register(registry, 1.0, :value)
         {:ok, _} = Registry.register(registry, 1, :value)
@@ -861,6 +883,19 @@ defmodule RegistryTest do
     end
   end
 
+  test "raises if :compressed is not a boolean" do
+    assert_raise ArgumentError, ~r/expected :compressed to be a boolean, got/, fn ->
+      Registry.start_link(keys: :unique, name: :name, compressed: :fail)
+    end
+  end
+
+  test "unregistration on crash with {registry, key, value} via tuple", %{registry: registry} do
+    name = {:via, Registry, {registry, :name, :value}}
+    spec = %{id: :foo, start: {Agent, :start_link, [fn -> raise "some error" end, [name: name]]}}
+    assert {:error, {error, _childspec}} = start_supervised(spec)
+    assert {%RuntimeError{message: "some error"}, _stacktrace} = error
+  end
+
   defp register_task(registry, key, value) do
     parent = self()
 
