@@ -96,23 +96,22 @@ defmodule SystemTest do
     end
 
     test "cmd/3 (with options)" do
-      assert {["hello\r\n"], 0} =
-               System.cmd(
-                 "cmd",
-                 ~w[/c echo hello],
-                 into: [],
-                 cd: File.cwd!(),
-                 env: %{"foo" => "bar", "baz" => nil},
-                 arg0: "echo",
-                 stderr_to_stdout: true,
-                 parallelism: true
-               )
+      opts = [
+        into: [],
+        cd: File.cwd!(),
+        env: %{"foo" => "bar", "baz" => nil},
+        arg0: "echo",
+        stderr_to_stdout: true,
+        parallelism: true
+      ]
+
+      assert {["hello\r\n"], 0} = System.cmd("cmd", ~w[/c echo hello], opts)
     end
 
     @echo "echo-elixir-test"
-
-    test "cmd/2 with absolute and relative paths" do
-      echo = tmp_path(@echo)
+    @tag :tmp_dir
+    test "cmd/3 with absolute and relative paths", config do
+      echo = Path.join(config.tmp_dir, @echo)
       File.mkdir_p!(Path.dirname(echo))
       File.cp!(System.find_executable("cmd"), echo)
 
@@ -128,6 +127,22 @@ defmodule SystemTest do
                  System.cmd(Path.join(File.cwd!(), @echo), ~w[/c echo hello], [{:arg0, "echo"}])
       end)
     end
+
+    test "shell/1" do
+      assert {"hello\r\n", 0} = System.shell("echo hello")
+    end
+
+    test "shell/2 (with options)" do
+      opts = [
+        into: [],
+        cd: File.cwd!(),
+        env: %{"foo" => "bar", "baz" => nil},
+        stderr_to_stdout: true,
+        parallelism: true
+      ]
+
+      assert {["bar\r\n"], 0} = System.shell("echo %foo%", opts)
+    end
   end
 
   describe "Unix" do
@@ -151,9 +166,9 @@ defmodule SystemTest do
     end
 
     @echo "echo-elixir-test"
-
-    test "cmd/2 with absolute and relative paths" do
-      echo = tmp_path(@echo)
+    @tag :tmp_dir
+    test "cmd/3 with absolute and relative paths", config do
+      echo = Path.join(config.tmp_dir, @echo)
       File.mkdir_p!(Path.dirname(echo))
       File.cp!(System.find_executable("echo"), echo)
 
@@ -169,6 +184,54 @@ defmodule SystemTest do
                  System.cmd(Path.join(File.cwd!(), @echo), ["hello"], [{:arg0, "echo"}])
       end)
     end
+
+    test "shell/1" do
+      assert {"hello\n", 0} = System.shell("echo hello")
+    end
+
+    test "shell/2 (with options)" do
+      opts = [
+        into: [],
+        cd: File.cwd!(),
+        env: %{"foo" => "bar", "baz" => nil},
+        stderr_to_stdout: true
+      ]
+
+      assert {["bar\n"], 0} = System.shell("echo $foo", opts)
+    end
+  end
+
+  @tag :unix
+  test "vm signals" do
+    assert System.trap_signal(:sigquit, :example, fn -> :ok end) == {:ok, :example}
+    assert System.trap_signal(:sigquit, :example, fn -> :ok end) == {:error, :already_registered}
+    assert {:ok, ref} = System.trap_signal(:sigquit, fn -> :ok end)
+
+    assert System.untrap_signal(:sigquit, :example) == :ok
+    assert System.trap_signal(:sigquit, :example, fn -> :ok end) == {:ok, :example}
+    assert System.trap_signal(:sigquit, ref, fn -> :ok end) == {:error, :already_registered}
+
+    assert System.untrap_signal(:sigusr1, :example) == {:error, :not_found}
+    assert System.untrap_signal(:sigquit, :example) == :ok
+    assert System.untrap_signal(:sigquit, :example) == {:error, :not_found}
+    assert System.untrap_signal(:sigquit, ref) == :ok
+    assert System.untrap_signal(:sigquit, ref) == {:error, :not_found}
+  end
+
+  @tag :unix
+  test "os signals" do
+    parent = self()
+
+    assert System.trap_signal(:sighup, :example, fn ->
+             send(parent, :sighup_called)
+             :ok
+           end) == {:ok, :example}
+
+    {"", 0} = System.cmd("kill", ["-s", "hup", System.pid()])
+
+    assert_receive :sighup_called
+  after
+    System.untrap_signal(:sighup, :example)
   end
 
   test "find_executable/1" do
