@@ -103,44 +103,50 @@ defmodule TypespecTest do
     end
 
     test "redefined type" do
-      assert_raise CompileError, ~r"type foo/0 is already defined", fn ->
-        test_module do
-          @type foo :: atom
-          @type foo :: integer
-        end
-      end
+      assert_raise CompileError,
+                   ~r"type foo/0 is already defined in test/elixir/typespec_test.exs:110",
+                   fn ->
+                     test_module do
+                       @type foo :: atom
+                       @type foo :: integer
+                     end
+                   end
 
-      assert_raise CompileError, ~r"type foo/2 is already defined", fn ->
-        test_module do
-          @type foo :: atom
-          @type foo(var1, var2) :: {var1, var2}
-          @type foo(x, y) :: {x, y}
-        end
-      end
+      assert_raise CompileError,
+                   ~r"type foo/2 is already defined in test/elixir/typespec_test.exs:120",
+                   fn ->
+                     test_module do
+                       @type foo :: atom
+                       @type foo(var1, var2) :: {var1, var2}
+                       @type foo(x, y) :: {x, y}
+                     end
+                   end
 
-      assert_raise CompileError, ~r"type foo/0 is already defined", fn ->
-        test_module do
-          @type foo :: atom
-          @typep foo :: integer
-        end
-      end
+      assert_raise CompileError,
+                   ~r"type foo/0 is already defined in test/elixir/typespec_test.exs:129",
+                   fn ->
+                     test_module do
+                       @type foo :: atom
+                       @typep foo :: integer
+                     end
+                   end
     end
 
     test "type variable unused (singleton type variable)" do
-      assert_raise CompileError, ~r"type variable x is unused", fn ->
+      assert_raise CompileError, ~r"type variable x is used only once", fn ->
         test_module do
           @type foo(x) :: integer
         end
       end
     end
 
-    test "@type with a variable starting with underscore" do
+    test "type variable starting with underscore" do
       test_module do
         assert @type(foo(_hello) :: integer) == :ok
       end
     end
 
-    test "type variable _ should be invalid" do
+    test "type variable named _" do
       assert_raise CompileError, ~r"type variable '_' is invalid", fn ->
         test_module do
           @type foo(_) :: integer
@@ -162,8 +168,8 @@ defmodule TypespecTest do
       end
     end
 
-    test "spec variable unused (singleton type variable)" do
-      assert_raise CompileError, ~r"type variable x is unused", fn ->
+    test "spec variable used only once (singleton type variable)" do
+      assert_raise CompileError, ~r"type variable x is used only once", fn ->
         test_module do
           @spec foo(x, integer) :: integer when x: var
           def foo(x, y), do: x + y
@@ -171,7 +177,7 @@ defmodule TypespecTest do
       end
     end
 
-    test "ill defined optional callback" do
+    test "invalid optional callback" do
       assert_raise CompileError, ~r"invalid optional callback :foo", fn ->
         test_module do
           @optional_callbacks :foo
@@ -290,40 +296,43 @@ defmodule TypespecTest do
       assert [type: {:my_type, {:type, _, :term, []}, []}] = types(bytecode)
     end
 
-    test "@type with an atom" do
+    test "@type with an atom/alias" do
       bytecode =
         test_module do
-          @type my_type :: :foo
+          @type foo :: :foo
+          @type bar :: Bar
         end
 
-      assert [type: {:my_type, {:atom, _, :foo}, []}] = types(bytecode)
-    end
-
-    test "@type with an atom alias" do
-      bytecode =
-        test_module do
-          @type my_type :: Atom
-        end
-
-      assert [type: {:my_type, {:atom, _, Atom}, []}] = types(bytecode)
+      assert [
+               type: {:bar, {:atom, _, Bar}, []},
+               type: {:foo, {:atom, _, :foo}, []}
+             ] = types(bytecode)
     end
 
     test "@type with an integer" do
       bytecode =
         test_module do
-          @type my_type :: 10
+          @type pos :: 10
+          @type neg :: -10
         end
 
-      assert [type: {:my_type, {:integer, _, 10}, []}] = types(bytecode)
+      assert [
+               type: {:neg, {:op, _, :-, {:integer, _, 10}}, []},
+               type: {:pos, {:integer, _, 10}, []}
+             ] = types(bytecode)
     end
 
-    test "@type with a negative integer" do
+    test "@type with a tuple" do
       bytecode =
         test_module do
-          @type my_type :: -10
+          @type tup :: tuple()
+          @type one :: {123}
         end
 
-      assert [type: {:my_type, {:op, _, :-, {:integer, _, 10}}, []}] = types(bytecode)
+      assert [
+               type: {:one, {:type, _, :tuple, [{:integer, _, 123}]}, []},
+               type: {:tup, {:type, _, :tuple, :any}, []}
+             ] = types(bytecode)
     end
 
     test "@type with a remote type" do
@@ -346,50 +355,21 @@ defmodule TypespecTest do
     test "@type with a binary" do
       bytecode =
         test_module do
-          @type my_type :: binary
+          @type bin :: binary
+          @type empty :: <<>>
+          @type size :: <<_::3>>
+          @type unit :: <<_::_*8>>
+          @type size_and_unit :: <<_::3, _::_*8>>
         end
 
-      assert [type: {:my_type, {:type, _, :binary, []}, []}] = types(bytecode)
-    end
-
-    test "@type with an empty binary" do
-      bytecode =
-        test_module do
-          @type my_type :: <<>>
-        end
-
-      assert [type: {:my_type, {:type, _, :binary, [{:integer, _, 0}, {:integer, _, 0}]}, []}] =
-               types(bytecode)
-    end
-
-    test "@type with a binary with a base size" do
-      bytecode =
-        test_module do
-          @type my_type :: <<_::3>>
-        end
-
-      assert [type: {:my_type, {:type, _, :binary, [{:integer, _, 3}, {:integer, _, 0}]}, []}] =
-               types(bytecode)
-    end
-
-    test "@type with a binary with a unit size" do
-      bytecode =
-        test_module do
-          @type my_type :: <<_::_*8>>
-        end
-
-      assert [type: {:my_type, {:type, _, :binary, [{:integer, _, 0}, {:integer, _, 8}]}, []}] =
-               types(bytecode)
-    end
-
-    test "@type with a binary with a size and unit size" do
-      bytecode =
-        test_module do
-          @type my_type :: <<_::3, _::_*8>>
-        end
-
-      assert [type: {:my_type, {:type, _, :binary, [{:integer, _, 3}, {:integer, _, 8}]}, []}] =
-               types(bytecode)
+      assert [
+               type: {:bin, {:type, _, :binary, []}, []},
+               type: {:empty, {:type, _, :binary, [{:integer, _, 0}, {:integer, _, 0}]}, []},
+               type: {:size, {:type, _, :binary, [{:integer, _, 3}, {:integer, _, 0}]}, []},
+               type:
+                 {:size_and_unit, {:type, _, :binary, [{:integer, _, 3}, {:integer, _, 8}]}, []},
+               type: {:unit, {:type, _, :binary, [{:integer, _, 0}, {:integer, _, 8}]}, []}
+             ] = types(bytecode)
     end
 
     test "@type with invalid binary spec" do
@@ -405,6 +385,12 @@ defmodule TypespecTest do
         end
       end
 
+      assert_raise CompileError, ~r"invalid binary specification", fn ->
+        test_module do
+          @type my_type :: <<_::integer>>
+        end
+      end
+
       assert_raise CompileError, ~r"invalid binary specification", fn ->
         test_module do
           @type my_type :: <<_::(-4)>>
@@ -422,6 +408,12 @@ defmodule TypespecTest do
           @type my_type :: <<_::3, _::_*(-8)>>
         end
       end
+
+      assert_raise CompileError, ~r"invalid binary specification", fn ->
+        test_module do
+          @type my_type :: <<_::3, _::_*257>>
+        end
+      end
     end
 
     test "@type with a range op" do
@@ -511,7 +503,7 @@ defmodule TypespecTest do
           @type my_type :: %TypespecSample{hello: :world}
         end
 
-      assert [type: {:my_type, type, []}] = types(bytecode)
+      assert [type: {:my_type, _type, []}] = types(bytecode)
     end
 
     test "@type with undefined struct" do
@@ -595,7 +587,7 @@ defmodule TypespecTest do
         end
 
       assert [type: {:my_type, type, []}] = types(bytecode)
-      assert {:type, _, :tuple, [my_timestamp, term, foo]} = type
+      assert {:type, _, :tuple, [my_timestamp, term, _foo]} = type
       assert {:atom, 0, :my_timestamp} = my_timestamp
       assert {:ann_type, 0, [{:var, 0, :date}, {:type, 0, :term, []}]} = term
       assert {:ann_type, 0, [{:var, 0, :time}, {:atom, 0, :foo}]}
@@ -765,25 +757,6 @@ defmodule TypespecTest do
       assert {:type, _, :integer, []} = fun_return
     end
 
-    test "@opaque(type)" do
-      bytecode =
-        test_module do
-          @opaque my_type(x) :: x
-        end
-
-      assert [opaque: {:my_type, {:var, _, :x}, [{:var, _, :x}]}] = types(bytecode)
-    end
-
-    test "@type + opaque" do
-      bytecode =
-        test_module do
-          @type my_type :: tuple
-          @opaque my_type1 :: {}
-        end
-
-      assert [opaque: {:my_type1, _, []}, type: {:my_type, _, []}] = types(bytecode)
-    end
-
     test "@type unquote fragment" do
       quoted =
         quote unquote: false do
@@ -816,6 +789,56 @@ defmodule TypespecTest do
       assert [{:atom, _, Keyword}, {:atom, _, :t}, [{:var, _, :value}]] = kw_with_value_args
     end
 
+    test "@type with a reserved signature" do
+      assert_raise CompileError,
+                   ~r"type required\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @type required(arg) :: any()
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"type optional\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @type optional(arg) :: any()
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"type required\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @typep required(arg) :: any()
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"type optional\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @typep optional(arg) :: any()
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"type required\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @opaque required(arg) :: any()
+                     end
+                   end
+
+      assert_raise CompileError,
+                   ~r"type optional\/1 is a reserved type and it cannot be defined",
+                   fn ->
+                     test_module do
+                       @opaque optional(arg) :: any()
+                     end
+                   end
+    end
+
     test "invalid remote @type with module attribute that does not evaluate to a module" do
       assert_raise CompileError, ~r/\(@foo is "bar"\)/, fn ->
         test_module do
@@ -846,7 +869,16 @@ defmodule TypespecTest do
       assert specs(bytecode) == callbacks(bytecode)
     end
 
-    test "@spec(spec)" do
+    test "@opaque" do
+      bytecode =
+        test_module do
+          @opaque my_type(x) :: x
+        end
+
+      assert [opaque: {:my_type, {:var, _, :x}, [{:var, _, :x}]}] = types(bytecode)
+    end
+
+    test "@spec" do
       bytecode =
         test_module do
           def my_fun1(x), do: x
@@ -876,7 +908,7 @@ defmodule TypespecTest do
       assert {:type, _, :product, [{:ann_type, _, [{:var, _, :x}, {:type, _, :integer, []}]}]} = x
     end
 
-    test "@spec(spec) with tuples and tuple vars" do
+    test "@spec with vars matching built-ins" do
       bytecode =
         test_module do
           def my_fun1(x), do: x
@@ -898,7 +930,7 @@ defmodule TypespecTest do
                type
     end
 
-    test "@spec(spec) with guards" do
+    test "@spec with guards" do
       bytecode =
         test_module do
           def my_fun1(x), do: x
@@ -1050,7 +1082,7 @@ defmodule TypespecTest do
       :code.purge(SpecModuleAttributes)
     end
 
-    test "@callback(callback)" do
+    test "@callback" do
       bytecode =
         test_module do
           @callback my_fun(integer) :: integer
@@ -1079,29 +1111,6 @@ defmodule TypespecTest do
                return_type
     end
 
-    test "@spec + @callback" do
-      bytecode =
-        test_module do
-          def my_fun(x), do: x
-          @spec my_fun(integer) :: integer
-          @spec my_fun(charlist) :: charlist
-          @callback cb(integer) :: integer
-        end
-
-      assert [{{:cb, 1}, [{:type, _, :fun, args}]}] = callbacks(bytecode)
-      assert [{:type, _, :product, [{:type, _, :integer, []}]}, {:type, _, :integer, []}] = args
-
-      assert [{{:my_fun, 1}, [integer_clause, charlist_clause]}] = specs(bytecode)
-
-      assert {:type, _, :fun, [{:type, _, :product, [arg]}, return]} = integer_clause
-      assert {:type, _, :integer, []} = arg
-      assert {:type, _, :integer, []} = return
-
-      assert {:type, _, :fun, [{:type, _, :product, [arg]}, return]} = charlist_clause
-      assert {:remote_type, _, [{:atom, _, :elixir}, {:atom, _, :charlist}, []]} = arg
-      assert {:remote_type, _, [{:atom, _, :elixir}, {:atom, _, :charlist}, []]} = return
-    end
-
     test "block handling" do
       bytecode =
         test_module do
@@ -1120,11 +1129,13 @@ defmodule TypespecTest do
     test "type_to_quoted" do
       quoted =
         Enum.sort([
+          quote(do: @type(tuple(arg) :: {:tuple, arg})),
           quote(do: @type(with_ann() :: t :: atom())),
           quote(do: @type(a_tuple() :: tuple())),
           quote(do: @type(empty_tuple() :: {})),
           quote(do: @type(one_tuple() :: {:foo})),
           quote(do: @type(two_tuple() :: {:foo, :bar})),
+          quote(do: @type(custom_tuple() :: tuple(:foo))),
           quote(do: @type(imm_type_1() :: 1)),
           quote(do: @type(imm_type_2() :: :foo)),
           quote(do: @type(simple_type() :: integer())),
@@ -1146,12 +1157,14 @@ defmodule TypespecTest do
           quote(do: @type(rng() :: 1..10)),
           quote(do: @type(opts() :: [first: integer(), step: integer(), last: integer()])),
           quote(do: @type(ops() :: {+1, -1})),
+          quote(do: @type(map(arg) :: {:map, arg})),
           quote(do: @type(a_map() :: map())),
           quote(do: @type(empty_map() :: %{})),
           quote(do: @type(my_map() :: %{hello: :world})),
           quote(do: @type(my_req_map() :: %{required(0) => :foo})),
           quote(do: @type(my_opt_map() :: %{optional(0) => :foo})),
           quote(do: @type(my_struct() :: %TypespecTest{hello: :world})),
+          quote(do: @type(custom_map() :: map(:foo))),
           quote(do: @type(list1() :: list())),
           quote(do: @type(list2() :: [0])),
           quote(do: @type(list3() :: [...])),
@@ -1183,6 +1196,12 @@ defmodule TypespecTest do
       quoted =
         Enum.sort([
           quote(do: @spec(foo() :: integer())),
+          quote(do: @spec(foo() :: union())),
+          quote(do: @spec(foo() :: union(integer()))),
+          quote(do: @spec(foo() :: truly_union())),
+          quote(do: @spec(foo(union()) :: union())),
+          quote(do: @spec(foo(union(integer())) :: union(integer()))),
+          quote(do: @spec(foo(truly_union()) :: truly_union())),
           quote(do: @spec(foo(atom()) :: integer() | [{}])),
           quote(do: @spec(foo(arg) :: integer() when [arg: integer()])),
           quote(do: @spec(foo(arg) :: arg when [arg: var])),
@@ -1191,6 +1210,10 @@ defmodule TypespecTest do
 
       bytecode =
         test_module do
+          @type union :: any()
+          @type union(t) :: t
+          @type truly_union :: list | map | union
+
           def foo(), do: 1
           def foo(arg), do: arg
           Module.eval_quoted(__MODULE__, quote(do: (unquote_splicing(quoted))))
@@ -1210,6 +1233,69 @@ defmodule TypespecTest do
       end)
     end
 
+    test "spec_to_quoted with maps with __struct__ key" do
+      defmodule A do
+        defstruct [:key]
+      end
+
+      defmodule B do
+        defstruct [:key]
+      end
+
+      bytecode =
+        test_module do
+          @spec single_struct(%A{}) :: :ok
+          def single_struct(arg), do: {:ok, arg}
+
+          @spec single_struct_key(%{__struct__: A}) :: :ok
+          def single_struct_key(arg), do: {:ok, arg}
+
+          @spec single_struct_key_type(%{__struct__: atom()}) :: :ok
+          def single_struct_key_type(arg), do: {:ok, arg}
+
+          @spec union_struct(%A{} | %B{}) :: :ok
+          def union_struct(arg), do: {:ok, arg}
+
+          @spec union_struct_key(%{__struct__: A | B}) :: :ok
+          def union_struct_key(arg), do: {:ok, arg}
+
+          @spec union_struct_key_type(%{__struct__: atom() | A | binary()}) :: :ok
+          def union_struct_key_type(arg), do: {:ok, arg}
+        end
+
+      [
+        {{:single_struct, 1}, [ast_single_struct]},
+        {{:single_struct_key, 1}, [ast_single_struct_key]},
+        {{:single_struct_key_type, 1}, [ast_single_struct_key_type]},
+        {{:union_struct, 1}, [ast_union_struct]},
+        {{:union_struct_key, 1}, [ast_union_struct_key]},
+        {{:union_struct_key_type, 1}, [ast_union_struct_key_type]}
+      ] = specs(bytecode)
+
+      assert Code.Typespec.spec_to_quoted(:single_struct, ast_single_struct)
+             |> Macro.to_string() ==
+               "single_struct(%TypespecTest.A{key: term()}) :: :ok"
+
+      assert Code.Typespec.spec_to_quoted(:single_struct_key, ast_single_struct_key)
+             |> Macro.to_string() ==
+               "single_struct_key(%TypespecTest.A{}) :: :ok"
+
+      assert Code.Typespec.spec_to_quoted(:single_struct_key_type, ast_single_struct_key_type)
+             |> Macro.to_string() ==
+               "single_struct_key_type(%{__struct__: atom()}) :: :ok"
+
+      assert Code.Typespec.spec_to_quoted(:union_struct, ast_union_struct) |> Macro.to_string() ==
+               "union_struct(%TypespecTest.A{key: term()} | %TypespecTest.B{key: term()}) :: :ok"
+
+      assert Code.Typespec.spec_to_quoted(:union_struct_key, ast_union_struct_key)
+             |> Macro.to_string() ==
+               "union_struct_key(%{__struct__: TypespecTest.A | TypespecTest.B}) :: :ok"
+
+      assert Code.Typespec.spec_to_quoted(:union_struct_key_type, ast_union_struct_key_type)
+             |> Macro.to_string() ==
+               "union_struct_key_type(%{__struct__: atom() | TypespecTest.A | binary()}) :: :ok"
+    end
+
     test "non-variables are given as arguments" do
       msg = ~r/The type one_bad_variable\/1 has an invalid argument\(s\): String.t\(\)/
 
@@ -1241,7 +1327,7 @@ defmodule TypespecTest do
       assert Code.Typespec.fetch_specs(Unknown) == :error
     end
 
-    # This is a test that implements all types specified in lib/elixir/pages/Typespecs.md
+    # This is a test that implements all types specified in lib/elixir/pages/typespecs.md
     test "documented types and their AST" do
       defmodule SomeStruct do
         defstruct [:key]
