@@ -4,8 +4,6 @@
   atom_test,
   control_test,
   function_test,
-  match_test,
-  operators_test,
   string_test,
   tokenizer_test
 ]).
@@ -28,7 +26,7 @@ run_and_remove(Fun, Modules) ->
 
 % Throws an error with the Erlang Abstract Form from the Elixir string
 throw_elixir(String) ->
-  Forms = elixir:'string_to_quoted!'(String, 1, <<"nofile">>, []),
+  Forms = elixir:'string_to_quoted!'(String, 1, 1, <<"nofile">>, []),
   {Expr, _, _} = elixir:quoted_to_erl(Forms, elixir:env_for_eval([])),
   erlang:error(io:format("~p~n", [Expr])).
 
