@@ -13,7 +13,7 @@ defmodule ExUnit do
 
       # 2) Create a new test module (test case) and use "ExUnit.Case".
       defmodule AssertionTest do
-        # 3) Notice we pass "async: true", this runs the test case
+        # 3) Note that we pass "async: true", this runs the test case
         #    concurrently with other test cases. The individual tests
         #    within each test case are still run serially.
         use ExUnit.Case, async: true
@@ -54,7 +54,7 @@ defmodule ExUnit do
 
   Mix will load the `test_helper.exs` file before executing the tests.
   It is not necessary to `require` the `test_helper.exs` file in your test
-  files. See `Mix.Tasks.Test` for more information.
+  files. Run `mix help test` for more information.
   """
 
   @typedoc """
@@ -70,7 +70,7 @@ defmodule ExUnit do
 
   """
   @type state ::
-          nil | {:failed, failed} | {:skipped, binary} | {:excluded, binary} | {:invalid, module}
+          nil | {:excluded, binary} | {:failed, failed} | {:invalid, module} | {:skipped, binary}
 
   @typedoc "The error state returned by `ExUnit.Test` and `ExUnit.TestModule`"
   @type failed :: [{Exception.kind(), reason :: term, Exception.stacktrace()}]
@@ -83,6 +83,8 @@ defmodule ExUnit do
           total: non_neg_integer
         }
 
+  @type test_id :: {module, name :: atom}
+
   defmodule Test do
     @moduledoc """
     A struct that keeps information about the test.
@@ -117,18 +119,27 @@ defmodule ExUnit do
 
     It is received by formatters and contains the following fields:
 
+      * `:file`  - (since v1.11.0) the file of the test module
+
       * `:name`  - the test module name
+
       * `:state` - the test error state (see `t:ExUnit.state/0`)
+
       * `:tests` - all tests in this module
 
     """
-    defstruct [:name, :state, tests: []]
+    defstruct [:file, :name, :state, tests: []]
 
-    @type t :: %__MODULE__{name: module, state: ExUnit.state(), tests: [ExUnit.Test.t()]}
+    @type t :: %__MODULE__{
+            file: binary(),
+            name: module,
+            state: ExUnit.state(),
+            tests: [ExUnit.Test.t()]
+          }
   end
 
   defmodule TestCase do
-    # TODO: Remove this module on v2.0 (it has been replacede by TestModule)
+    # TODO: Remove this module on v2.0 (it has been replaced by TestModule)
     @moduledoc false
     defstruct [:name, :state, tests: []]
 
@@ -227,7 +238,7 @@ defmodule ExUnit do
       * `:diff_insert` - color of the insertions on diffs, defaults to `:green`;
       * `:diff_insert_whitespace` - color of the whitespace insertions on diffs,
         defaults to `IO.ANSI.color_background(2, 0, 0)`;
-      * `:diff_delete` - color of the deletiopns on diffs, defaults to `:red`;
+      * `:diff_delete` - color of the deletions on diffs, defaults to `:red`;
       * `:diff_delete_whitespace` - color of the whitespace deletions on diffs,
         defaults to `IO.ANSI.color_background(0, 2, 0)`;
 
@@ -263,7 +274,9 @@ defmodule ExUnit do
     * `:seed` - an integer seed value to randomize the test suite. This seed
       is also mixed with the test module and name to create a new unique seed
       on every test, which is automatically fed into the `:rand` module. This
-      provides randomness between tests, but predictable and reproducible results;
+      provides randomness between tests, but predictable and reproducible
+      results. A `:seed` of `0` will disable randomization and the tests in each
+      file will always run in the order that they were defined in;
 
     * `:slowest` - prints timing information for the N slowest tests. Running
       ExUnit with slow test reporting automatically runs in `trace` mode. It
@@ -339,10 +352,36 @@ defmodule ExUnit do
   """
   @spec run() :: suite_result()
   def run do
+    _ = ExUnit.Server.modules_loaded()
     options = persist_defaults(configuration())
     ExUnit.Runner.run(options, nil)
   end
 
+  @doc """
+  Starts tests asynchronously while test cases are still loading.
+
+  It returns a task that must be given to `await_run/0` when a result
+  is desired.
+  """
+  @doc since: "1.12.0"
+  @spec async_run() :: Task.t()
+  def async_run() do
+    Task.async(fn ->
+      options = persist_defaults(configuration())
+      ExUnit.Runner.run(options, nil)
+    end)
+  end
+
+  @doc """
+  Awaits for a test suite that has been started with `async_run/0`.
+  """
+  @doc since: "1.12.0"
+  @spec await_run(Task.t()) :: suite_result()
+  def await_run(task) do
+    ExUnit.Server.modules_loaded()
+    Task.await(task, :infinity)
+  end
+
   @doc """
   Sets a callback to be executed after the completion of a test suite.
 
@@ -360,6 +399,32 @@ defmodule ExUnit do
     configure(after_suite: [function | current_callbacks])
   end
 
+  @doc """
+  Fetches the test supervisor for the current test.
+
+  Returns `{:ok, supervisor_pid}` or `:error` if not called from the test process.
+
+  This is the same supervisor as used by `ExUnit.Callbacks.start_supervised/2`
+  and similar, see `ExUnit.Callbacks` module documentation for more information.
+  """
+  @doc since: "1.11.0"
+  @spec fetch_test_supervisor() :: {:ok, pid()} | :error
+  def fetch_test_supervisor() do
+    case ExUnit.OnExitHandler.get_supervisor(self()) do
+      {:ok, nil} ->
+        opts = [strategy: :one_for_one, max_restarts: 1_000_000, max_seconds: 1]
+        {:ok, sup} = Supervisor.start_link([], opts)
+        ExUnit.OnExitHandler.put_supervisor(self(), sup)
+        {:ok, sup}
+
+      {:ok, _} = ok ->
+        ok
+
+      :error ->
+        :error
+    end
+  end
+
   # Persists default values in application
   # environment before the test suite starts.
   defp persist_defaults(config) do
