@@ -22,7 +22,7 @@ defmodule ExUnit.Case do
 
   This module automatically includes all callbacks defined in
   `ExUnit.Callbacks`. See that module for more information on `setup`,
-  `start_supervised`, `on_exit` and the test process lifecycle.
+  `start_supervised`, `on_exit` and the test process life cycle.
 
   For grouping tests together, see `describe/2` in this module.
 
@@ -141,20 +141,30 @@ defmodule ExUnit.Case do
   therefore reserved:
 
     * `:module`     - the module on which the test was defined
+
     * `:file`       - the file on which the test was defined
+
     * `:line`       - the line on which the test was defined
+
     * `:test`       - the test name
+
     * `:async`      - if the test case is in async mode
+
     * `:registered` - used for `ExUnit.Case.register_attribute/3` values
+
     * `:describe`   - the describe block the test belongs to
 
   The following tags customize how tests behave:
 
     * `:capture_log` - see the "Log Capture" section below
+
     * `:skip` - skips the test with the given reason
+
     * `:timeout` - customizes the test timeout in milliseconds (defaults to 60000).
       Accepts `:infinity` as a timeout value.
 
+    * `:tmp_dir` - (since v1.11.0) see the "Tmp Dir" section below
+
   The `:test_type` tag is automatically set by ExUnit, but is **not** reserved.
   This tag is available for users to customize if they desire.
 
@@ -209,6 +219,32 @@ defmodule ExUnit.Case do
 
       config :logger, backends: []
 
+  ## Tmp Dir
+
+  ExUnit automatically creates a temporary directory for tests tagged with
+  `:tmp_dir` and puts the path to that directory into the test context.
+  The directory is removed before being created to ensure we start with a blank
+  slate.
+
+  The temporary directory path is unique (includes the test module and test name)
+  and thus appropriate for running tests concurrently. You can customize the path
+  further by setting the tag to a string, e.g.: `tmp_dir: "my_path"`, which would
+  make the final path to be: `tmp/<module>/<test>/my_path`.
+
+  Example:
+
+      defmodule MyTest do
+        use ExUnit.Case, async: true
+
+        @tag :tmp_dir
+        test "with tmp_dir", %{tmp_dir: tmp_dir} do
+          assert tmp_dir =~ "with tmp_dir"
+          assert File.dir?(tmp_dir)
+        end
+      end
+
+  As with other tags, `:tmp_dir` can also be set as `@moduletag` and
+  `@describetag`.
   """
 
   @type env :: module() | Macro.Env.t()
@@ -223,41 +259,10 @@ defmodule ExUnit.Case do
     end
 
     quote do
-      unless Module.has_attribute?(__MODULE__, :ex_unit_tests) do
-        tag_check =
-          [:moduletag, :describetag, :tag]
-          |> Enum.any?(&Module.has_attribute?(__MODULE__, &1))
-
-        if tag_check do
-          raise "you must set @tag, @describetag, and @moduletag after the call to \"use ExUnit.Case\""
-        end
-
-        attributes = [
-          :ex_unit_tests,
-          :tag,
-          :describetag,
-          :moduletag,
-          :ex_unit_registered_test_attributes,
-          :ex_unit_registered_describe_attributes,
-          :ex_unit_registered_module_attributes,
-          :ex_unit_used_describes
-        ]
-
-        Enum.each(attributes, &Module.register_attribute(__MODULE__, &1, accumulate: true))
-
-        @before_compile ExUnit.Case
-        @after_compile ExUnit.Case
-        @ex_unit_async false
-        @ex_unit_describe nil
+      unless ExUnit.Case.__register__(__MODULE__, unquote(opts)) do
         use ExUnit.Callbacks
       end
 
-      async = unquote(opts)[:async]
-
-      if is_boolean(async) do
-        @ex_unit_async async
-      end
-
       import ExUnit.Callbacks
       import ExUnit.Assertions
       import ExUnit.Case, only: [describe: 2, test: 1, test: 2, test: 3]
@@ -265,11 +270,54 @@ defmodule ExUnit.Case do
     end
   end
 
+  @doc false
+  def __register__(module, opts) do
+    registered? = Module.has_attribute?(module, :ex_unit_tests)
+
+    unless registered? do
+      tag_check = Enum.any?([:moduletag, :describetag, :tag], &Module.has_attribute?(module, &1))
+
+      if tag_check do
+        raise "you must set @tag, @describetag, and @moduletag after the call to \"use ExUnit.Case\""
+      end
+
+      attributes = [
+        :ex_unit_tests,
+        :tag,
+        :describetag,
+        :moduletag,
+        :ex_unit_registered_test_attributes,
+        :ex_unit_registered_describe_attributes,
+        :ex_unit_registered_module_attributes,
+        :ex_unit_used_describes
+      ]
+
+      Enum.each(attributes, &Module.register_attribute(module, &1, accumulate: true))
+
+      attributes = [
+        before_compile: ExUnit.Case,
+        after_compile: ExUnit.Case,
+        ex_unit_async: false,
+        ex_unit_describe: nil
+      ]
+
+      Enum.each(attributes, fn {k, v} -> Module.put_attribute(module, k, v) end)
+    end
+
+    async? = opts[:async]
+
+    if is_boolean(async?) do
+      Module.put_attribute(module, :ex_unit_async, async?)
+    end
+
+    registered?
+  end
+
   @doc """
   Defines a test with `message`.
 
-  The test may also define a `var`, which will pattern match
-  on the test context. For more information on contexts, see
+  The test may also define a pattern, which will be matched
+  against the test context. For more information on contexts, see
   `ExUnit.Callbacks`.
 
   ## Examples
@@ -280,6 +328,14 @@ defmodule ExUnit.Case do
 
   """
   defmacro test(message, var \\ quote(do: _), contents) do
+    unless is_tuple(var) do
+      IO.warn(
+        "test context is always a map. The pattern " <>
+          "#{inspect(Macro.to_string(var))} will never match",
+        Macro.Env.stacktrace(__CALLER__)
+      )
+    end
+
     contents =
       case contents do
         [do: block] ->
@@ -297,9 +353,17 @@ defmodule ExUnit.Case do
 
     var = Macro.escape(var)
     contents = Macro.escape(contents, unquote: true)
+    %{module: mod, file: file, line: line} = __CALLER__
 
-    quote bind_quoted: [var: var, contents: contents, message: message] do
-      name = ExUnit.Case.register_test(__ENV__, :test, message, [])
+    quote bind_quoted: [
+            var: var,
+            contents: contents,
+            message: message,
+            mod: mod,
+            file: file,
+            line: line
+          ] do
+      name = ExUnit.Case.register_test(mod, file, line, :test, message, [])
       def unquote(name)(unquote(var)), do: unquote(contents)
     end
   end
@@ -318,8 +382,10 @@ defmodule ExUnit.Case do
 
   """
   defmacro test(message) do
+    %{module: mod, file: file, line: line} = __CALLER__
+
     quote bind_quoted: binding() do
-      name = ExUnit.Case.register_test(__ENV__, :test, message, [:not_implemented])
+      name = ExUnit.Case.register_test(mod, file, line, :test, message, [:not_implemented])
       def unquote(name)(_), do: flunk("Not implemented")
     end
   end
@@ -387,23 +453,14 @@ defmodule ExUnit.Case do
   """
   defmacro describe(message, do: block) do
     quote do
-      ExUnit.Case.__describe__(__MODULE__, __ENV__.line, unquote(message))
-
-      try do
+      ExUnit.Case.__describe__(__MODULE__, __ENV__.line, unquote(message), fn ->
         unquote(block)
-      after
-        @ex_unit_describe nil
-        Module.delete_attribute(__MODULE__, :describetag)
-
-        for attribute <- Module.get_attribute(__MODULE__, :ex_unit_registered_describe_attributes) do
-          Module.delete_attribute(__MODULE__, attribute)
-        end
-      end
+      end)
     end
   end
 
   @doc false
-  def __describe__(module, line, message) do
+  def __describe__(module, line, message, fun) do
     if Module.get_attribute(module, :ex_unit_describe) do
       raise "cannot call \"describe\" inside another \"describe\". See the documentation " <>
               "for ExUnit.Case.describe/2 on named setups and how to handle hierarchies"
@@ -427,14 +484,24 @@ defmodule ExUnit.Case do
 
     Module.put_attribute(module, :ex_unit_describe, {line, message})
     Module.put_attribute(module, :ex_unit_used_describes, message)
-    :ok
+
+    try do
+      fun.()
+    after
+      Module.put_attribute(module, :ex_unit_describe, nil)
+      Module.delete_attribute(module, :describetag)
+
+      for attribute <- Module.get_attribute(module, :ex_unit_registered_describe_attributes) do
+        Module.delete_attribute(module, attribute)
+      end
+    end
   end
 
   @doc false
   defmacro __before_compile__(_) do
     quote do
       def __ex_unit__ do
-        %ExUnit.TestModule{name: __MODULE__, tests: @ex_unit_tests}
+        %ExUnit.TestModule{file: __ENV__.file, name: __MODULE__, tests: @ex_unit_tests}
       end
     end
   end
@@ -460,7 +527,7 @@ defmodule ExUnit.Case do
   display. You can use `ExUnit.plural_rule/2` to set a custom
   pluralization.
   """
-  def register_test(%{module: mod, file: file, line: line}, test_type, name, tags) do
+  def register_test(mod, file, line, test_type, name, tags) do
     unless Module.has_attribute?(mod, :ex_unit_tests) do
       raise "cannot define #{test_type}. Please make sure you have invoked " <>
               "\"use ExUnit.Case\" in the current module"
@@ -521,6 +588,17 @@ defmodule ExUnit.Case do
     name
   end
 
+  @doc """
+  Registers a test with the given environment.
+
+  This function is deprecated in favor of `register_test/6` which performs
+  better under tight loops by avoiding `__ENV__`.
+  """
+  @doc deprecated: "Use register_test/6 instead"
+  def register_test(%{module: mod, file: file, line: line}, test_type, name, tags) do
+    register_test(mod, file, line, test_type, name, tags)
+  end
+
   @doc """
   Registers a new attribute to be used during `ExUnit.Case` tests.
 
@@ -621,11 +699,11 @@ defmodule ExUnit.Case do
         @module_fixtures {:post, insert: false}
 
         test "using custom attribute", context do
-          assert context.registered.fixtures == [{:post, insert: false}, :user]
+          assert context.registered.module_fixtures == [{:post, insert: false}, :user]
         end
 
         test "still using custom attribute", context do
-          assert context.registered.fixtures == [{:post, insert: false}, :user]
+          assert context.registered.module_fixtures == [{:post, insert: false}, :user]
         end
       end
 
