@@ -1,23 +1,21 @@
 defmodule ExUnit.DocTest do
   @moduledoc """
-  ExUnit.DocTest implements functionality similar to [Python's
-  doctest](https://docs.python.org/2/library/doctest.html).
+  Extract test cases from the documentation.
 
-  It allows us to generate tests from the code
-  examples in a module/function/macro's documentation.
-  To do this, invoke the `doctest/1` macro from within
-  your test case and ensure your code examples are written
-  according to the syntax and guidelines below.
+  Doctests allow us to generate tests from code examples found
+  in `@moduledoc` and `@doc` attributes. To do this, invoke the
+  `doctest/1` macro from within your test case and ensure your
+  code examples are written according to the syntax and guidelines
+  below.
 
   ## Syntax
 
   Every new test starts on a new line, with an `iex>` prefix.
-  Multiline expressions can be used by prefixing subsequent lines with either
-  `...>` (recommended) or `iex>`.
+  Multiline expressions can be used by prefixing subsequent lines
+  with either `...>` (recommended) or `iex>`.
 
-  The expected result should start at the next line after the `iex>`
-  or `...>` line(s) and is terminated either by a newline, new
-  `iex>` prefix or the end of the string literal.
+  The expected result should start the line after the `iex>`
+  and `...>` line(s) and be terminated by a newline.
 
   ## Examples
 
@@ -130,15 +128,15 @@ defmodule ExUnit.DocTest do
 
   You can also showcase expressions raising an exception, for example:
 
-      iex(1)> String.to_atom((fn -> 1 end).())
-      ** (ArgumentError) argument error
+      iex(1)> raise "some error"
+      ** (RuntimeError) some error
 
-  What DocTest will be looking for is a line starting with `** (` and it
-  will parse it accordingly to extract the exception name and message.
-  At this moment, the exception parser would make the parser treat the next
-  line as a start of a completely new expression (if it is prefixed with `iex>`)
-  or a no-op line with documentation. Thus, multiline messages are not
-  supported.
+  Doctest will looking for a line starting with `** (` and it will parse it
+  accordingly to extract the exception name and message. The exception parser
+  will consider all following lines part of the exception message until there
+  is an empty line or there is a new expression prefixed with `iex>`.
+  Therefore, it is possible to match on multiline messages as long as there
+  are no empty lines on the message itself.
 
   ## When not to use doctest
 
@@ -183,10 +181,12 @@ defmodule ExUnit.DocTest do
 
     * `:import` - when `true`, one can test a function defined in the module
       without referring to the module name. However, this is not feasible when
-      there is a clash with a module like Kernel. In these cases, `:import`
+      there is a clash with a module like `Kernel`. In these cases, `:import`
       should be set to `false` and a full `Module.function` construct should be
       used.
 
+    * `:tags` - a list of tags to apply to all generated doctests.
+
   ## Examples
 
       doctest MyModule, except: [:moduledoc, trick_fun: 1]
@@ -207,6 +207,10 @@ defmodule ExUnit.DocTest do
         file = ExUnit.DocTest.__file__(module)
 
         for {name, test} <- ExUnit.DocTest.__doctests__(module, opts) do
+          if tags = Keyword.get(opts, :tags) do
+            @tag tags
+          end
+
           @file file
           doc = ExUnit.Case.register_test(env, :doctest, name, [])
           def unquote(doc)(_), do: unquote(test)
@@ -277,7 +281,9 @@ defmodule ExUnit.DocTest do
       raise Error,
         line: line,
         module: module,
-        message: "multiple exceptions in one doctest case are not supported"
+        message:
+          "multiple exceptions in the same doctest example are not supported, " <>
+            "please separate your iex> prompts by multiple newlines to start new examples"
     end
 
     tests =
@@ -303,31 +309,17 @@ defmodule ExUnit.DocTest do
     doctest = "\n" <> formatted <> "\n" <> expected
     expr_ast = string_to_quoted(location, stack, expr, doctest) |> insert_assertions()
     expected_ast = string_to_quoted(location, stack, expected, doctest)
-    last_expr_ast = last_expr(expr_ast)
+    last_expr = Macro.to_string(last_expr(expr_ast))
 
     quote do
+      value = unquote(expr_ast)
       expected = unquote(expected_ast)
+      formatted = unquote(formatted)
+      last_expr = unquote(last_expr)
+      expected_expr = unquote(expected)
+      stack = unquote(stack)
 
-      case unquote(expr_ast) do
-        ^expected ->
-          :ok
-
-        actual ->
-          doctest = unquote(doctest)
-
-          expr =
-            "#{unquote(Macro.to_string(last_expr_ast))} === #{unquote(String.trim(expected))}"
-
-          error = [
-            message: "Doctest failed",
-            doctest: doctest,
-            expr: expr,
-            left: actual,
-            right: expected
-          ]
-
-          reraise ExUnit.AssertionError, error, unquote(stack)
-      end
+      ExUnit.DocTest.__test__(value, expected, formatted, last_expr, expected_expr, stack)
     end
   end
 
@@ -335,32 +327,17 @@ defmodule ExUnit.DocTest do
     doctest = "\n" <> formatted <> "\n" <> expected
     expr_ast = string_to_quoted(location, stack, expr, doctest) |> insert_assertions()
     expected_ast = string_to_quoted(location, stack, expected, doctest)
-    last_expr_ast = last_expr(expr_ast)
+    last_expr = Macro.to_string(last_expr(expr_ast))
 
     quote do
+      value = unquote(expr_ast)
       expected = unquote(expected_ast)
+      formatted = unquote(formatted)
+      last_expr = unquote(last_expr)
+      expected_expr = unquote(expected)
+      stack = unquote(stack)
 
-      case inspect(unquote(expr_ast)) do
-        ^expected ->
-          :ok
-
-        actual ->
-          doctest = unquote(doctest)
-
-          expr =
-            "inspect(#{unquote(Macro.to_string(last_expr_ast))}) === " <>
-              "#{unquote(String.trim(expected))}"
-
-          error = [
-            message: "Doctest failed",
-            doctest: doctest,
-            expr: expr,
-            left: actual,
-            right: expected
-          ]
-
-          reraise ExUnit.AssertionError, error, unquote(stack)
-      end
+      ExUnit.DocTest.__inspect__(value, expected, formatted, last_expr, expected_expr, stack)
     end
   end
 
@@ -370,44 +347,93 @@ defmodule ExUnit.DocTest do
 
     quote do
       stack = unquote(stack)
-      exception = unquote(exception)
-      doctest = unquote(doctest)
       message = unquote(message)
+      formatted = unquote(formatted)
+      exception = unquote(exception)
+      ExUnit.DocTest.__error__(fn -> unquote(expr_ast) end, message, exception, formatted, stack)
+    end
+  end
 
+  @doc false
+  def __test__(value, expected, formatted, last_expr, expected_expr, stack) do
+    case value do
+      ^expected ->
+        :ok
+
+      _ ->
+        error = [
+          message: "Doctest failed",
+          doctest: "\n" <> formatted <> "\n" <> expected_expr,
+          expr: "#{last_expr} === #{String.trim(expected_expr)}",
+          left: value,
+          right: expected
+        ]
+
+        reraise ExUnit.AssertionError, error, stack
+    end
+  end
+
+  @doc false
+  def __inspect__(value, expected, formatted, last_expr, expected_expr, parent_stack) do
+    result =
       try do
-        unquote(expr_ast)
+        inspect(value, safe: false)
       rescue
-        error ->
-          actual_exception = error.__struct__
-          actual_message = Exception.message(error)
-
-          message =
-            cond do
-              actual_exception != exception ->
-                "Doctest failed: expected exception #{inspect(exception)} but got " <>
-                  "#{inspect(actual_exception)} with message #{inspect(actual_message)}"
-
-              actual_message != message ->
-                "Doctest failed: wrong message for #{inspect(actual_exception)}\n" <>
-                  "expected:\n" <>
-                  "  #{inspect(message)}\n" <>
-                  "actual:\n" <> "  #{inspect(actual_message)}"
-
-              true ->
-                nil
-            end
-
-          if message do
-            reraise ExUnit.AssertionError, [message: message, doctest: doctest], stack
-          end
+        e ->
+          stack = Enum.drop(__STACKTRACE__, 1)
+          {[message: Exception.message(e)], ExUnit.Runner.prune_stacktrace(stack)}
       else
-        _ ->
-          message =
-            "Doctest failed: expected exception #{inspect(exception)} but nothing was raised"
-
-          error = [message: message, doctest: doctest]
-          reraise ExUnit.AssertionError, error, stack
+        ^expected -> :ok
+        actual -> {[left: actual, right: expected, message: "Doctest failed"], []}
       end
+
+    case result do
+      :ok ->
+        :ok
+
+      {extra, stack} ->
+        doctest = "\n" <> formatted <> "\n" <> expected_expr
+        expr = "inspect(#{last_expr}) === #{String.trim(expected_expr)}"
+        error = [doctest: doctest, expr: expr] ++ extra
+        reraise ExUnit.AssertionError, error, stack ++ parent_stack
+    end
+  end
+
+  @doc false
+  def __error__(fun, message, exception, formatted, stack) do
+    try do
+      fun.()
+    rescue
+      error ->
+        actual_exception = error.__struct__
+        actual_message = Exception.message(error)
+
+        failed =
+          cond do
+            actual_exception != exception ->
+              "Doctest failed: expected exception #{inspect(exception)} but got " <>
+                "#{inspect(actual_exception)} with message #{inspect(actual_message)}"
+
+            actual_message != message ->
+              "Doctest failed: wrong message for #{inspect(actual_exception)}\n" <>
+                "expected:\n" <>
+                "  #{inspect(message)}\n" <>
+                "actual:\n" <> "  #{inspect(actual_message)}"
+
+            true ->
+              nil
+          end
+
+        if failed do
+          doctest = "\n" <> formatted <> "\n** (#{inspect(exception)}) #{inspect(message)}"
+          reraise ExUnit.AssertionError, [message: failed, doctest: doctest], stack
+        end
+    else
+      _ ->
+        doctest = "\n" <> formatted <> "\n** (#{inspect(exception)}) #{inspect(message)}"
+        failed = "Doctest failed: expected exception #{inspect(exception)} but nothing was raised"
+        error = [message: failed, doctest: doctest]
+        reraise ExUnit.AssertionError, error, stack
     end
   end
 
@@ -474,22 +500,18 @@ defmodule ExUnit.DocTest do
   defp explain_docs_error({:invalid_chunk, _}),
     do: "The documentation chunk in the module is invalid"
 
-  defp extract_from_moduledoc(_, doc, _module) when doc in [:none, :hidden], do: []
-
   defp extract_from_moduledoc(annotation, %{"en" => doc}, module) do
     for test <- extract_tests(:erl_anno.line(annotation), doc, module) do
       normalize_test(test, :moduledoc)
     end
   end
 
+  defp extract_from_moduledoc(_, _doc, _module), do: []
+
   defp extract_from_docs(docs, module) do
     for doc <- docs, doc <- extract_from_doc(doc, module), do: doc
   end
 
-  defp extract_from_doc({{kind, _, _}, _, _, doc, _}, _module)
-       when kind not in [:function, :macro] or doc in [:none, :hidden],
-       do: []
-
   defp extract_from_doc({{_, name, arity}, annotation, _, %{"en" => doc}, _}, module) do
     line = :erl_anno.line(annotation)
 
@@ -498,6 +520,9 @@ defmodule ExUnit.DocTest do
     end
   end
 
+  defp extract_from_doc(_doc, _module),
+    do: []
+
   defp extract_tests(line_no, doc, module) do
     all_lines = String.split(doc, "\n", trim: false)
     lines = adjust_indent(all_lines, line_no + 1, module)
