@@ -10,9 +10,9 @@ defmodule ExUnit.Formatter do
     * `{:suite_started, opts}` -
       the suite has started with the specified options to the runner.
 
-    * `{:suite_finished, run_us, load_us}` -
-      the suite has finished. `run_us` and `load_us` are the run and load
-      times in microseconds respectively.
+    * `{:suite_finished, times_us}` -
+      the suite has finished. Returns several measurements in microseconds
+      for running the suite. See `t:times_us` for more information.
 
     * `{:module_started, test_module}` -
       a test module has started. See `ExUnit.TestModule` for details.
@@ -26,14 +26,18 @@ defmodule ExUnit.Formatter do
     * `{:test_finished, test}` -
       a test has finished. See `ExUnit.Test` for details.
 
+    * `{:sigquit, [test | test_module]}` -
+      the VM is going to shutdown. It receives the test cases (or test
+      module in case of `setup_all`) still running.
+
   The formatter will also receive the following events but they are deprecated
   and should be ignored:
 
     * `{:case_started, test_module}` -
-      a test module has started. See `ExUnit.TestCase` for details.
+      a test module has started. See `ExUnit.TestModule` for details.
 
     * `{:case_finished, test_module}` -
-      a test module has finished. See `ExUnit.TestCase` for details.
+      a test module has finished. See `ExUnit.TestModule` for details.
 
   The full ExUnit configuration is passed as the argument to `c:GenServer.init/1` callback when the
   formatters are started. If you need to do runtime configuration of a
@@ -44,8 +48,24 @@ defmodule ExUnit.Formatter do
 
   @type id :: term
   @type test :: ExUnit.Test.t()
-  @type run_us :: pos_integer
-  @type load_us :: pos_integer | nil
+
+  @typedoc """
+  The times spent on several parts of the test suite.
+
+  The following properties can be computed:
+
+      sync = run - (async || 0)
+      total = run + (load || 0)
+
+  `async` is nil when there are no async tests.
+  `load` is nil when the test suite is running and loading
+  tests concurrently.
+  """
+  @type times_us :: %{
+          run: pos_integer,
+          async: pos_integer | nil,
+          load: pos_integer | nil
+        }
 
   import Exception, only: [format_stacktrace_entry: 1, format_file_line: 3]
 
@@ -54,43 +74,48 @@ defmodule ExUnit.Formatter do
 
   @counter_padding "     "
   @mailbox_label_padding @counter_padding <> "  "
+  @formatter_exceptions [ExUnit.AssertionError, FunctionClauseError]
   @no_value ExUnit.AssertionError.no_value()
 
   @doc """
   Formats time taken running the test suite.
 
-  It receives the time spent running the tests and
-  optionally the time spent loading the test suite.
-
   ## Examples
 
-      iex> format_time(10000, nil)
-      "Finished in 0.01 seconds"
+      iex> format_times(%{run: 10000, async: nil, load: nil})
+      "Finished in 0.01 seconds (0.00s async, 0.01s sync)"
 
-      iex> format_time(10000, 20000)
-      "Finished in 0.03 seconds (0.02s on load, 0.01s on tests)"
+      iex> format_times(%{run: 10000, async: nil, load: 20000})
+      "Finished in 0.03 seconds (0.02s on load, 0.00s async, 0.01s sync)"
 
-      iex> format_time(10000, 200_000)
-      "Finished in 0.2 seconds (0.2s on load, 0.01s on tests)"
+      iex> format_times(%{run: 10000, async: nil, load: 200_000})
+      "Finished in 0.2 seconds (0.2s on load, 0.00s async, 0.01s sync)"
+
+      iex> format_times(%{run: 100_000, async: 50000, load: 200_000})
+      "Finished in 0.3 seconds (0.2s on load, 0.05s async, 0.05s sync)"
 
   """
-  @spec format_time(run_us, load_us) :: String.t()
-  def format_time(run_us, nil) do
-    "Finished in #{run_us |> normalize_us |> format_us} seconds"
-  end
-
-  def format_time(run_us, load_us) do
-    run_us = run_us |> normalize_us
-    load_us = load_us |> normalize_us
+  @spec format_times(times_us) :: String.t()
+  def format_times(times) do
+    run_us = normalize_us(times.run)
+    load_us = normalize_us(times.load)
+    async_us = normalize_us(times.async)
+    sync_us = run_us - async_us
     total_us = run_us + load_us
 
+    maybe_load =
+      if times.load do
+        "#{format_us(load_us)}s on load, "
+      else
+        ""
+      end
+
     "Finished in #{format_us(total_us)} seconds " <>
-      "(#{format_us(load_us)}s on load, #{format_us(run_us)}s on tests)"
+      "(#{maybe_load}#{format_us(async_us)}s async, #{format_us(sync_us)}s sync)"
   end
 
-  defp normalize_us(us) do
-    div(us, 10000)
-  end
+  defp normalize_us(nil), do: 0
+  defp normalize_us(us), do: div(us, 10000)
 
   defp format_us(us) do
     if us < 10 do
@@ -101,6 +126,12 @@ defmodule ExUnit.Formatter do
     end
   end
 
+  # Deprecate me on Elixir v1.16
+  @doc false
+  def format_time(run, load) do
+    format_times(%{run: run, load: load, async: nil})
+  end
+
   @doc """
   Formats filters used to constrain cases to be run.
 
@@ -136,10 +167,10 @@ defmodule ExUnit.Formatter do
 
   @doc false
   def format_assertion_error(%ExUnit.AssertionError{} = struct) do
-    format_assertion_error(%{}, struct, [], :infinity, fn _, msg -> msg end, "")
+    format_exception(%{}, struct, [], :infinity, fn _, msg -> msg end, "") |> elem(0)
   end
 
-  defp format_assertion_error(test, struct, stack, width, formatter, counter_padding) do
+  defp format_exception(test, %ExUnit.AssertionError{} = struct, stack, width, formatter, pad) do
     label_padding_size = if has_value?(struct.right), do: 7, else: 6
     padding_size = label_padding_size + byte_size(@counter_padding)
 
@@ -148,16 +179,27 @@ defmodule ExUnit.Formatter do
         do: &pad_multiline(&1, padding_size),
         else: &code_multiline(&1, padding_size)
 
-    [
-      note: if_value(struct.message, &format_message(&1, formatter)),
-      doctest: if_value(struct.doctest, &pad_multiline(&1, 2 + byte_size(@counter_padding))),
-      code: if_value(struct.expr, code_multiline),
-      code: unless_value(struct.expr, fn -> get_code(test, stack) || @no_value end),
-      arguments: if_value(struct.args, &format_args(&1, width))
-    ]
-    |> Kernel.++(format_context(struct, formatter, padding_size, width))
-    |> format_meta(formatter, counter_padding, label_padding_size)
-    |> IO.iodata_to_binary()
+    formatted =
+      [
+        note: if_value(struct.message, &format_message(&1, formatter)),
+        doctest: if_value(struct.doctest, &pad_multiline(&1, 2 + byte_size(@counter_padding))),
+        code: if_value(struct.expr, code_multiline),
+        code: unless_value(struct.expr, fn -> get_code(test, stack) || @no_value end),
+        arguments: if_value(struct.args, &format_args(&1, width))
+      ]
+      |> Kernel.++(format_context(struct, formatter, padding_size, width))
+      |> format_meta(formatter, pad, label_padding_size)
+      |> IO.iodata_to_binary()
+
+    {formatted, stack}
+  end
+
+  defp format_exception(test, %FunctionClauseError{} = struct, stack, _width, formatter, _pad) do
+    {blamed, stack} = Exception.blame(:error, struct, stack)
+    banner = Exception.format_banner(:error, struct)
+    blamed = FunctionClauseError.blame(blamed, &inspect/1, &blame_match(&1, &2, formatter))
+    message = error_info(banner, formatter) <> "\n" <> pad(String.trim_leading(blamed, "\n"))
+    {message <> format_code(test, stack, formatter), stack}
   end
 
   @doc false
@@ -179,30 +221,48 @@ defmodule ExUnit.Formatter do
       end)
   end
 
-  defp format_kind_reason(
-         test,
-         :error,
-         %ExUnit.AssertionError{} = struct,
-         stack,
-         width,
-         formatter
-       ) do
-    {format_assertion_error(test, struct, stack, width, formatter, @counter_padding), stack}
+  defp format_kind_reason(test, :error, %mod{} = struct, stack, width, formatter)
+       when mod in @formatter_exceptions do
+    format_exception(test, struct, stack, width, formatter, @counter_padding)
   end
 
-  defp format_kind_reason(test, :error, %FunctionClauseError{} = struct, stack, _width, formatter) do
-    {blamed, stack} = Exception.blame(:error, struct, stack)
-    banner = Exception.format_banner(:error, struct)
-    blamed = FunctionClauseError.blame(blamed, &inspect/1, &blame_match(&1, &2, formatter))
-    message = error_info(banner, formatter) <> "\n" <> pad(String.trim_leading(blamed, "\n"))
-    {message <> format_code(test, stack, formatter), stack}
+  defp format_kind_reason(test, kind, reason, stack, width, formatter) do
+    case linked_or_trapped_exit(kind, reason) do
+      {header, wrapped_reason, wrapped_stack} ->
+        struct = Exception.normalize(:error, wrapped_reason, wrapped_stack)
+
+        {formatted_reason, wrapped_stack} =
+          format_exception(test, struct, wrapped_stack, width, formatter, @counter_padding)
+
+        formatted_stack = format_stacktrace(wrapped_stack, test.module, test.name, formatter)
+        {error_info(header, formatter) <> pad(formatted_reason <> formatted_stack), stack}
+
+      :error ->
+        {reason, stack} = Exception.blame(kind, reason, stack)
+        message = error_info(Exception.format_banner(kind, reason), formatter)
+        {message <> format_code(test, stack, formatter), stack}
+    end
   end
 
-  defp format_kind_reason(test, kind, reason, stack, _width, formatter) do
-    message = error_info(Exception.format_banner(kind, reason), formatter)
-    {message <> format_code(test, stack, formatter), stack}
+  defp linked_or_trapped_exit({:EXIT, pid}, {reason, [_ | _] = stack})
+       when reason.__struct__ in @formatter_exceptions
+       when reason == :function_clause do
+    {"** (EXIT from #{inspect(pid)}) an exception was raised:\n", reason, stack}
   end
 
+  defp linked_or_trapped_exit(:exit, {{reason, [_ | _] = stack}, {mod, fun, args}})
+       when is_atom(mod) and is_atom(fun) and is_list(args) and
+              reason.__struct__ in @formatter_exceptions
+       when is_atom(mod) and is_atom(fun) and is_list(args) and reason == :function_clause do
+    {
+      "** (exit) exited in: #{Exception.format_mfa(mod, fun, args)}\n   ** (EXIT) an exception was raised:",
+      reason,
+      stack
+    }
+  end
+
+  defp linked_or_trapped_exit(_kind, _reason), do: :error
+
   defp format_code(test, stack, formatter) do
     if snippet = get_code(test, stack) do
       "     " <> formatter.(:extra_info, "code: ") <> snippet <> "\n"
@@ -386,9 +446,6 @@ defmodule ExUnit.Formatter do
 
         {left, right}
 
-      nil when formatter == :expr ->
-        {if_value(left, inspect), if_value(right, inspect)}
-
       nil ->
         {if_value(left, &code_multiline(&1, padding_size)), if_value(right, inspect)}
     end
