@@ -5,7 +5,7 @@ defmodule ExUnitTest do
 
   import ExUnit.CaptureIO
 
-  test "supports many runs" do
+  test "supports many runs and loads" do
     defmodule SampleTest do
       use ExUnit.Case
 
@@ -18,12 +18,72 @@ defmodule ExUnitTest do
       end
     end
 
-    ExUnit.Server.modules_loaded()
-    configure_and_reload_on_exit(colors: [enabled: false])
+    configure_and_reload_on_exit([])
 
     assert capture_io(fn ->
              assert ExUnit.run() == %{failures: 2, skipped: 0, total: 2, excluded: 0}
            end) =~ "\n2 tests, 2 failures\n"
+
+    ExUnit.Server.modules_loaded()
+
+    assert capture_io(fn ->
+             assert ExUnit.async_run() |> ExUnit.await_run() ==
+                      %{failures: 0, skipped: 0, total: 0, excluded: 0}
+           end) =~ "\n0 failures\n"
+  end
+
+  test "prints aborted runs on sigquit", config do
+    Process.register(self(), :aborted_on_sigquit)
+    line = __ENV__.line + 5
+
+    defmodule SleepOnTest do
+      use ExUnit.Case, async: true
+
+      test "true" do
+        send(:aborted_on_sigquit, :sleeping)
+        Process.sleep(:infinity)
+      end
+    end
+
+    defmodule SleepOnSetupAll do
+      use ExUnit.Case, async: true
+
+      setup_all do
+        send(:aborted_on_sigquit, :sleeping)
+        Process.sleep(:infinity)
+      end
+
+      test "true", do: :ok
+    end
+
+    configure_and_reload_on_exit(max_cases: 8)
+
+    result =
+      capture_io(fn ->
+        {pid, ref} = spawn_monitor(fn -> ExUnit.run() end)
+        assert_receive :sleeping
+        assert_receive :sleeping
+
+        # We are testing implementation details but since this involves
+        # the signal handler, it is truly the only way to test it.
+        send(pid, {ref, self(), :sigquit})
+
+        receive do
+          ^ref -> :ok
+        end
+      end)
+
+    assert result =~ "Aborting test suite, the following have not completed:"
+    assert result =~ ~r"\* ExUnitTest.SleepOnSetupAll \[.*test/ex_unit_test.exs\]"
+    assert result =~ ~r"\* test true \[.*test/ex_unit_test.exs:#{line}\]"
+
+    assert result =~ """
+           Showing results so far...
+
+           0 failures
+
+           Randomized with seed 0
+           """
   end
 
   test "doesn't hang on exits" do
@@ -39,8 +99,7 @@ defmodule ExUnitTest do
       end
     end
 
-    ExUnit.Server.modules_loaded()
-    configure_and_reload_on_exit(colors: [enabled: false])
+    configure_and_reload_on_exit([])
 
     assert capture_io(fn ->
              assert ExUnit.run() == %{failures: 1, skipped: 0, total: 1, excluded: 0}
@@ -57,8 +116,6 @@ defmodule ExUnitTest do
       end
     end
 
-    ExUnit.Server.modules_loaded()
-
     output = capture_io(fn -> ExUnit.run() end)
     assert output =~ "** (ExUnit.TimeoutError) test timed out after 10ms"
     assert output =~ ~r"\(elixir #{System.version()}\) lib/process\.ex:\d+: Process\.sleep/1"
@@ -74,7 +131,6 @@ defmodule ExUnitTest do
     end
 
     ExUnit.configure(timeout: 5)
-    ExUnit.Server.modules_loaded()
     output = capture_io(fn -> ExUnit.run() end)
     assert output =~ "** (ExUnit.TimeoutError) test timed out after 5ms"
   after
@@ -100,38 +156,24 @@ defmodule ExUnitTest do
       end
     end
 
-    configure_and_reload_on_exit([])
-    ExUnit.start(slowest: 2)
-    ExUnit.Server.modules_loaded()
+    configure_and_reload_on_exit(slowest: 2)
 
     output = capture_io(fn -> ExUnit.run() end)
-
     assert output =~ ~r"Top 2 slowest \(\d+\.\d+s\), \d+.\d% of total time:"
     assert output =~ ~r"\* test slowest \(.+ms\)"
     assert output =~ ~r"\* test delayed \(.+ms\)"
   end
 
   test "sets max cases to one with trace enabled" do
-    configure_and_reload_on_exit([])
-    ExUnit.start(trace: true, max_cases: 10, autorun: false)
+    configure_and_reload_on_exit(trace: true, max_cases: 10)
     config = ExUnit.configuration()
     assert config[:trace]
     assert config[:max_cases] == 1
     assert config[:timeout] == 60000
   end
 
-  test "does not set timeout to infinity and the max cases to 1 with trace disabled" do
-    configure_and_reload_on_exit([])
-    ExUnit.start(trace: false, autorun: false)
-    config = ExUnit.configuration()
-    refute config[:trace]
-    assert config[:max_cases] == System.schedulers_online() * 2
-    assert config[:timeout] == 60000
-  end
-
   test "sets trace when slowest is enabled" do
-    configure_and_reload_on_exit([])
-    ExUnit.start(slowest: 10, max_cases: 10, autorun: false)
+    configure_and_reload_on_exit(slowest: 10, max_cases: 10)
     config = ExUnit.configuration()
     assert config[:trace]
     assert config[:slowest] == 10
@@ -231,7 +273,6 @@ defmodule ExUnitTest do
       end
     end
 
-    ExUnit.Server.modules_loaded()
     output = capture_io(&ExUnit.run/0)
     assert output =~ "[debug] two\n"
     refute output =~ "[debug] one\n"
@@ -266,8 +307,7 @@ defmodule ExUnitTest do
       end
     end)
 
-    ExUnit.Server.modules_loaded()
-    configure_and_reload_on_exit(colors: [enabled: false])
+    configure_and_reload_on_exit([])
 
     output =
       capture_io(fn ->
@@ -322,8 +362,7 @@ defmodule ExUnitTest do
       test "this is not implemented yet"
     end
 
-    ExUnit.Server.modules_loaded()
-    configure_and_reload_on_exit(colors: [enabled: false])
+    configure_and_reload_on_exit([])
 
     output =
       capture_io(fn ->
@@ -350,12 +389,10 @@ defmodule ExUnitTest do
       test "this will also raise", do: raise("oops")
     end
 
-    ExUnit.Server.modules_loaded()
     configure_and_reload_on_exit([])
 
     output =
       capture_io(fn ->
-        predictable_ex_unit_start([])
         assert ExUnit.run() == %{failures: 0, skipped: 2, total: 2, excluded: 0}
       end)
 
@@ -420,8 +457,7 @@ defmodule ExUnitTest do
       test "sample", do: :ok
     end
 
-    ExUnit.Server.modules_loaded()
-    configure_and_reload_on_exit(colors: [enabled: false])
+    configure_and_reload_on_exit([])
 
     output =
       capture_io(fn ->
@@ -442,8 +478,6 @@ defmodule ExUnitTest do
       test "sample", do: :ok
     end
 
-    ExUnit.Server.modules_loaded()
-
     output =
       capture_io(fn ->
         assert ExUnit.run() == %{failures: 1, skipped: 0, total: 1, excluded: 0}
@@ -463,16 +497,12 @@ defmodule ExUnitTest do
       test "sample", do: :ok
     end
 
-    ExUnit.Server.modules_loaded()
-
     capture_io(fn ->
       assert ExUnit.run() == %{failures: 0, skipped: 0, total: 1, excluded: 0}
     end)
   end
 
   test "seed is predictable and different for each test" do
-    configure_and_reload_on_exit(seed: 1, colors: [enabled: false])
-
     defmodule PredictableSeedTest do
       use ExUnit.Case, async: true
 
@@ -493,7 +523,7 @@ defmodule ExUnitTest do
       end
     end
 
-    ExUnit.Server.modules_loaded()
+    configure_and_reload_on_exit(seed: 1)
 
     assert capture_io(fn ->
              assert ExUnit.run() == %{failures: 0, skipped: 0, total: 3, excluded: 0}
@@ -527,12 +557,10 @@ defmodule ExUnitTest do
       test "excluded #{__ENV__.line}", do: assert(false)
     end
 
-    ExUnit.Server.modules_loaded()
     configure_and_reload_on_exit([])
 
     output =
       capture_io(fn ->
-        predictable_ex_unit_start([])
         assert ExUnit.run() == %{total: 6, failures: 4, excluded: 1, skipped: 1}
       end)
 
@@ -552,8 +580,6 @@ defmodule ExUnitTest do
         end
       end
 
-      ExUnit.Server.modules_loaded()
-
       ExUnit.after_suite(fn _ -> send(:after_suite_test_process, :first_after_suite) end)
       ExUnit.after_suite(fn result -> send(:after_suite_test_process, result) end)
       ExUnit.after_suite(fn _ -> send(:after_suite_test_process, :third_after_suite) end)
@@ -606,12 +632,10 @@ defmodule ExUnitTest do
         test "excluded #{__ENV__.line}", do: assert(false)
       end
 
-      ExUnit.Server.modules_loaded()
-      configure_and_reload_on_exit([])
+      configure_and_reload_on_exit(max_failures: 2)
 
       output =
         capture_io(fn ->
-          predictable_ex_unit_start(max_failures: 2)
           assert ExUnit.run() == %{total: 6, failures: 2, skipped: 1, excluded: 1}
         end)
 
@@ -639,12 +663,10 @@ defmodule ExUnitTest do
         test "excluded #{__ENV__.line}", do: assert(false)
       end
 
-      ExUnit.Server.modules_loaded()
-      configure_and_reload_on_exit([])
+      configure_and_reload_on_exit(max_failures: 3)
 
       output =
         capture_io(fn ->
-          predictable_ex_unit_start(max_failures: 3)
           assert ExUnit.run() == %{total: 8, excluded: 2, failures: 2, skipped: 1}
         end)
 
@@ -675,12 +697,10 @@ defmodule ExUnitTest do
         test "excluded #{__ENV__.line}", do: assert(true)
       end
 
-      ExUnit.Server.modules_loaded()
-      configure_and_reload_on_exit([])
+      configure_and_reload_on_exit(max_failures: 2)
 
       output =
         capture_io(fn ->
-          predictable_ex_unit_start(max_failures: 2)
           assert ExUnit.run() == %{total: 7, failures: 2, excluded: 2, skipped: 2}
         end)
 
@@ -711,12 +731,10 @@ defmodule ExUnitTest do
         test "excluded #{__ENV__.line}", do: assert(false)
       end
 
-      ExUnit.Server.modules_loaded()
-      configure_and_reload_on_exit([])
+      configure_and_reload_on_exit(max_failures: 2)
 
       output =
         capture_io(fn ->
-          predictable_ex_unit_start(max_failures: 2)
           assert ExUnit.run() == %{total: 4, failures: 2, excluded: 1, skipped: 1}
         end)
 
@@ -740,12 +758,10 @@ defmodule ExUnitTest do
         test "error", do: assert(false)
       end
 
-      ExUnit.Server.modules_loaded()
-      configure_and_reload_on_exit([])
+      configure_and_reload_on_exit(max_failures: 1)
 
       output =
         capture_io(fn ->
-          predictable_ex_unit_start(max_failures: 1)
           assert ExUnit.run() == %{total: 1, failures: 1, excluded: 0, skipped: 0}
         end)
 
@@ -753,10 +769,27 @@ defmodule ExUnitTest do
       assert output =~ "\n1 test, 1 failure\n"
 
       capture_io(fn ->
-        ExUnit.Server.modules_loaded()
         assert ExUnit.run() == %{total: 0, failures: 0, excluded: 0, skipped: 0}
       end)
     end
+
+    test "warns and errors when context is not a map" do
+      assert capture_io(:stderr, fn ->
+               defmodule ContextTest do
+                 use ExUnit.Case, async: true
+
+                 test "I made a typo", conn: conn do
+                   assert true
+                 end
+               end
+             end) =~ "test context is always a map. The pattern \"[conn: conn]\" will never match"
+
+      configure_and_reload_on_exit([])
+
+      assert capture_io(fn ->
+               assert ExUnit.run() == %{failures: 1, skipped: 0, total: 1, excluded: 0}
+             end) =~ "** (FunctionClauseError) no function clause matching"
+    end
   end
 
   ##  Helpers
@@ -784,18 +817,17 @@ defmodule ExUnitTest do
 
   defp configure_and_reload_on_exit(opts) do
     old_opts = ExUnit.configuration()
-    ExUnit.configure(opts)
+
+    ExUnit.configure(
+      Keyword.merge(
+        [autorun: false, seed: 0, colors: [enabled: false], exclude: [:exclude]],
+        opts
+      )
+    )
 
     on_exit(fn -> ExUnit.configure(old_opts) end)
   end
 
-  # Runs ExUnit.start/1 with common options needed for predictability
-  defp predictable_ex_unit_start(options) do
-    ExUnit.start(
-      options ++ [autorun: false, seed: 0, colors: [enabled: false], exclude: [:exclude]]
-    )
-  end
-
   defp max_failures_reached_msg() do
     "--max-failures reached, aborting test suite"
   end
