@@ -29,6 +29,17 @@ defmodule IEx do
   to work. `--werl` may be permanently enabled by setting the `IEX_WITH_WERL`
   environment variable.
 
+  ## Coloring
+
+  Coloring is enabled by default on most Unix terminals. They are also
+  available on Windows consoles from Windows 10, although it must be
+  explicitly enabled for the current user in the registry by running
+  the following command:
+
+      reg add HKCU\Console /v VirtualTerminalLevel /t REG_DWORD /d 1
+
+  After running the command above, you must restart your current console.
+
   ## Shell history
 
   It is possible to get shell history by passing some options that enable it
@@ -87,42 +98,43 @@ defmodule IEx do
 
   ## Pasting multiline expressions into IEx
 
-  IEx evaluates its input line by line in an eagerly fashion which means
-  that if at the end of a line the code seen so far is a complete expression
-  IEx will evaluate it at that point. This behaviour may produce errors for
-  expressions that have been formatted across multiple lines which is often
-  the case for piped expressions. Consider the following expression using
-  the `|>/2` operator:
+  IEx evaluates its input line by line in an eager fashion. If at the end of a
+  line the code seen so far is a complete expression, IEx will evaluate it at
+  that point.
 
-      iex(1)> [1, [2], 3] |> List.flatten()
-      [1, 2, 3]
+      iex(1)> [1, [2], 3]
+      [1, [2], 3]
 
-  When written in multiline form and pasted into IEx this valid expression
-  produces a syntax error:
+  To prevent this behaviour breaking valid code where the subsequent line
+  begins with a binary operator, such as `|>/2` or `++/2` , IEx automatically
+  treats such lines as if they were prepended with `IEx.Helpers.v/0`, which
+  returns the value of the previous expression, if available.
 
       iex(1)> [1, [2], 3]
       [1, [2], 3]
       iex(2)> |> List.flatten()
-      ** (SyntaxError) iex:2: syntax error before: '|>'
-
-  As IEx evaluates its input line by line, it will first encounter
-  `[1, [2], 3]`. As a list is a valid expression, IEx will evaluate
-  it immediately before looking at the next input line. Only then
-  will IEx attempt to evaluate the now incomplete expression
-  `|> List.flatten()`, which on its own is missing its left operand.
-  The evaluation thus fails with the above syntax error.
-
-  In order to help IEx understand that an expression consists of multiple
-  lines we can wrap it into parentheses:
-
-      iex(1)> (
-      ...(1)> [1, [2], 3]
-      ...(1)> |> List.flatten()
-      ...(1)> )
       [1, 2, 3]
 
-  Note that this not only works with single expressions but also with
-  arbitrary code blocks.
+  The above is equivalent to:
+
+      iex(1)> [1, [2], 3]
+      [1, [2], 3]
+      iex(2)> v() |> List.flatten()
+      [1, 2, 3]
+
+  If there are no previous expressions in the history, the pipe operator will
+  fail:
+
+      iex(1)> |> List.flatten()
+      ** (RuntimeError) v(-1) is out of bounds
+
+  Note however the above does not work for `+/2` and `-/2`, as they
+  are ambiguous with the unary `+/1` and `-/1`:
+
+      iex(1)> 1
+      1
+      iex(2)> + 2
+      2
 
   ## The BREAK menu
 
@@ -211,7 +223,7 @@ defmodule IEx do
   First of all, we can only connect to a shell if we give names
   both to the current shell and the shell we want to connect to.
 
-  Let's give it a try. First start a new shell:
+  Let's give it a try. First, start a new shell:
 
       $ iex --sname foo
       iex(foo@HOST)1>
@@ -273,12 +285,16 @@ defmodule IEx do
 
   When starting, IEx looks for a local `.iex.exs` file (located in the current
   working directory), then a global one (located at `~/.iex.exs`) and loads the
-  first one it finds (if any). The code in the loaded `.iex.exs` file is
-  evaluated in the shell's context. So, for instance, any modules that are
-  loaded or variables that are bound in the `.iex.exs` file will be available in the
-  shell after it has booted.
+  first one it finds (if any). Note the location of the `.iex.exs` files, both
+  in the current directory and the global one, are taken relative to the user
+  that started the application, not to the user that is connecting to the node in
+  case of remote IEx connections.
 
-  For example, take the following `.iex.exs` file:
+  The code in the loaded `.iex.exs` file is evaluated in the shell's context.
+  For instance, any modules that are loaded or variables that are bound in the
+  `.iex.exs` file will be available in the shell after it has booted.
+
+  Take the following `.iex.exs` file:
 
       # Load another ".iex.exs" file
       import_file("~/.iex.exs")
@@ -296,7 +312,7 @@ defmodule IEx do
   results in:
 
       $ iex
-      Erlang/OTP 21 [...]
+      Erlang/OTP 22 [...]
 
       hello world
       Interactive Elixir - press Ctrl+C to exit (type h() ENTER for help)
@@ -320,7 +336,7 @@ defmodule IEx do
   Now run the shell:
 
       $ iex
-      Erlang/OTP 21 [...]
+      Erlang/OTP 22 [...]
 
       Interactive Elixir - press Ctrl+C to exit (type h() ENTER for help)
       iex(1)> [1, 2, 3, 4, 5]
@@ -341,6 +357,7 @@ defmodule IEx do
     * `:continuation_prompt`
     * `:alive_prompt`
     * `:alive_continuation_prompt`
+    * `:parser`
 
   They are discussed individually in the sections below.
 
@@ -437,6 +454,18 @@ defmodule IEx do
     * `%prefix`  - a prefix given by `IEx.Server`
     * `%node`    - the name of the local node
 
+  ## Parser
+
+  This is an option determining the parser to use for IEx.
+
+  The parser is a "mfargs", which is a tuple with three elements:
+  the module name, the function name, and extra arguments to
+  be appended. The parser receives at least three arguments, the
+  current input as a string, the parsing options as a keyword list,
+  and the buffer as a string. It must return `{:ok, expr, buffer}`
+  or `{:incomplete, buffer}`.
+
+  If the parser raises, the buffer is reset to an empty string.
   """
   @spec configure(keyword()) :: :ok
   def configure(options) do
@@ -480,7 +509,7 @@ defmodule IEx do
 
   ANSI escapes in `string` are not processed in any way.
   """
-  @spec color(atom(), String.t()) :: String.t()
+  @spec color(atom(), iodata()) :: iodata()
   def color(color, string) do
     case IEx.Config.color(color) do
       nil ->
@@ -753,7 +782,7 @@ defmodule IEx do
   `IEx.break!` multiple times with different patterns, only the last
   pattern is kept.
 
-  Notice that, while patterns may be given to macros, remember that
+  Note that, while patterns may be given to macros, remember that
   macros receive ASTs as arguments, and not values. For example, if
   you try to break on a macro with the following pattern:
 
