@@ -12,7 +12,8 @@ defmodule IEx.Config do
     :continuation_prompt,
     :alive_prompt,
     :alive_continuation_prompt,
-    :width
+    :width,
+    :parser
   ]
 
   # Read API
@@ -58,6 +59,10 @@ defmodule IEx.Config do
     Application.get_env(:iex, :alive_continuation_prompt, alive_prompt())
   end
 
+  def parser() do
+    Application.get_env(:iex, :parser, {IEx.Evaluator, :parse, []})
+  end
+
   def color(color) do
     color(color, Application.get_env(:iex, :colors, []))
   end
@@ -119,10 +124,8 @@ defmodule IEx.Config do
 
   def ansi_docs() do
     colors = Application.get_env(:iex, :colors, [])
-
-    if enabled = colors_enabled?(colors) do
-      [width: width(), enabled: enabled] ++ colors
-    end
+    enabled = colors_enabled?(colors)
+    [width: width(), enabled: enabled] ++ colors
   end
 
   def inspect_opts() do
@@ -174,7 +177,7 @@ defmodule IEx.Config do
   end
 
   def handle_configure(tab, options) do
-    options = :lists.ukeysort(1, options)
+    Enum.each(options, &validate_option/1)
 
     configuration()
     |> Keyword.merge(options, &merge_option/3)
@@ -191,15 +194,19 @@ defmodule IEx.Config do
 
   defp merge_option(:colors, old, new) when is_list(new), do: Keyword.merge(old, new)
   defp merge_option(:inspect, old, new) when is_list(new), do: Keyword.merge(old, new)
-  defp merge_option(:history_size, _old, new) when is_integer(new), do: new
-  defp merge_option(:default_prompt, _old, new) when is_binary(new), do: new
-  defp merge_option(:continuation_prompt, _old, new) when is_binary(new), do: new
-  defp merge_option(:alive_prompt, _old, new) when is_binary(new), do: new
-  defp merge_option(:alive_continuation_prompt, _old, new) when is_binary(new), do: new
-  defp merge_option(:width, _old, new) when is_integer(new), do: new
+  defp merge_option(_key, _old, new), do: new
 
-  defp merge_option(key, _old, new) do
-    raise ArgumentError,
-          "invalid configuration or value for pair #{inspect(key)} - #{inspect(new)}"
+  defp validate_option({:colors, new}) when is_list(new), do: :ok
+  defp validate_option({:inspect, new}) when is_list(new), do: :ok
+  defp validate_option({:history_size, new}) when is_integer(new), do: :ok
+  defp validate_option({:default_prompt, new}) when is_binary(new), do: :ok
+  defp validate_option({:continuation_prompt, new}) when is_binary(new), do: :ok
+  defp validate_option({:alive_prompt, new}) when is_binary(new), do: :ok
+  defp validate_option({:alive_continuation_prompt, new}) when is_binary(new), do: :ok
+  defp validate_option({:width, new}) when is_integer(new), do: :ok
+  defp validate_option({:parser, tuple}) when tuple_size(tuple) == 3, do: :ok
+
+  defp validate_option(option) do
+    raise ArgumentError, "invalid configuration #{inspect(option)}"
   end
 end
