@@ -2,7 +2,7 @@ defmodule IEx.State do
   @moduledoc false
   # This state is exchanged between IEx.Server and
   # IEx.Evaluator which is why it is a struct.
-  defstruct cache: '', counter: 1, prefix: "iex", on_eof: :stop_evaluator
+  defstruct buffer: "", counter: 1, prefix: "iex", on_eof: :stop_evaluator, evaluator_options: []
   @type t :: %__MODULE__{}
 end
 
@@ -56,8 +56,8 @@ defmodule IEx.Server do
 
   defp shell_loop(opts, pid, ref) do
     receive do
-      {:take_over, take_pid, take_ref, take_identifier, take_opts} ->
-        if take_over?(take_pid, take_ref, take_identifier) do
+      {:take_over, take_pid, take_ref, take_location, take_whereami, take_opts} ->
+        if take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) do
           run_without_registration(take_opts)
         else
           shell_loop(opts, pid, ref)
@@ -119,7 +119,7 @@ defmodule IEx.Server do
     counter = state.counter
 
     {prompt_type, prefix} =
-      if state.cache != [] do
+      if state.buffer != "" do
         {:continuation_prompt, "..."}
       else
         {:prompt, state.prefix}
@@ -144,7 +144,7 @@ defmodule IEx.Server do
       # Triggered by pressing "i" as the job control switch
       {:input, ^input, {:error, :interrupted}} ->
         io_error("** (EXIT) interrupted")
-        loop(%{state | cache: ''}, evaluator, evaluator_ref)
+        loop(%{state | buffer: ""}, evaluator, evaluator_ref)
 
       # Triggered when IO dies while waiting for input
       {:input, ^input, {:error, :terminated}} ->
@@ -183,7 +183,7 @@ defmodule IEx.Server do
   # A take process may also happen if the evaluator dies,
   # then a new evaluator is created to replace the dead one.
   defp handle_take_over(
-         {:take_over, take_pid, take_ref, take_identifier, opts},
+         {:take_over, take_pid, take_ref, take_location, take_whereami, take_opts},
          state,
          evaluator,
          evaluator_ref,
@@ -191,16 +191,18 @@ defmodule IEx.Server do
          callback
        ) do
     cond do
-      evaluator == opts[:evaluator] ->
+      evaluator == take_opts[:evaluator] ->
+        IO.puts(IEx.color(:eval_interrupt, "Break reached: #{take_location}#{take_whereami}"))
+
         if take_over?(take_pid, take_ref, true) do
           kill_input(input)
-          loop(iex_state(opts), evaluator, evaluator_ref)
+          loop(iex_state(take_opts), evaluator, evaluator_ref)
         else
           callback.(state)
         end
 
-      take_over?(take_pid, take_ref, take_identifier) ->
-        rerun(opts, evaluator, evaluator_ref, input)
+      take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) ->
+        rerun(take_opts, evaluator, evaluator_ref, input)
 
       true ->
         callback.(state)
@@ -221,8 +223,8 @@ defmodule IEx.Server do
     Process.delete(:evaluator)
     Process.exit(evaluator, :kill)
     Process.demonitor(evaluator_ref, [:flush])
-    evaluator = start_evaluator([])
-    loop(%{state | cache: ''}, evaluator, Process.monitor(evaluator))
+    evaluator = start_evaluator(state.evaluator_options)
+    loop(%{state | buffer: ""}, evaluator, Process.monitor(evaluator))
   end
 
   defp handle_take_over(
@@ -240,8 +242,8 @@ defmodule IEx.Server do
     end
   end
 
-  defp handle_take_over({:respawn, evaluator}, _state, evaluator, evaluator_ref, input, _callback) do
-    rerun([], evaluator, evaluator_ref, input)
+  defp handle_take_over({:respawn, evaluator}, state, evaluator, evaluator_ref, input, _callback) do
+    rerun(state.evaluator_options, evaluator, evaluator_ref, input)
   end
 
   defp handle_take_over({:continue, evaluator}, state, evaluator, evaluator_ref, input, _callback) do
@@ -252,18 +254,18 @@ defmodule IEx.Server do
 
   defp handle_take_over(
          {:DOWN, evaluator_ref, :process, evaluator, :normal},
-         _state,
+         state,
          evaluator,
          evaluator_ref,
          input,
          _callback
        ) do
-    rerun([], evaluator, evaluator_ref, input)
+    rerun(state.evaluator_options, evaluator, evaluator_ref, input)
   end
 
   defp handle_take_over(
          {:DOWN, evaluator_ref, :process, evaluator, reason},
-         _state,
+         state,
          evaluator,
          evaluator_ref,
          input,
@@ -279,19 +281,21 @@ defmodule IEx.Server do
         io_error("** (IEx.Error) #{type} when printing EXIT message: #{inspect(detail)}")
     end
 
-    rerun([], evaluator, evaluator_ref, input)
+    rerun(state.evaluator_options, evaluator, evaluator_ref, input)
   end
 
   defp handle_take_over(_, state, _evaluator, _evaluator_ref, _input, callback) do
     callback.(state)
   end
 
-  defp take_over?(take_pid, take_ref, take_identifier) when is_binary(take_identifier) do
-    message = IEx.color(:eval_interrupt, "#{take_identifier}\nAllow? [Yn] ")
-    take_over?(take_pid, take_ref, yes?(IO.gets(:stdio, message)))
+  defp take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) do
+    evaluator = take_opts[:evaluator]
+    message = "Request to pry #{inspect(evaluator)} at #{take_location}#{take_whereami}"
+    interrupt = IEx.color(:eval_interrupt, "#{message}\nAllow? [Yn] ")
+    take_over?(take_pid, take_ref, yes?(IO.gets(:stdio, interrupt)))
   end
 
-  defp take_over?(take_pid, take_ref, take_response) do
+  defp take_over?(take_pid, take_ref, take_response) when is_boolean(take_response) do
     case IEx.Broker.respond(take_pid, take_ref, take_response) do
       :ok ->
         true
@@ -317,7 +321,12 @@ defmodule IEx.Server do
   defp iex_state(opts) do
     prefix = Keyword.get(opts, :prefix, "iex")
     on_eof = Keyword.get(opts, :on_eof, :stop_evaluator)
-    %IEx.State{prefix: prefix, on_eof: on_eof}
+
+    %IEx.State{
+      prefix: prefix,
+      on_eof: on_eof,
+      evaluator_options: Keyword.take(opts, [:dot_iex_path])
+    }
   end
 
   ## IO
