@@ -21,7 +21,7 @@ defmodule IEx.InteractionTest do
 
   test "invalid input" do
     assert capture_iex("if true do ) false end") =~
-             "** (SyntaxError) iex:1: unexpected token: ). The \"do\" at line 1 is missing terminator \"end\""
+             "** (SyntaxError) iex:1:12: unexpected token: ). The \"do\" at line 1 is missing terminator \"end\""
   end
 
   test "multiple vars" do
@@ -158,6 +158,18 @@ defmodule IEx.InteractionTest do
     assert content =~ ~r"\(elixir #{System.version()}\) lib/access\.ex:\d+: Access\.fetch/2"
   end
 
+  test "parser" do
+    defmodule EchoParser do
+      def parse(input, _opts, buffer) do
+        {:ok, input, buffer}
+      end
+    end
+
+    assert capture_iex("foo", parser: {EchoParser, :parse, []}) == "\"foo\""
+  after
+    IEx.configure(parser: {IEx.Evaluator, :parse, []})
+  end
+
   ## .iex file loading
 
   describe ".iex" do
@@ -166,31 +178,34 @@ defmodule IEx.InteractionTest do
                capture_iex("my_variable")
     end
 
-    test "single .iex" do
-      path = write_dot_iex!("dot-iex", "my_variable = 144")
+    @tag :tmp_dir
+    test "single .iex", %{tmp_dir: tmp_dir} do
+      path = write_dot_iex!(tmp_dir, "dot-iex", "my_variable = 144")
       assert capture_iex("my_variable", [], dot_iex_path: path) == "144"
     end
 
-    test "nested .iex" do
-      write_dot_iex!("dot-iex-1", "nested_var = 13\nimport IO")
-      path = write_dot_iex!("dot-iex", "import_file \"tmp/dot-iex-1\"\nmy_variable=14")
+    @tag :tmp_dir
+    test "nested .iex", %{tmp_dir: tmp_dir} do
+      write_dot_iex!(tmp_dir, "dot-iex-1", "nested_var = 13\nimport IO")
+
+      path =
+        write_dot_iex!(tmp_dir, "dot-iex", "import_file \"#{tmp_dir}/dot-iex-1\"\nmy_variable=14")
 
       input = "nested_var\nmy_variable\nputs \"hello\""
       assert capture_iex(input, [], dot_iex_path: path) == "13\n14\nhello\n:ok"
     end
 
-    test "malformed .iex" do
-      path = write_dot_iex!("dot-iex", "malformed")
+    @tag :tmp_dir
+    test "malformed .iex", %{tmp_dir: tmp_dir} do
+      path = write_dot_iex!(tmp_dir, "dot-iex", "malformed")
 
       assert capture_iex("1 + 2", [], dot_iex_path: path) =~
                "dot-iex:1: undefined function malformed/0"
     end
   end
 
-  defp write_dot_iex!(name, contents) do
-    dir = "#{__DIR__}/../../tmp"
-    File.mkdir_p!(dir)
-    path = Path.join(dir, name)
+  defp write_dot_iex!(tmp_dir, name, contents) do
+    path = Path.join(tmp_dir, name)
     File.write!(path, contents)
     path
   end
