@@ -34,8 +34,6 @@ defmodule Logger.Translator do
   """
   def translate(min_level, level, kind, message)
 
-  ## Erlang/OTP 21 and after
-
   def translate(min_level, _level, :report, {:logger, %{label: label} = report}) do
     case label do
       {:gen_server, :terminate} ->
@@ -357,15 +355,22 @@ defmodule Logger.Translator do
     []
   end
 
-  defp child_debug(:debug, restart_type: restart, shutdown: shutdown, child_type: type) do
-    ["\nRestart: ", inspect(restart), "\nShutdown: ", inspect(shutdown)] ++
-      ["\nType: ", inspect(type)]
+  defp child_debug(:debug, opts) do
+    for {key, value} <- opts do
+      child_debug_key(key, value)
+    end
   end
 
   defp child_debug(_min_level, _child) do
     []
   end
 
+  defp child_debug_key(:restart_type, value), do: ["\nRestart: " | inspect(value)]
+  defp child_debug_key(:shutdown, value), do: ["\nShutdown: " | inspect(value)]
+  defp child_debug_key(:child_type, value), do: ["\nType: " | inspect(value)]
+  defp child_debug_key(:significant, value), do: if(value, do: "\nSignificant: true", else: [])
+  defp child_debug_key(_, _), do: []
+
   defp report_crash(min_level, [[{:initial_call, initial_call} | crashed], linked]) do
     mfa = initial_call_to_mfa(initial_call)
     report_crash(min_level, crashed, [{:initial_call, mfa}], linked)
@@ -385,11 +390,11 @@ defmodule Logger.Translator do
     dictionary = crashed[:dictionary]
     reason = Exception.normalize(kind, reason, stack)
 
-    case Keyword.get(dictionary, :logger_metadata, {true, []}) do
-      {false, _} ->
+    case Keyword.get(dictionary, :logger_enabled, true) do
+      false ->
         :skip
 
-      {true, _user_metadata} ->
+      true ->
         user_metadata = Keyword.get(dictionary, :"$logger_metadata$", %{}) |> Map.to_list()
 
         msg =
@@ -550,7 +555,7 @@ defmodule Logger.Translator do
     # If this is already an exception (even an ErlangError), we format it as an
     # exception. Otherwise, we try to normalize it, and if it's normalized as an
     # ErlangError we instead format it as an exit.
-    if Exception.exception?(reason) do
+    if is_exception(reason) do
       {[?\n | Exception.format_banner(:error, reason, stacktrace)], reason}
     else
       case Exception.normalize(:error, reason, stacktrace) do
