@@ -78,22 +78,33 @@ defmodule Mix.CLI do
   defp run_task(name, args) do
     try do
       ensure_no_slashes(name)
+      # We must go through the task instead of invoking the module directly
+      # because projects like Nerves use this to invoke it early.
       Mix.Task.run("loadconfig")
       Mix.Task.run(name, args)
     rescue
       # We only rescue exceptions in the Mix namespace, all
       # others pass through and will explode on the users face
       exception ->
-        if Map.get(exception, :mix, false) and not Mix.debug?() do
-          mod = exception.__struct__ |> Module.split() |> Enum.at(0, "Mix")
-          Mix.shell().error("** (#{mod}) #{Exception.message(exception)}")
-          exit({:shutdown, 1})
-        else
-          reraise exception, __STACKTRACE__
+        case {Mix.debug?(), Map.get(exception, :mix, false)} do
+          {false, true} ->
+            simplified_exception(exception, 1)
+
+          {false, code} when code in 0..255 ->
+            simplified_exception(exception, code)
+
+          _ ->
+            reraise exception, __STACKTRACE__
         end
     end
   end
 
+  defp simplified_exception(%name{} = exception, code) do
+    mod = name |> Module.split() |> hd()
+    Mix.shell().error("** (#{mod}) #{Exception.message(exception)}")
+    exit({:shutdown, code})
+  end
+
   defp env_variable_activated?(name) do
     System.get_env(name) in ~w(1 true)
   end
@@ -144,7 +155,7 @@ defmodule Mix.CLI do
     path = Path.join(Mix.Utils.mix_config(), "config.exs")
 
     if File.regular?(path) do
-      Mix.Task.run("loadconfig", [path])
+      Mix.Tasks.Loadconfig.load_compile(path)
     end
   end
 
