@@ -64,7 +64,7 @@ defmodule Mix.Shell.Process do
   """
   def print_app do
     if name = Mix.Shell.printable_app_name() do
-      send(self(), {:mix_shell, :info, ["==> #{name}"]})
+      send(message_target(), {:mix_shell, :info, ["==> #{name}"]})
     end
   end
 
@@ -73,7 +73,7 @@ defmodule Mix.Shell.Process do
   """
   def info(message) do
     print_app()
-    send(self(), {:mix_shell, :info, [format(message)]})
+    send(message_target(), {:mix_shell, :info, [format(message)]})
     :ok
   end
 
@@ -82,7 +82,7 @@ defmodule Mix.Shell.Process do
   """
   def error(message) do
     print_app()
-    send(self(), {:mix_shell, :error, [format(message)]})
+    send(message_target(), {:mix_shell, :error, [format(message)]})
     :ok
   end
 
@@ -112,7 +112,7 @@ defmodule Mix.Shell.Process do
   """
   def prompt(message) do
     print_app()
-    send(self(), {:mix_shell, :prompt, [message]})
+    send(message_target(), {:mix_shell, :prompt, [message]})
 
     receive do
       {:mix_shell_input, :prompt, response} -> response
@@ -140,7 +140,7 @@ defmodule Mix.Shell.Process do
   """
   def yes?(message) do
     print_app()
-    send(self(), {:mix_shell, :yes?, [message]})
+    send(message_target(), {:mix_shell, :yes?, [message]})
 
     receive do
       {:mix_shell_input, :yes?, response} -> response
@@ -158,7 +158,14 @@ defmodule Mix.Shell.Process do
 
     Mix.Shell.cmd(command, opts, fn data ->
       if print_app?, do: print_app()
-      send(self(), {:mix_shell, :run, [data]})
+      send(message_target(), {:mix_shell, :run, [data]})
     end)
   end
+
+  defp message_target() do
+    case Process.get(:"$callers") do
+      [parent | _] -> parent
+      _ -> self()
+    end
+  end
 end
