@@ -1,6 +1,6 @@
 defmodule Mix.Tasks.Compile.Erlang do
   use Mix.Task.Compiler
-  import Mix.Compilers.Erlang
+  alias Mix.Compilers.Erlang
 
   @recursive true
   @manifest "compile.erlang"
@@ -65,9 +65,8 @@ defmodule Mix.Tasks.Compile.Erlang do
   defp do_run([], _, _, _), do: {:noop, []}
 
   defp do_run(files, opts, project, source_paths) do
-    include_path = to_erl_file(project[:erlc_include_path])
-    compile_path = to_erl_file(Mix.Project.compile_path(project))
-
+    include_path = Erlang.to_erl_file(project[:erlc_include_path])
+    compile_path = Erlang.to_erl_file(Mix.Project.compile_path(project))
     erlc_options = project[:erlc_options] || []
 
     unless is_list(erlc_options) do
@@ -79,31 +78,31 @@ defmodule Mix.Tasks.Compile.Erlang do
 
     erlc_options =
       Enum.map(erlc_options, fn
-        {kind, dir} when kind in [:i, :outdir] -> {kind, to_erl_file(dir)}
+        {kind, dir} when kind in [:i, :outdir] -> {kind, Erlang.to_erl_file(dir)}
         opt -> opt
       end)
 
     compile_path = Path.relative_to(compile_path, File.cwd!())
 
-    tuples =
-      files
-      |> scan_sources(include_path, source_paths)
-      |> sort_dependencies()
-      |> Enum.map(&annotate_target(&1, compile_path, opts[:force]))
+    {erls, tuples} =
+      Enum.unzip(scan_sources(files, include_path, source_paths, compile_path, opts))
 
-    Mix.Compilers.Erlang.compile(manifest(), tuples, opts, fn input, _output ->
+    opts = [parallel: MapSet.new(find_parallel(erls))] ++ opts
+
+    Erlang.compile(manifest(), tuples, opts, fn input, _output ->
       # We're purging the module because a previous compiler (for example, Phoenix)
       # might have already loaded the previous version of it.
       module = input |> Path.basename(".erl") |> String.to_atom()
       :code.purge(module)
       :code.delete(module)
 
-      file = to_erl_file(Path.rootname(input, ".erl"))
+      file = Erlang.to_erl_file(Path.rootname(input, ".erl"))
 
       case :compile.file(file, erlc_options) do
-        {:error, :badarg} ->
+        # TODO: Don't handle {:error, :badarg} when we require OTP 24
+        error when error == :error or error == {:error, :badarg} ->
           message =
-            "Compiling Erlang #{inspect(file)} failed with ArgumentError, probably because of invalid :erlc_options"
+            "Compiling Erlang file #{inspect(file)} failed, probably because of invalid :erlc_options"
 
           Mix.raise(message)
 
@@ -115,7 +114,6 @@ defmodule Mix.Tasks.Compile.Erlang do
 
   @impl true
   def manifests, do: [manifest()]
-
   defp manifest, do: Path.join(Mix.Project.manifest_path(), @manifest)
 
   @impl true
@@ -125,12 +123,21 @@ defmodule Mix.Tasks.Compile.Erlang do
 
   ## Internal helpers
 
-  defp scan_sources(files, include_path, source_paths) do
+  defp scan_sources(files, include_path, source_paths, compile_path, opts) do
     include_paths = [include_path | source_paths]
-    Enum.reduce(files, [], &scan_source(&2, &1, include_paths)) |> Enum.reverse()
+
+    files
+    |> Task.async_stream(&scan_source(&1, include_paths, compile_path, opts),
+      timeout: :infinity,
+      ordered: false
+    )
+    |> Enum.flat_map(fn
+      {:ok, {:ok, erl_file, target_tuple}} -> [{erl_file, target_tuple}]
+      {:ok, :error} -> []
+    end)
   end
 
-  defp scan_source(acc, file, include_paths) do
+  defp scan_source(file, include_paths, compile_path, opts) do
     erl_file = %{
       file: file,
       module: module_from_artifact(file),
@@ -140,12 +147,14 @@ defmodule Mix.Tasks.Compile.Erlang do
       invalid: false
     }
 
-    case :epp.parse_file(to_erl_file(file), include_paths, []) do
+    case :epp.parse_file(Erlang.to_erl_file(file), include_paths, []) do
       {:ok, forms} ->
-        [List.foldl(tl(forms), erl_file, &do_form(file, &1, &2)) | acc]
+        erl_file = List.foldl(tl(forms), erl_file, &do_form(file, &1, &2))
+        target_tuple = annotate_target(erl_file, compile_path, opts[:force])
+        {:ok, erl_file, target_tuple}
 
       {:error, _error} ->
-        acc
+        :error
     end
   end
 
@@ -175,40 +184,24 @@ defmodule Mix.Tasks.Compile.Erlang do
     end
   end
 
-  defp sort_dependencies(erls) do
-    graph = :digraph.new()
+  defp find_parallel(erls) do
+    serial = MapSet.new(find_dependencies(erls))
 
-    _ =
-      for erl <- erls do
-        :digraph.add_vertex(graph, erl.module, erl)
-      end
+    erls
+    |> Enum.reject(&(&1.module in serial))
+    |> Enum.map(& &1.file)
+  end
 
-    _ =
-      for erl <- erls do
-        _ = for b <- erl.behaviours, do: :digraph.add_edge(graph, b, erl.module)
+  defp find_dependencies(erls) do
+    Enum.flat_map(erls, fn erl ->
+      transforms =
+        Enum.flat_map(erl.compile, fn
+          {:parse_transform, transform} -> [transform]
+          _ -> []
+        end)
 
-        _ =
-          for c <- erl.compile do
-            case c do
-              {:parse_transform, transform} -> :digraph.add_edge(graph, transform, erl.module)
-              _ -> :ok
-            end
-          end
-
-        :ok
-      end
-
-    result =
-      case :digraph_utils.topsort(graph) do
-        false ->
-          erls
-
-        mods ->
-          for m <- mods, do: elem(:digraph.vertex(graph, m), 1)
-      end
-
-    :digraph.delete(graph)
-    result
+      transforms ++ erl.behaviours
+    end)
   end
 
   defp annotate_target(erl, compile_path, force) do
