@@ -72,7 +72,8 @@ defmodule Mix.Tasks.Deps do
       other project that depends on the current project won't be forced to
       use the optional dependency. However, if the other project includes
       the optional dependency on its own, the requirements and options
-      specified here will also be applied.
+      specified here will also be applied. Optional dependencies will _not_
+      be started by the application.
 
     * `:only` - the dependency is made available only in the given environments,
       useful when declaring dev- or test-only dependencies; by default the
@@ -144,6 +145,11 @@ defmodule Mix.Tasks.Deps do
       [locked at REF]
       STATUS
 
+  For dependencies satisfied by Hex, `REF` is the package checksum.
+
+  For dependencies satisfied by git, `REF` is the commit object name,
+  and may include branch or tag information.
+
   It supports the following options:
 
     * `--all` - lists all dependencies, regardless of specified environment
@@ -158,7 +164,9 @@ defmodule Mix.Tasks.Deps do
 
     shell = Mix.shell()
 
-    Enum.each(load_on_environment(loaded_opts), fn dep ->
+    load_on_environment(loaded_opts)
+    |> Enum.sort_by(& &1.app)
+    |> Enum.each(fn dep ->
       %Mix.Dep{scm: scm, manager: manager} = dep
       dep = check_lock(dep)
       extra = if manager, do: " (#{manager})", else: ""
