@@ -1,131 +1,4 @@
 defmodule Mix.Tasks.Test do
-  defmodule Cover do
-    @default_threshold 90
-
-    @moduledoc false
-
-    def start(compile_path, opts) do
-      Mix.shell().info("Cover compiling modules ...")
-      _ = :cover.stop()
-      _ = :cover.start()
-
-      case :cover.compile_beam_directory(compile_path |> to_charlist) do
-        results when is_list(results) ->
-          :ok
-
-        {:error, _} ->
-          Mix.raise("Failed to cover compile directory: " <> compile_path)
-      end
-
-      fn ->
-        Mix.shell().info("\nGenerating cover results ...\n")
-        {:result, ok, _fail} = :cover.analyse(:coverage, :line)
-
-        {module_results, totals} = gather_coverage(ok, :cover.modules())
-        module_results = Enum.sort_by(module_results, &percentage(elem(&1, 1)), &>=/2)
-
-        if summary_opts = Keyword.get(opts, :summary, true) do
-          console(module_results, totals, summary_opts)
-        end
-
-        html(module_results, opts)
-      end
-    end
-
-    defp gather_coverage(results, keep) do
-      keep_set = MapSet.new(keep)
-
-      # When gathering coverage results, we need to skip any
-      # entry with line equal to 0 as those are generated code.
-      #
-      # We may also have multiple entries on the same line.
-      # Each line is only considered once.
-      #
-      # We use ETS for performance, to avoid working with nested maps.
-      table = :ets.new(__MODULE__, [:set, :private])
-
-      try do
-        for {{module, line}, cov} <- results, module in keep_set, line != 0 do
-          case cov do
-            {1, 0} -> :ets.insert(table, {{module, line}, true})
-            {0, 1} -> :ets.insert_new(table, {{module, line}, false})
-          end
-        end
-
-        module_results =
-          for module <- keep,
-              results = read_module_cover_results(table, module),
-              do: {module, results}
-
-        total_covered = :ets.select_count(table, [{{:_, true}, [], [true]}])
-        total_not_covered = :ets.select_count(table, [{{:_, false}, [], [true]}])
-
-        {module_results, {total_covered, total_not_covered}}
-      after
-        :ets.delete(table)
-      end
-    end
-
-    defp read_module_cover_results(table, module) do
-      covered = :ets.select_count(table, [{{{module, :_}, true}, [], [true]}])
-      not_covered = :ets.select_count(table, [{{{module, :_}, false}, [], [true]}])
-      {covered, not_covered}
-    end
-
-    defp console(results, totals, true), do: console(results, totals, [])
-
-    defp console(results, totals, opts) when is_list(opts) do
-      Mix.shell().info("Percentage | Module")
-      Mix.shell().info("-----------|--------------------------")
-      results |> Enum.sort() |> Enum.each(&display(&1, opts))
-      Mix.shell().info("-----------|--------------------------")
-      display({"Total", totals}, opts)
-      Mix.shell().info("")
-    end
-
-    defp html(results, opts) do
-      output = opts[:output]
-      File.mkdir_p!(output)
-
-      for {mod, _} <- results do
-        {:ok, _} = :cover.analyse_to_file(mod, '#{output}/#{mod}.html', [:html])
-      end
-
-      Mix.shell().info([
-        "Generated HTML coverage results in '",
-        output,
-        "' directory\n"
-      ])
-    end
-
-    defp color(percentage, true), do: color(percentage, @default_threshold)
-    defp color(_, false), do: ""
-    defp color(percentage, threshold) when percentage > threshold, do: :green
-    defp color(_, _), do: :red
-
-    defp display({name, coverage}, opts) do
-      threshold = Keyword.get(opts, :threshold, @default_threshold)
-      percentage = percentage(coverage)
-
-      Mix.shell().info([
-        color(percentage, threshold),
-        format(percentage, 9),
-        "%",
-        :reset,
-        " | ",
-        format_name(name)
-      ])
-    end
-
-    defp percentage({0, 0}), do: 100.0
-    defp percentage({covered, not_covered}), do: covered / (covered + not_covered) * 100
-
-    defp format(number, length), do: :io_lib.format("~#{length}.2f", [number])
-
-    defp format_name(name) when is_binary(name), do: name
-    defp format_name(mod) when is_atom(mod), do: inspect(mod)
-  end
-
   use Mix.Task
 
   alias Mix.Compilers.Test, as: CT
@@ -135,17 +8,18 @@ defmodule Mix.Tasks.Test do
   @recursive true
   @preferred_cli_env :test
 
-  @moduledoc """
+  @moduledoc ~S"""
   Runs the tests for a project.
 
   This task starts the current application, loads up
-  `test/test_helper.exs` and then requires all files matching the
+  `test/test_helper.exs` and then, requires all files matching the
   `test/**/*_test.exs` pattern in parallel.
 
-  A list of files can be given after the task name in order to select
-  the files to compile:
+  A list of files and/or directories can be given after the task
+  name in order to select the files to run:
 
       mix test test/some/particular/file_test.exs
+      mix test test/some/particular/dir
 
   Tests in umbrella projects can be run from the root by specifying
   the full suite path, including `apps/my_app/test`, in which case
@@ -157,46 +31,153 @@ defmodule Mix.Tasks.Test do
       # To run a given test file on my_app from the umbrella root
       mix test apps/my_app/test/some/particular/file_test.exs
 
+  ## Understanding test results
+
+  When you run your test suite, it prints results as they run with
+  a summary at the end, as seen below:
+
+      $ mix test
+      ...
+
+        1) test greets the world (FooTest)
+           test/foo_test.exs:5
+           Assertion with == failed
+           code:  assert Foo.hello() == :world!
+           left:  :world
+           right: :world!
+           stacktrace:
+             test/foo_test.exs:6: (test)
+
+      ........
+
+      Finished in 0.05 seconds (0.00s async, 0.05s sync)
+      1 doctest, 11 tests, 1 failure
+
+      Randomized with seed 646219
+
+  For each test, the test suite will print a dot. Failed tests
+  are printed immediately in the format described in the next
+  section.
+
+  After all tests run, we print the suite summary. The first
+  line contains the total time spent on the suite, followed
+  by how much time was spent on async tests (defined with
+  `use ExUnit.Case, async: true`) vs sync ones:
+
+      Finished in 0.05 seconds (0.00s async, 0.05s sync)
+
+  Developers want to minimize the time spent on sync tests
+  whenever possible, as sync tests run serially and async
+  tests run concurrently.
+
+  Finally, how many tests we have run, how many of them
+  failed, how many were invalid, etc.
+
+  ### Understanding test failures
+
+  First, it contains the failure counter, followed by the test
+  name and the module the test was defined:
+
+      1) test greets the world (FooTest)
+
+  The next line contains the exact location of the test in the
+  `FILE:LINE` format:
+
+      test/foo_test.exs:5
+
+  If you want to re-run only this test, all you need to do is to
+  copy the line above and past it in front of `mix test`:
+
+      mix test test/foo_test.exs:5
+
+  Then we show the error message, code snippet, and general information
+  about the failed test:
+
+      Assertion with == failed
+      code:  assert Foo.hello() == :world!
+      left:  :world
+      right: :world!
+
+  If your terminal supports coloring (see the  "Coloring" section below),
+  a diff is typically shown between `left` and `right` sides. Finally,
+  we print the stacktrace of the failure:
+
+      stacktrace:
+        test/foo_test.exs:6: (test)
+
   ## Command line options
 
     * `--color` - enables color in the output
+
     * `--cover` - runs coverage tool. See "Coverage" section below
+
     * `--exclude` - excludes tests that match the filter
+
+    * `--export-coverage` - the name of the file to export coverage results to.
+      Only has an effect when used with `--cover`
+
     * `--failed` - runs only tests that failed the last time they ran
+
     * `--force` - forces compilation regardless of modification times
+
     * `--formatter` - sets the formatter module that will print the results.
       Defaults to ExUnit's built-in CLI formatter
+
     * `--include` - includes tests that match the filter
-    * `--listen-on-stdin` - runs tests, and then listens on stdin. Receiving a newline will
-      result in the tests being run again. Very useful when combined with `--stale` and
-      external commands which produce output on stdout upon file system modifications
+
+    * `--listen-on-stdin` - runs tests, and then listens on stdin. It will
+      re-run tests once a newline is received. See the "File system watchers"
+      section below
+
     * `--max-cases` - sets the maximum number of tests running asynchronously. Only tests from
       different modules run in parallel. Defaults to twice the number of cores
+
     * `--max-failures` - the suite stops evaluating tests when this number of test
       failures is reached. It runs all tests if omitted
+
     * `--no-archives-check` - does not check archives
+
     * `--no-color` - disables color in the output
+
     * `--no-compile` - does not compile, even if files require compilation
+
     * `--no-deps-check` - does not check dependencies
+
     * `--no-elixir-version-check` - does not check the Elixir version from `mix.exs`
+
     * `--no-start` - does not start applications after compilation
+
     * `--only` - runs only tests that match the filter
+
     * `--partitions` - sets the amount of partitions to split tests in. This option
       requires the `MIX_TEST_PARTITION` environment variable to be set. See the
       "Operating system process partitioning" section for more information
+
     * `--preload-modules` - preloads all modules defined in applications
+
     * `--raise` - raises if the test suite failed
+
     * `--seed` - seeds the random number generator used to randomize the order of tests;
-      `--seed 0` disables randomization
+      `--seed 0` disables randomization so the tests in a single file will always be ran
+      in the same order they were defined in
+
     * `--slowest` - prints timing information for the N slowest tests.
       Automatically sets `--trace` and `--preload-modules`
+
     * `--stale` - runs only tests which reference modules that changed since the
       last time tests were ran with `--stale`. You can read more about this option
       in the "The --stale option" section below
+
     * `--timeout` - sets the timeout for the tests
+
     * `--trace` - runs tests with detailed reporting. Automatically sets `--max-cases` to `1`.
       Note that in trace mode test timeouts will be ignored as timeout is set to `:infinity`
 
+    * `--warnings-as-errors` - (since v1.12.0) treats warnings as errors and returns a non-zero
+      exit code. This option only applies to test files. To treat warnings as errors during
+      compilation and during tests, run:
+          MIX_ENV=test mix do compile --warnings-as-errors, test --warnings-as-errors
+
   ## Configuration
 
   These configurations can be set in the `def project` section of your `mix.exs`:
@@ -213,6 +194,17 @@ defmodule Mix.Tasks.Test do
     * `:test_coverage` - a set of options to be passed down to the coverage
       mechanism
 
+  ## Coloring
+
+  Coloring is enabled by default on most Unix terminals. They are also
+  available on Windows consoles from Windows 10, although it must be
+  explicitly enabled for the current user in the registry by running
+  the following command:
+
+      reg add HKCU\Console /v VirtualTerminalLevel /t REG_DWORD /d 1
+
+  After running the command above, you must restart your current console.
+
   ## Filters
 
   ExUnit provides tags and filtering functionality that allow developers
@@ -275,10 +267,18 @@ defmodule Mix.Tasks.Test do
     * `:output` - the output directory for cover results. Defaults to `"cover"`
     * `:tool` - the coverage tool
     * `:summary` - summary output configuration; can be either a boolean
-      or a keyword list. When a keyword list is passed, it can specify a `:threshold`,
-      which is a boolean or numeric value that enables coloring of code coverage
-      results in red or green depending on whether the percentage is below or
-      above the specified threshold, respectively. Defaults to `[threshold: 90]`
+      or a keyword list. When a keyword list is passed, it can specify a
+      `:threshold`, which is a boolean or numeric value that enables coloring
+      of code coverage results in red or green depending on whether the
+      percentage is below or above the specified threshold, respectively.
+      Defaults to `[threshold: 90]`
+    * `:export` - a file name to export results to instead of generating
+      the result on the fly. The `.coverdata` extension is automatically
+      added to the given file. This option is automatically set via the
+      `--export-coverage` option or when using process partitioning.
+      See `mix test.coverage` to compile a report from multiple exports.
+    * `:ignore_modules` - modules to ignore from generating reports and
+      in summaries
 
   By default, a very simple wrapper around OTP's `cover` is used as a tool,
   but it can be overridden as follows:
@@ -317,10 +317,16 @@ defmodule Mix.Tasks.Test do
       MIX_TEST_PARTITION=3 mix test --partitions 4
       MIX_TEST_PARTITION=4 mix test --partitions 4
 
-  The test files are sorted and distributed in a round-robin fashion. Note the
-  partition itself is given as an environment variable so it can be accessed in
-  configuration files and test scripts. For example, it can be used to setup a
-  different database instance per partition in `config/test.exs`.
+  The test files are sorted upfront in a round-robin fashion. Note the partition
+  itself is given as an environment variable so it can be accessed in config files
+  and test scripts. For example, it can be used to setup a different database instance
+  per partition in `config/test.exs`.
+
+  If partitioning is enabled and `--cover` is used, no cover reports are generated,
+  as they only contain a subset of the coverage data. Instead, the coverage data
+  is exported to files such as `cover/MIX_TEST_PARTITION.coverdata`. Once you have
+  the results of all partitions inside `cover/`, you can run `mix test.coverage` to
+  get the unified report.
 
   ## The --stale option
 
@@ -336,12 +342,34 @@ defmodule Mix.Tasks.Test do
 
   The `--stale` option is extremely useful for software iteration, allowing you to
   run only the relevant tests as you perform changes to the codebase.
+
+  ## File-system watchers
+
+  You can integrate `mix test` with filesystem watchers through the command line
+  via the `--listen-on-stdin` option. For example, you can use [fswatch](https://github.com/emcrisostomo/fswatch)
+  or similar to emit newlines whenever there is a change, which will cause your test
+  suite to re-run:
+
+      fswatch lib test | mix test --listen-on-stdin
+
+  This can be combined with the `--stale` option to re-run only the test files that
+  have changed as well as the tests that have gone stale due to changes in `lib`.
+
+  ## Aborting the suite
+
+  It is possible to abort the test suite with `Ctrl+\ `, which sends a SIGQUIT
+  signal to the Erlang VM. ExUnit will intercept this signal to show all tests
+  that have been aborted and print the results collected so far.
+
+  This can be useful in case the suite gets stuck and you don't want to wait
+  until the timeout times passes (which defaults to 30 seconds).
   """
 
   @switches [
     force: :boolean,
     color: :boolean,
     cover: :boolean,
+    export_coverage: :string,
     trace: :boolean,
     max_cases: :integer,
     max_failures: :integer,
@@ -362,10 +390,11 @@ defmodule Mix.Tasks.Test do
     formatter: :keep,
     slowest: :integer,
     partitions: :integer,
-    preload_modules: :boolean
+    preload_modules: :boolean,
+    warnings_as_errors: :boolean
   ]
 
-  @cover [output: "cover", tool: Cover]
+  @cover [output: "cover", tool: Mix.Tasks.Test.Coverage]
 
   @impl true
   def run(args) do
@@ -425,10 +454,16 @@ defmodule Mix.Tasks.Test do
       """)
     end
 
-    Mix.Task.run("loadpaths", args)
+    # Load ExUnit before we compile anything
+    Application.ensure_loaded(:ex_unit)
 
-    if Keyword.get(opts, :compile, true) do
-      Mix.Project.compile(args)
+    old_warnings_as_errors = Code.get_compiler_option(:warnings_as_errors)
+    args = args -- ["--warnings-as-errors"]
+
+    Mix.Task.run("compile", args)
+
+    if opts[:warnings_as_errors] do
+      Code.put_compiler_option(:warnings_as_errors, true)
     end
 
     project = Mix.Project.config()
@@ -437,7 +472,13 @@ defmodule Mix.Tasks.Test do
     cover =
       if opts[:cover] do
         compile_path = Mix.Project.compile_path(project)
-        cover = Keyword.merge(@cover, project[:test_coverage] || [])
+        partition = opts[:partitions] && System.get_env("MIX_TEST_PARTITION")
+
+        cover =
+          @cover
+          |> Keyword.put(:export, opts[:export_coverage] || partition)
+          |> Keyword.merge(project[:test_coverage] || [])
+
         cover[:tool].start(compile_path, cover)
       end
 
@@ -448,12 +489,6 @@ defmodule Mix.Tasks.Test do
     app_start_args = if opts[:slowest], do: ["--preload-modules" | args], else: args
     Mix.Task.run("app.start", app_start_args)
 
-    # Ensure ExUnit is loaded.
-    case Application.load(:ex_unit) do
-      :ok -> :ok
-      {:error, {:already_loaded, :ex_unit}} -> :ok
-    end
-
     # The test helper may change the Mix.shell(), so revert it whenever we raise and after suite
     shell = Mix.shell()
 
@@ -478,41 +513,45 @@ defmodule Mix.Tasks.Test do
 
     display_warn_test_pattern(test_files, test_pattern, matched_test_files, warn_test_pattern)
 
-    case CT.require_and_run(matched_test_files, test_paths, opts) do
-      {:ok, %{excluded: excluded, failures: failures, total: total}} ->
-        Mix.shell(shell)
-        cover && cover.()
+    result =
+      case CT.require_and_run(matched_test_files, test_paths, opts) do
+        {:ok, %{excluded: excluded, failures: failures, total: total}} ->
+          Mix.shell(shell)
+          cover && cover.()
 
-        cond do
-          failures > 0 and opts[:raise] ->
-            raise_with_shell(shell, "\"mix test\" failed")
+          cond do
+            failures > 0 and opts[:raise] ->
+              raise_with_shell(shell, "\"mix test\" failed")
 
-          failures > 0 ->
-            System.at_exit(fn _ -> exit({:shutdown, 1}) end)
+            failures > 0 ->
+              System.at_exit(fn _ -> exit({:shutdown, 1}) end)
 
-          excluded == total and Keyword.has_key?(opts, :only) ->
-            message = "The --only option was given to \"mix test\" but no test was executed"
-            raise_or_error_at_exit(shell, message, opts)
+            excluded == total and Keyword.has_key?(opts, :only) ->
+              message = "The --only option was given to \"mix test\" but no test was executed"
+              raise_or_error_at_exit(shell, message, opts)
 
-          true ->
-            :ok
-        end
+            true ->
+              :ok
+          end
 
-      :noop ->
-        cond do
-          opts[:stale] ->
-            Mix.shell().info("No stale tests")
+        :noop ->
+          cond do
+            opts[:stale] ->
+              Mix.shell().info("No stale tests")
 
-          files == [] ->
-            Mix.shell().info("There are no tests to run")
+            files == [] ->
+              Mix.shell().info("There are no tests to run")
 
-          true ->
-            message = "Paths given to \"mix test\" did not match any directory/file: "
-            raise_or_error_at_exit(shell, message <> Enum.join(files, ", "), opts)
-        end
+            true ->
+              message = "Paths given to \"mix test\" did not match any directory/file: "
+              raise_or_error_at_exit(shell, message <> Enum.join(files, ", "), opts)
+          end
 
-        :ok
-    end
+          :ok
+      end
+
+    Code.put_compiler_option(:warnings_as_errors, old_warnings_as_errors)
+    result
   end
 
   defp raise_with_shell(shell, message) do
