@@ -3,11 +3,15 @@ defmodule EscriptTest do
     :ok = Application.start(:escript_test)
   end
 
-  def main([]) do
-    IO.puts(Application.get_env(:foobar, :value, "TEST"))
-  end
-
-  def main([protocol]) do
+  def main(["--protocol", protocol]) do
     IO.puts(Protocol.consolidated?(Module.concat([protocol])))
   end
+
+  def main(["--nesting"]) do
+    IO.inspect(Application.get_env(:foobar, :nesting, "TEST"))
+  end
+
+  def main(_argv) do
+    IO.puts(Application.get_env(:foobar, :value, "TEST"))
+  end
 end
