@@ -13,9 +13,11 @@ defmodule ReleaseTest do
       node: node(),
       protocols_consolidated?: Protocol.consolidated?(Enumerable),
       release_name: System.get_env("RELEASE_NAME"),
+      release_mode: System.get_env("RELEASE_MODE"),
       release_node: System.get_env("RELEASE_NODE"),
       release_root: System.get_env("RELEASE_ROOT"),
       release_vsn: System.get_env("RELEASE_VSN"),
+      release_prog: System.get_env("RELEASE_PROG"),
       root_dir: :code.root_dir() |> to_string(),
       runtime_config: Application.fetch_env(:release_test, :runtime),
       static_config: Application.fetch_env(:release_test, :static),
