@@ -3,6 +3,8 @@ Code.require_file("../test_helper.exs", __DIR__)
 defmodule Mix.CLITest do
   use MixTest.Case
 
+  @moduletag :tmp_dir
+
   test "default task" do
     in_fixture("no_mixfile", fn ->
       File.write!("mix.exs", """
@@ -17,8 +19,35 @@ defmodule Mix.CLITest do
     end)
   end
 
-  test "compiles and invokes simple task from CLI", context do
-    in_tmp(context.test, fn ->
+  @tag :unix
+  test "Mix.raise/2 can set exit code", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
+      File.mkdir_p!("lib")
+
+      File.write!("mix.exs", """
+      defmodule MyProject do
+        use Mix.Project
+
+        def project do
+          [app: :my_project, version: "0.0.1", aliases: aliases()]
+        end
+
+        defp aliases do
+          [
+            custom: &error(&1, exit_code: 99),
+          ]
+        end
+
+        defp error(_args, opts), do: Mix.raise("oops", opts)
+      end
+      """)
+
+      assert {_, 99} = mix_code(~w[custom])
+    end)
+  end
+
+  test "compiles and invokes simple task from CLI", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       File.mkdir_p!("lib")
 
       File.write!("mix.exs", """
@@ -58,46 +87,46 @@ defmodule Mix.CLITest do
       assert contents =~ "This won't appear"
 
       contents = mix(~w[my_hello], [{"MIX_DEBUG", "1"}])
-      assert contents =~ "** Running mix my_hello (inside MyProject)"
+      assert contents =~ "-> Running mix my_hello (inside MyProject)"
       assert contents =~ "** (Mix.Error) oops"
 
       contents = mix(~w[my_hello], [{"MIX_DEBUG", "0"}])
-      refute contents =~ "** Running mix my_hello (inside MyProject)"
+      refute contents =~ "-> Running mix my_hello (inside MyProject)"
       refute contents =~ "** (Mix.Error) oops"
     end)
   end
 
-  test "no task error", context do
-    in_tmp(context.test, fn ->
+  test "no task error", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       contents = mix(~w[no_task])
       assert contents =~ "** (Mix) The task \"no_task\" could not be found"
     end)
   end
 
-  test "tasks with slashes in them raise a NoTaskError right away", context do
-    in_tmp(context.test, fn ->
+  test "tasks with slashes in them raise a NoTaskError right away", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       contents = mix(~w[my/task])
       assert contents =~ "** (Mix) The task \"my/task\" could not be found"
     end)
   end
 
-  test "--help smoke test", context do
-    in_tmp(context.test, fn ->
+  test "--help smoke test", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       output = mix(~w[--help])
       assert output =~ "Mix is a build tool for Elixir"
       assert output =~ "mix help TASK"
     end)
   end
 
-  test "--version smoke test", context do
-    in_tmp(context.test, fn ->
+  test "--version smoke test", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       output = mix(~w[--version])
       assert output =~ ~r/Erlang.+\n\nMix [0-9\.a-z]+/
     end)
   end
 
-  test "env config", context do
-    in_tmp(context.test, fn ->
+  test "env config", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       File.write!("custom.exs", """
       defmodule P do
         use Mix.Project
@@ -116,8 +145,8 @@ defmodule Mix.CLITest do
     System.delete_env("MIX_EXS")
   end
 
-  test "env config defaults to the tasks's preferred cli environment", context do
-    in_tmp(context.test, fn ->
+  test "env config defaults to the tasks's preferred cli environment", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       File.write!("custom.exs", """
       defmodule P do
         use Mix.Project
@@ -143,8 +172,8 @@ defmodule Mix.CLITest do
     System.delete_env("MIX_EXS")
   end
 
-  test "target config defaults to the user's preferred cli target", context do
-    in_tmp(context.test, fn ->
+  test "target config defaults to the user's preferred cli target", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       File.write!("custom.exs", """
       defmodule P do
         use Mix.Project
@@ -169,8 +198,33 @@ defmodule Mix.CLITest do
     System.delete_env("MIX_EXS")
   end
 
-  test "new with tests and cover" do
-    in_tmp("new_with_tests", fn ->
+  test "env config and target use defaults when empty", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
+      File.write!("custom.exs", """
+      defmodule P do
+        use Mix.Project
+        def project, do: [app: :p, version: "0.1.0"]
+      end
+      """)
+
+      System.put_env("MIX_ENV", "")
+      System.put_env("MIX_TARGET", "")
+      System.put_env("MIX_EXS", "custom.exs")
+
+      output =
+        mix(["run", "-e", "IO.inspect {Mix.env(), Mix.target(), System.argv()}", "--", "a", "b"])
+
+      assert output =~ ~s({:dev, :host, ["a", "b"]})
+    end)
+  after
+    System.delete_env("MIX_ENV")
+    System.delete_env("MIX_TARGET")
+    System.delete_env("MIX_EXS")
+  end
+
+  @tag tmp_dir: "new_with_tests"
+  test "new with tests and cover", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       output = mix(~w[new .])
       assert output =~ "* creating lib/new_with_tests.ex"
 
@@ -182,8 +236,9 @@ defmodule Mix.CLITest do
     end)
   end
 
-  test "new --sup with tests" do
-    in_tmp("sup_with_tests", fn ->
+  @tag tmp_dir: "sup_with_tests"
+  test "new --sup with tests", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
       output = mix(~w[new --sup .])
       assert output =~ "* creating lib/sup_with_tests.ex"
 
