@@ -99,19 +99,21 @@ defmodule Mix.RebarTest do
                {:git_rebar, "~> 1.0", override: true, hex: :rebar_fork}
 
       assert parse_dep({:git_rebar, {:pkg, :rebar_fork}}) ==
-               {:git_rebar, ">= 0.0.0", override: true, hex: :rebar_fork}
+               {:git_rebar, override: true, hex: :rebar_fork}
 
-      assert parse_dep({:git_rebar, '0.1..*', {:git, @git_rebar_charlist, :master}}) ==
-               {:git_rebar, ~r"0.1..*", override: true, git: @git_rebar_string, ref: "master"}
+      assert parse_dep({:git_rebar, '0.1..*', {:git, @git_rebar_charlist, :main}}) ==
+               {:git_rebar, ~r"0.1..*", override: true, git: @git_rebar_string, ref: "main"}
 
-      assert parse_dep({:git_rebar, {:git, @git_rebar_charlist, :master}}) ==
-               {:git_rebar, ">= 0.0.0", override: true, git: @git_rebar_string, ref: "master"}
+      assert parse_dep({:git_rebar, {:git, @git_rebar_charlist, :main}}) ==
+               {:git_rebar, override: true, git: @git_rebar_string, ref: "main"}
 
       assert parse_dep({:git_rebar, '0.1..*', {:git, @git_rebar_charlist}, [:raw]}) ==
                {:git_rebar, ~r"0.1..*", override: true, git: @git_rebar_string, compile: false}
 
       assert parse_dep({:git_rebar, '', {:git, @git_rebar_charlist, {:ref, '64691eb'}}}) ==
                {:git_rebar, ~r"", override: true, git: @git_rebar_string, ref: "64691eb"}
+
+      assert parse_dep(:git_rebar) == {:git_rebar, override: true}
     end
   end
 
@@ -199,7 +201,7 @@ defmodule Mix.RebarTest do
       assert Enum.find(deps, fn %Mix.Dep{app: app, opts: opts} ->
                if app == :git_rebar do
                  assert Enum.find(opts, &match?({:git, _}, &1))
-                 assert Enum.find(opts, &match?({:ref, "master"}, &1))
+                 assert Enum.find(opts, &match?({:ref, "main"}, &1))
                  true
                end
              end)
@@ -219,15 +221,15 @@ defmodule Mix.RebarTest do
     end
 
     test "gets and compiles dependencies for Rebar" do
-      Mix.Project.push(RebarAsDep)
+      Mix.Project.push(RebarAsDepWithEnv)
 
       in_tmp("get and compile dependencies for Rebar", fn ->
         Mix.Tasks.Deps.Get.run([])
         assert_received {:mix_shell, :info, ["* Getting git_rebar" <> _]}
 
         Mix.Tasks.Deps.Compile.run([])
-        assert_received {:mix_shell, :run, ["===> Compiling git_rebar\n"]}
-        assert_received {:mix_shell, :run, ["===> Compiling rebar_dep\n"]}
+        assert_received {:mix_shell, :run, ["==> git_rebar (compile)\n"]}
+        assert_received {:mix_shell, :run, ["==> rebar_dep (compile)\n"]}
         assert :git_rebar.any_function() == :ok
         assert :rebar_dep.any_function() == :ok
 
@@ -266,6 +268,9 @@ defmodule Mix.RebarTest do
       end)
     end
 
+    # We run only on Unix because Windows has a hard time
+    # removing the Rebar executable after executed.
+    @tag [unix: true]
     test "gets and compiles dependencies for Rebar3" do
       Mix.Project.push(Rebar3AsDep)
 
