@@ -16,12 +16,9 @@ defmodule Mix.Tasks.CleanTest do
     end
   end
 
-  setup do
-    Mix.Project.push(Sample)
-    :ok
-  end
-
   test "cleans the application build" do
+    Mix.Project.push(Sample)
+
     in_fixture("deps_status", fn ->
       File.mkdir_p!("_build/dev/lib/sample/consolidated")
       File.mkdir_p!("_build/dev/lib/sample")
@@ -37,6 +34,8 @@ defmodule Mix.Tasks.CleanTest do
   end
 
   test "cleans dependencies build" do
+    Mix.Project.push(Sample)
+
     in_fixture("deps_status", fn ->
       File.mkdir_p!("_build/dev/lib/ok")
       File.mkdir_p!("_build/test/lib/ok")
@@ -49,4 +48,36 @@ defmodule Mix.Tasks.CleanTest do
       refute File.exists?("_build/test")
     end)
   end
+
+  test "invokes compiler hook defined in project" do
+    Mix.ProjectStack.post_config(compilers: Mix.compilers() ++ [:testc])
+    Mix.Project.push(MixTest.Case.Sample)
+
+    in_fixture("no_mixfile", fn ->
+      File.write!("lib/testc.ex", """
+      defmodule Mix.Tasks.Compile.Testc do
+        use Mix.Task.Compiler
+
+        @impl true
+        def run(_args) do
+          Mix.shell().info("Compiling...")
+          :ok
+        end
+
+        @impl true
+        def clean do
+          Mix.shell().info("Cleaning...")
+          :ok
+        end
+      end
+      """)
+
+      Mix.Task.run("compile")
+      assert_received {:mix_shell, :info, ["Compiling..."]}
+      purge([Mix.Tasks.Compile.Testc])
+
+      Mix.Task.run("clean")
+      assert_received {:mix_shell, :info, ["Cleaning..."]}
+    end)
+  end
 end
