@@ -13,7 +13,11 @@ defmodule Mix.Tasks.Compile.AppTest do
     end
 
     def application do
-      [maxT: :infinity, applications: [:example_app], extra_applications: [:logger]]
+      [
+        maxT: :infinity,
+        applications: [:example_app, mix: :optional],
+        extra_applications: [:logger, ex_unit: :optional]
+      ]
     end
   end
 
@@ -36,7 +40,9 @@ defmodule Mix.Tasks.Compile.AppTest do
         {:ok6, path: "../ok", optional: true},
         {:ok7, path: "../ok", optional: false},
         {:ok8, path: "../ok", app: false},
-        {:ok9, path: "../ok"}
+        {:ok9, path: "../ok"},
+        {:ok10, path: "../ok", targets: [:will_never_be_listed]},
+        {:ok11, path: "../ok", targets: [Mix.target()]}
       ]
     end
   end
@@ -126,7 +132,7 @@ defmodule Mix.Tasks.Compile.AppTest do
       properties = parse_resource_file(:custom_deps)
 
       assert properties[:applications] ==
-               [:kernel, :stdlib, :elixir, :logger, :ok1, :ok3, :ok4, :ok7]
+               [:kernel, :stdlib, :elixir, :logger, :ok1, :ok3, :ok4, :ok7, :ok11]
     end)
   end
 
@@ -167,7 +173,7 @@ defmodule Mix.Tasks.Compile.AppTest do
       Process.put(:application, extra_applications: ["invalid"])
 
       message =
-        "Application extra applications (:extra_applications) should be a list of atoms, got: [\"invalid\"]"
+        "Application extra applications (:extra_applications) should be a list of atoms or {app, :required | :optional} pairs, got: [\"invalid\"]"
 
       assert_raise Mix.Error, message, fn ->
         Mix.Tasks.Compile.App.run([])
@@ -185,14 +191,16 @@ defmodule Mix.Tasks.Compile.AppTest do
       Process.put(:application, applications: ["invalid"])
 
       message =
-        "Application applications (:applications) should be a list of atoms, got: [\"invalid\"]"
+        "Application applications (:applications) should be a list of atoms or {app, :required | :optional} pairs, got: [\"invalid\"]"
 
       assert_raise Mix.Error, message, fn ->
         Mix.Tasks.Compile.App.run([])
       end
 
       Process.put(:application, applications: nil)
-      message = "Application applications (:applications) should be a list of atoms, got: nil"
+
+      message =
+        "Application applications (:applications) should be a list of atoms or {app, :required | :optional} pairs, got: nil"
 
       assert_raise Mix.Error, message, fn ->
         Mix.Tasks.Compile.App.run([])
