@@ -179,7 +179,7 @@ defmodule Mix.Tasks.DepsTest do
     end)
   end
 
-  test "cleans and recompiles artifacts if --force given" do
+  test "cleans and recompiles artifacts if --force is given" do
     Mix.Project.push(SuccessfulDepsApp)
 
     in_fixture("deps_status", fn ->
@@ -191,11 +191,9 @@ defmodule Mix.Tasks.DepsTest do
     end)
   end
 
-  test "doesn't compile any umbrella apps if --skip-umbrella-children given" do
+  test "doesn't compile any umbrella apps if --skip-umbrella-children is given" do
     in_fixture("umbrella_dep/deps/umbrella", fn ->
       Mix.Project.in_project(:umbrella, ".", fn _ ->
-        refute File.exists?("_build/dev/lib/foo/ebin")
-        refute File.exists?("_build/dev/lib/bar/ebin")
         Mix.Tasks.Deps.Compile.run(["--skip-umbrella-children"])
         refute File.exists?("_build/dev/lib/foo/ebin")
         refute File.exists?("_build/dev/lib/bar/ebin")
@@ -203,6 +201,16 @@ defmodule Mix.Tasks.DepsTest do
     end)
   end
 
+  test "doesn't compile any path deps if --skip-local-deps is given" do
+    Mix.Project.push(SuccessfulDepsApp)
+
+    in_fixture("deps_status", fn ->
+      File.rm_rf!("_build/dev/lib/ok/ebin")
+      Mix.Tasks.Deps.Compile.run(["--skip-local-deps"])
+      refute File.exists?("_build/dev/lib/ok/ebin")
+    end)
+  end
+
   ## deps.loadpaths
 
   test "checks list of dependencies and their status with success" do
@@ -354,6 +362,13 @@ defmodule Mix.Tasks.DepsTest do
       assert Mix.Dep.Lock.read() == %{whatever: "abcdef", ok: "abcdef"}
       Mix.Tasks.Deps.Unlock.run(["--unused"])
       assert Mix.Dep.Lock.read() == %{ok: "abcdef"}
+
+      output = """
+      Unlocked deps:
+      * whatever
+      """
+
+      assert_received {:mix_shell, :info, [^output]}
     end)
   end
 
@@ -366,6 +381,13 @@ defmodule Mix.Tasks.DepsTest do
       assert Mix.Dep.Lock.read() == %{another: "hash"}
       error = "warning: unknown dependency is not locked"
       assert_received {:mix_shell, :error, [^error]}
+
+      output = """
+      Unlocked deps:
+      * git_repo
+      """
+
+      assert_received {:mix_shell, :info, [^output]}
     end)
   end
 
