@@ -9,12 +9,13 @@ defmodule Mix.Tasks.ReleaseTest do
   defmacrop release_node(name), do: :"#{name}@#{@hostname}"
 
   describe "customize" do
-    test "env and vm.args with EEx" do
+    test "env, vm.args and overlays templates" do
       in_fixture("release_test", fn ->
         Mix.Project.in_project(:release_test, ".", fn _ ->
-          File.mkdir_p!("rel")
+          File.mkdir_p!("rel/overlays/empty/directory")
+          File.write!("rel/overlays/hello", "world")
 
-          for file <- ~w(rel/vm.args.eex rel/env.sh.eex rel/env.bat.eex) do
+          for file <- ~w(rel/vm.args.eex rel/remote.vm.args.eex rel/env.sh.eex rel/env.bat.eex) do
             File.write!(file, """
             #{file} FOR <%= @release.name %>
             """)
@@ -32,6 +33,75 @@ defmodule Mix.Tasks.ReleaseTest do
 
           assert root |> Path.join("releases/0.1.0/vm.args") |> File.read!() ==
                    "rel/vm.args.eex FOR release_test\n"
+
+          assert root |> Path.join("releases/0.1.0/remote.vm.args") |> File.read!() ==
+                   "rel/remote.vm.args.eex FOR release_test\n"
+
+          assert root |> Path.join("empty/directory") |> File.dir?()
+          assert root |> Path.join("hello") |> File.read!() == "world"
+        end)
+      end)
+    end
+
+    test "env, vm.args and overlays templates with custom rel_templates_path" do
+      in_fixture("release_test", fn ->
+        config = [
+          releases: [
+            release_test: [rel_templates_path: "custom_rel"]
+          ]
+        ]
+
+        Mix.Project.in_project(:release_test, ".", config, fn _ ->
+          File.mkdir_p!("custom_rel/overlays/empty/directory")
+          File.write!("custom_rel/overlays/hello", "world")
+
+          for file <-
+                ~w(custom_rel/vm.args.eex custom_rel/remote.vm.args.eex custom_rel/env.sh.eex custom_rel/env.bat.eex) do
+            File.write!(file, """
+            #{file} FOR <%= @release.name %>
+            """)
+          end
+
+          root = Path.absname("_build/dev/rel/release_test")
+          Mix.Task.run("release")
+          assert_received {:mix_shell, :info, ["* assembling release_test-0.1.0 on MIX_ENV=dev"]}
+
+          assert root |> Path.join("releases/0.1.0/env.sh") |> File.read!() ==
+                   "custom_rel/env.sh.eex FOR release_test\n"
+
+          assert root |> Path.join("releases/0.1.0/env.bat") |> File.read!() ==
+                   "custom_rel/env.bat.eex FOR release_test\n"
+
+          assert root |> Path.join("releases/0.1.0/vm.args") |> File.read!() ==
+                   "custom_rel/vm.args.eex FOR release_test\n"
+
+          assert root |> Path.join("releases/0.1.0/remote.vm.args") |> File.read!() ==
+                   "custom_rel/remote.vm.args.eex FOR release_test\n"
+
+          assert root |> Path.join("empty/directory") |> File.dir?()
+          assert root |> Path.join("hello") |> File.read!() == "world"
+        end)
+      end)
+    end
+
+    test "custom overlays" do
+      in_fixture("release_test", fn ->
+        config = [releases: [release_test: [overlays: "rel/another"]]]
+
+        Mix.Project.in_project(:release_test, ".", config, fn _ ->
+          File.mkdir_p!("rel/overlays")
+          File.write!("rel/overlays/hello", "world")
+          File.write!("rel/overlays/original", "kept")
+
+          File.mkdir_p!("rel/another/empty/directory")
+          File.write!("rel/another/hello", "override")
+
+          root = Path.absname("_build/dev/rel/release_test")
+          Mix.Task.rerun("release", ["--overwrite"])
+
+          assert root |> Path.join("empty/directory") |> File.dir?()
+          assert root |> Path.join("hello") |> File.read!() == "override"
+          assert root |> Path.join("original") |> File.read!() == "kept"
         end)
       end)
     end
@@ -66,15 +136,28 @@ defmodule Mix.Tasks.ReleaseTest do
       end)
     end
 
-    test "include_executables_for" do
+    test "include_executables_for and strip_beams" do
       in_fixture("release_test", fn ->
-        config = [releases: [release_test: [include_executables_for: []]]]
+        config = [
+          releases: [
+            release_test: [
+              include_executables_for: [],
+              strip_beams: [keep: "Docs"]
+            ]
+          ]
+        ]
 
         Mix.Project.in_project(:release_test, ".", config, fn _ ->
           root = Path.absname("_build/dev/rel/release_test")
           Mix.Task.run("release")
           assert_received {:mix_shell, :info, ["* assembling release_test-0.1.0 on MIX_ENV=dev"]}
 
+          beam =
+            File.read!(Path.join(root, "lib/release_test-0.1.0/ebin/Elixir.ReleaseTest.beam"))
+
+          assert {:ok, {_, [{'Docs', _}]}} = :beam_lib.chunks(beam, ['Docs'])
+          assert {:error, _, _} = :beam_lib.chunks(beam, ['Dbgi'])
+
           refute root |> Path.join("bin/start") |> File.exists?()
           refute root |> Path.join("bin/start.bat") |> File.exists?()
           refute root |> Path.join("releases/0.1.0/elixir") |> File.exists?()
@@ -84,42 +167,6 @@ defmodule Mix.Tasks.ReleaseTest do
         end)
       end)
     end
-
-    test "default overlays" do
-      in_fixture("release_test", fn ->
-        Mix.Project.in_project(:release_test, ".", fn _ ->
-          File.mkdir_p!("rel/overlays/empty/directory")
-          File.write!("rel/overlays/hello", "world")
-
-          root = Path.absname("_build/dev/rel/release_test")
-          Mix.Task.run("release")
-
-          assert root |> Path.join("empty/directory") |> File.dir?()
-          assert root |> Path.join("hello") |> File.read!() == "world"
-        end)
-      end)
-    end
-
-    test "custom overlays" do
-      in_fixture("release_test", fn ->
-        config = [releases: [release_test: [overlays: "rel/another"]]]
-
-        Mix.Project.in_project(:release_test, ".", config, fn _ ->
-          assert_raise Mix.Error, ~r"a string pointing to an existing directory", fn ->
-            Mix.Task.run("release", ["--overwrite"])
-          end
-
-          File.mkdir_p!("rel/another/empty/directory")
-          File.write!("rel/another/hello", "world")
-
-          root = Path.absname("_build/dev/rel/release_test")
-          Mix.Task.rerun("release", ["--overwrite"])
-
-          assert root |> Path.join("empty/directory") |> File.dir?()
-          assert root |> Path.join("hello") |> File.read!() == "world"
-        end)
-      end)
-    end
   end
 
   describe "errors" do
@@ -168,6 +215,7 @@ defmodule Mix.Tasks.ReleaseTest do
           assert "bin/demo" in files
           assert "releases/0.1.0/sys.config" in files
           assert "releases/0.1.0/vm.args" in files
+          assert "releases/0.1.0/remote.vm.args" in files
           assert "releases/COOKIE" in files
           assert "releases/start_erl.data" in files
           assert "hello" in files
@@ -263,6 +311,7 @@ defmodule Mix.Tasks.ReleaseTest do
         assert root |> Path.join("releases/0.1.0/env.sh") |> File.exists?()
         assert root |> Path.join("releases/0.1.0/env.bat") |> File.exists?()
         assert root |> Path.join("releases/0.1.0/vm.args") |> File.exists?()
+        assert root |> Path.join("releases/0.1.0/remote.vm.args") |> File.exists?()
 
         assert root
                |> Path.join("releases/0.1.0/sys.config")
@@ -281,12 +330,13 @@ defmodule Mix.Tasks.ReleaseTest do
         assert %{
                  app_dir: app_dir,
                  cookie_env: ^cookie,
-                 encoding: {:"£", "£", '£'},
+                 encoding: {:time_μs, :"£", "£", '£'},
                  mode: :embedded,
                  node: release_node("release_test"),
                  protocols_consolidated?: true,
                  release_name: "release_test",
                  release_node: "release_test",
+                 release_prog: "release_test" <> ext,
                  release_root: release_root,
                  release_vsn: "0.1.0",
                  root_dir: root_dir,
@@ -303,26 +353,42 @@ defmodule Mix.Tasks.ReleaseTest do
           assert root_dir =~ ~r"_build/dev/rel/(release_test|RELEAS~1)$"
           assert String.ends_with?(sys_config_env, "releases\\0.1.0\\sys")
           assert String.ends_with?(sys_config_init, "releases\\0.1.0\\sys")
+          assert ext == ".bat"
         else
           assert app_dir == Path.join(root, "lib/release_test-0.1.0")
           assert release_root == root
           assert root_dir == root
           assert sys_config_env == Path.join(root, "releases/0.1.0/sys")
           assert sys_config_init == Path.join(root, "releases/0.1.0/sys")
+          assert ext == ""
         end
       end)
     end)
   end
 
-  test "assembles a bootable release with runtime configuration" do
+  test "assembles a rebootable release with runtime configuration" do
     in_fixture("release_test", fn ->
-      config = [releases: [runtime_config: []]]
+      config = [releases: [runtime_config: [reboot_system_after_config: true]]]
 
       Mix.Project.in_project(:release_test, ".", config, fn _ ->
-        File.write!("config/releases.exs", """
+        File.write!("config/config.exs", """
+        #{File.read!("config/config.exs")}
+        config :release_test, :runtime, keep: :static, override: :static
+        """)
+
+        File.write!("config/runtime.exs", """
         import Config
-        config :release_test, :runtime, :was_set
-        config :release_test, :encoding, {:runtime, :"£", "£", '£'}
+
+        if System.get_env("RELEASE_MODE") == nil do
+          raise "file should not be loaded while assembling release"
+        end
+
+        config :release_test, :runtime,
+          override: :runtime,
+          config_env: config_env(),
+          config_target: config_target()
+
+        config :release_test, :encoding, {:runtime, :time_μs, :"£", "£", '£'}
         """)
 
         root = Path.absname("_build/dev/rel/runtime_config")
@@ -332,7 +398,7 @@ defmodule Mix.Tasks.ReleaseTest do
         assert_received {:mix_shell, :info, ["* assembling runtime_config-0.1.0 on MIX_ENV=dev"]}
 
         assert_received {:mix_shell, :info,
-                         ["* using config/releases.exs to configure the release at runtime"]}
+                         ["* using config/runtime.exs to configure the release at runtime"]}
 
         # Assert structure
         assert root
@@ -348,14 +414,18 @@ defmodule Mix.Tasks.ReleaseTest do
         open_port(Path.join(root, "bin/runtime_config"), ['start'])
 
         assert %{
-                 encoding: {:runtime, :"£", "£", '£'},
+                 encoding: {:runtime, :time_μs, :"£", "£", '£'},
                  mode: :embedded,
                  node: release_node("runtime_config"),
                  protocols_consolidated?: true,
                  release_name: "runtime_config",
+                 release_mode: "embedded",
                  release_node: "runtime_config",
+                 release_prog: "runtime_config" <> ext,
                  release_vsn: "0.1.0",
-                 runtime_config: {:ok, :was_set},
+                 runtime_config:
+                   {:ok,
+                    [keep: :static, override: :runtime, config_env: :dev, config_target: :host]},
                  static_config: {:ok, :was_set},
                  sys_config_env: sys_config_env,
                  sys_config_init: sys_config_init
@@ -364,9 +434,11 @@ defmodule Mix.Tasks.ReleaseTest do
         if match?({:win32, _}, :os.type()) do
           assert sys_config_env =~ "tmp\\runtime_config-0.1.0"
           assert sys_config_init =~ "tmp\\runtime_config-0.1.0"
+          assert ext == ".bat"
         else
           assert sys_config_env =~ "tmp/runtime_config-0.1.0"
           assert sys_config_init =~ "tmp/runtime_config-0.1.0"
+          assert ext == ""
         end
       end)
     end)
@@ -414,6 +486,7 @@ defmodule Mix.Tasks.ReleaseTest do
         assert root |> Path.join("releases/0.2.0/demo.rel") |> File.exists?()
         assert root |> Path.join("releases/0.2.0/sys.config") |> File.exists?()
         assert root |> Path.join("releases/0.2.0/vm.args") |> File.exists?()
+        assert root |> Path.join("releases/0.2.0/remote.vm.args") |> File.exists?()
 
         # Assert runtime
         open_port(Path.join(root, "bin/demo"), ['start'])
@@ -426,6 +499,7 @@ defmodule Mix.Tasks.ReleaseTest do
                  protocols_consolidated?: true,
                  release_name: "demo",
                  release_node: "demo",
+                 release_prog: "demo" <> ext,
                  release_root: release_root,
                  release_vsn: "0.2.0",
                  root_dir: root_dir,
@@ -436,9 +510,11 @@ defmodule Mix.Tasks.ReleaseTest do
         if match?({:win32, _}, :os.type()) do
           assert String.ends_with?(app_dir, "demo/lib/release_test-0.1.0")
           assert String.ends_with?(release_root, "demo")
+          assert ext == ".bat"
         else
           assert app_dir == Path.join(root, "lib/release_test-0.1.0")
           assert release_root == root
+          assert ext == ""
         end
 
         assert root_dir == :code.root_dir() |> to_string()
@@ -448,20 +524,24 @@ defmodule Mix.Tasks.ReleaseTest do
 
   test "validates compile_env" do
     in_fixture("release_test", fn ->
-      config = [releases: [compile_env_config: []]]
+      # Let's also use a non standard config_path.
+      config = [releases: [compile_env_config: []], config_path: "different_config/config.exs"]
 
       File.write!("lib/compile_env.ex", """
       _ = Application.compile_env(:release_test, :static)
       """)
 
+      File.mkdir_p!("different_config")
+      File.cp!("config/config.exs", "different_config/config.exs")
+
+      File.write!("different_config/runtime.exs", """
+      import Config
+      config :release_test, :static, String.to_atom(System.get_env("RELEASE_STATIC"))
+      """)
+
       Mix.Project.in_project(:release_test, ".", config, fn _ ->
         Mix.Task.run("loadconfig", [])
 
-        File.write!("config/releases.exs", """
-        import Config
-        config :release_test, :static, String.to_atom(System.get_env("RELEASE_STATIC"))
-        """)
-
         root = Path.absname("_build/dev/rel/compile_env_config")
         Mix.Task.run("release", ["compile_env_config"])
 
@@ -493,7 +573,7 @@ defmodule Mix.Tasks.ReleaseTest do
       Mix.Project.in_project(:release_test, ".", config, fn _ ->
         Mix.Task.run("loadconfig", [])
 
-        File.write!("config/releases.exs", """
+        File.write!("config/runtime.exs", """
         import Config
         config :release_test, :static, String.to_atom(System.get_env("RELEASE_STATIC"))
         """)
@@ -522,7 +602,8 @@ defmodule Mix.Tasks.ReleaseTest do
       Mix.Project.in_project(:release_test, ".", config, fn _ ->
         Mix.Task.run("loadconfig", [])
 
-        File.write!("config/releases.exs", """
+        # For compile env to be validated, we need a config/runtime.exs
+        File.write!("config/runtime.exs", """
         import Config
         """)
 
@@ -551,7 +632,7 @@ defmodule Mix.Tasks.ReleaseTest do
       config = [releases: [eval: [include_erts: false, cookie: "abcdefghij"]]]
 
       Mix.Project.in_project(:release_test, ".", config, fn _ ->
-        File.write!("config/releases.exs", """
+        File.write!("config/runtime.exs", """
         import Config
         config :release_test, :runtime, :was_set
         """)
@@ -577,11 +658,20 @@ defmodule Mix.Tasks.ReleaseTest do
                  protocols_consolidated?: true,
                  release_name: "eval",
                  release_node: "eval",
-                 release_root: root,
+                 release_prog: "eval" <> ext,
+                 release_root: release_root,
                  release_vsn: "0.1.0",
                  runtime_config: {:ok, :was_set},
                  static_config: {:ok, :was_set}
                } = wait_until_decoded(Path.join(root, "RELEASE_BOOTED"))
+
+        if match?({:win32, _}, :os.type()) do
+          assert String.ends_with?(release_root, "eval")
+          assert ext == ".bat"
+        else
+          assert release_root == root
+          assert ext == ""
+        end
       end)
     end)
   end
