@@ -23,7 +23,7 @@ defmodule Mix.Tasks.TestTest do
       assert ex_unit_opts_from_given(only: "focus", include: "special") == only
     end
 
-    test "translates :color into list containing an enabled key/value pair" do
+    test "translates :color into list containing an enabled key-value pair" do
       assert ex_unit_opts_from_given(color: false) == [colors: [enabled: false]]
       assert ex_unit_opts_from_given(color: true) == [colors: [enabled: true]]
     end
@@ -133,18 +133,22 @@ defmodule Mix.Tasks.TestTest do
       in_fixture("umbrella_test", fn ->
         output = mix(["test", "--cover"])
 
+        assert output =~ "3 tests, 0 failures"
+
         # For bar, we do regular --cover and also test protocols
         assert output =~ """
                Generating cover results ...
 
                Percentage | Module
                -----------|--------------------------
-                  100.00% | Bar
+                  100.00% | Bar.Protocol
                   100.00% | Bar.Protocol.BitString
                -----------|--------------------------
                   100.00% | Total
                """
 
+        assert output =~ "1 test, 0 failures"
+
         # For foo, we do regular --cover and test it does not include bar
         assert output =~ """
                Generating cover results ...
@@ -157,6 +161,27 @@ defmodule Mix.Tasks.TestTest do
                """
       end)
     end
+
+    test "supports unified reports by using test.coverage" do
+      in_fixture("umbrella_test", fn ->
+        assert mix(["test", "--export-coverage", "default", "--cover"]) =~
+                 "Run \"mix test.coverage\" once all exports complete"
+
+        assert mix(["test.coverage"]) =~ """
+               Importing cover results: apps/bar/cover/default.coverdata
+               Importing cover results: apps/foo/cover/default.coverdata
+
+               Percentage | Module
+               -----------|--------------------------
+                  100.00% | Bar
+                  100.00% | Bar.Protocol
+                  100.00% | Bar.Protocol.BitString
+                  100.00% | Foo
+               -----------|--------------------------
+                  100.00% | Total
+               """
+      end)
+    end
   end
 
   describe "--failed" do
@@ -271,14 +296,32 @@ defmodule Mix.Tasks.TestTest do
   describe "--partitions" do
     test "splits tests into partitions" do
       in_fixture("test_stale", fn ->
-        assert mix(["test", "--partitions", "3"], [{"MIX_TEST_PARTITION", "1"}]) =~
+        assert mix(["test", "--partitions", "3", "--cover"], [{"MIX_TEST_PARTITION", "1"}]) =~
                  "1 test, 0 failures"
 
-        assert mix(["test", "--partitions", "3"], [{"MIX_TEST_PARTITION", "2"}]) =~
+        assert mix(["test", "--partitions", "3", "--cover"], [{"MIX_TEST_PARTITION", "2"}]) =~
                  "1 test, 0 failures"
 
-        assert mix(["test", "--partitions", "3"], [{"MIX_TEST_PARTITION", "3"}]) =~
+        assert mix(["test", "--partitions", "3", "--cover"], [{"MIX_TEST_PARTITION", "3"}]) =~
                  "There are no tests to run"
+
+        assert File.regular?("cover/1.coverdata")
+        assert File.regular?("cover/2.coverdata")
+        refute File.regular?("cover/3.coverdata")
+
+        assert mix(["test.coverage"]) == """
+               Importing cover results: cover/1.coverdata
+               Importing cover results: cover/2.coverdata
+
+               Percentage | Module
+               -----------|--------------------------
+                  100.00% | A
+                  100.00% | B
+               -----------|--------------------------
+                  100.00% | Total
+
+               Generated HTML coverage results in \"cover\" directory
+               """
       end)
     end
 
@@ -347,7 +390,7 @@ defmodule Mix.Tasks.TestTest do
 
         assert output =~ """
                ==> bar
-               .
+               ...
                """
 
         refute output =~ "==> foo"
@@ -369,6 +412,31 @@ defmodule Mix.Tasks.TestTest do
     end
   end
 
+  describe "--warnings-as-errors" do
+    test "fail on warning in tests" do
+      in_fixture("test_stale", fn ->
+        File.write!("lib/warning.ex", """
+        unused_compile_var = 1
+        """)
+
+        File.write!("test/warning_test.exs", """
+        defmodule WarningTest do
+          use ExUnit.Case
+
+          test "warning" do
+            unused_test_var = 1
+          end
+        end
+        """)
+
+        output = mix(["test", "--warnings-as-errors", "test/warning_test.exs"])
+        assert output =~ "variable \"unused_compile_var\" is unused"
+        assert output =~ "variable \"unused_test_var\" is unused"
+        assert output =~ "Compilation failed due to warnings"
+      end)
+    end
+  end
+
   defp receive_until_match(port, expected, acc) do
     receive do
       {^port, {:data, output}} ->
